@@ -34,47 +34,63 @@ DialogPreferences::DialogPreferences(	BaseObjectType* cobject,
 	:	DialogEvent( cobject, refbuilder )
 {
 	Lifeobase::builder->get_widget( "hbox_max_idle_time", m_hbox_max_idle_time );
-	Lifeobase::builder->get_widget( "checkbutton_autologout", m_checkbox_autologout );
+	Lifeobase::builder->get_widget( "checkbutton_autologout", m_chkbtn_autologout );
 	Lifeobase::builder->get_widget( "spinbutton_max_idle_time", m_spinbutton_max_idle_time );
-	Lifeobase::builder->get_widget( "checkbutton_bg_image", m_checkbox_bg_image );
+	Lifeobase::builder->get_widget( "checkbutton_bg_image", m_chkbtn_bg_image );
 	Lifeobase::builder->get_widget( "filebutton_bg_image", m_filebutton_bg_image );
+	Lifeobase::builder->get_widget( "chkbtn_formatting_toolbar", m_chkbtn_formatting_toolbar );
 
-	m_checkbox_autologout->set_active( Lifeobase::m_settings.autologout );
+	m_chkbtn_autologout->set_active( Lifeobase::m_settings.autologout );
 	m_hbox_max_idle_time->set_sensitive( Lifeobase::m_settings.autologout );
 	m_spinbutton_max_idle_time->set_value( Lifeobase::m_settings.idletime );
+
 	if( Lifeobase::m_settings.background == "none" )
 	{
-		m_checkbox_bg_image->set_active( false );
+		m_chkbtn_bg_image->set_active( false );
 		m_filebutton_bg_image->set_sensitive( false );
 	}
 	else
 	{
-		m_checkbox_bg_image->set_active( true );
+		m_chkbtn_bg_image->set_active( true );
 		m_filebutton_bg_image->set_filename( Lifeobase::m_settings.background );
 	}
 
-	m_checkbox_autologout->signal_toggled().connect(
-			sigc::mem_fun( *this, &DialogPreferences::handle_checkbox_autologout_toggled ) );
-	m_checkbox_bg_image->signal_toggled().connect(
-			sigc::mem_fun( *this, &DialogPreferences::handle_checkbox_autologout_toggled ) );
+	m_chkbtn_formatting_toolbar->set_active( Lifeobase::m_settings.show_formatting_toolbar );
+
+	m_chkbtn_autologout->signal_toggled().connect(
+			sigc::mem_fun( *this, &DialogPreferences::handle_chkbtn_autologout_toggled ) );
+	m_chkbtn_bg_image->signal_toggled().connect(
+			sigc::mem_fun( *this, &DialogPreferences::handle_chkbtn_autologout_toggled ) );
+
+	m_chkbtn_formatting_toolbar->signal_toggled().connect(
+			sigc::mem_fun( *this, &DialogPreferences::handle_chkbtn_ftoolbar_toggled ) );
+
 	signal_response().connect(
 			sigc::mem_fun( *this, &DialogPreferences::handle_response ) );
 }
 
 void
-DialogPreferences::handle_checkbox_autologout_toggled( void )
+DialogPreferences::handle_chkbtn_autologout_toggled( void )
 {
-	m_hbox_max_idle_time->set_sensitive( m_checkbox_autologout->get_active() );
-	m_filebutton_bg_image->set_sensitive( m_checkbox_bg_image->get_active() );
+	m_hbox_max_idle_time->set_sensitive( m_chkbtn_autologout->get_active() );
+	m_filebutton_bg_image->set_sensitive( m_chkbtn_bg_image->get_active() );
+}
+
+void
+DialogPreferences::handle_chkbtn_ftoolbar_toggled( void )
+{
+	Lifeobase::m_settings.show_formatting_toolbar =
+			m_chkbtn_formatting_toolbar->get_active();
+	Lifeobase::base->handle_settings_changed();
 }
 
 void
 DialogPreferences::handle_response( int )
 {
 	// APPLY CHANGES
-	Lifeobase::m_settings.autologout = m_checkbox_autologout->get_active();
+	Lifeobase::m_settings.autologout = m_chkbtn_autologout->get_active();
 	Lifeobase::m_settings.idletime = m_spinbutton_max_idle_time->get_value_as_int();
-	if( m_checkbox_bg_image->get_active() )
+	if( m_chkbtn_bg_image->get_active() )
 	{
 		if( Lifeobase::m_settings.background != m_filebutton_bg_image->get_filename() )
 		{
