@@ -1,6 +1,6 @@
 /***********************************************************************************
 
-	Copyright (C) 2007-2010 Ahmet Öztürk (aoz_2@yahoo.com)
+	Copyright (C) 2007-2011 Ahmet Öztürk (aoz_2@yahoo.com)
 
 	This file is part of Lifeograph.
 
@@ -43,26 +43,66 @@ using namespace LIFEO;
 Diary				*Diary::d;
 Diary::Shower		*Diary::shower;
 
+static const char	DEFAULT_CHAPTER_CTG_NAME[]	= N_( "Default" );
+
+// PARSING HELPERS
+Date
+get_db_line_date( const Glib::ustring &line )
+{
+	Date date( 0 );
+
+	for( unsigned int i = 2;
+		 i < line.size() && i < 12 && int ( line[ i ] ) >= '0' && int ( line[ i ] ) <= '9';
+		 i++ )
+	{
+		date.m_date = ( date.m_date * 10 ) + int ( line[ i ] ) - '0';
+	}
+
+	return( date );
+}
+
+Glib::ustring
+get_db_line_name( const Glib::ustring &line )
+{
+	Glib::ustring::size_type begin( line.find( '\t' ) );
+	if( begin == Glib::ustring::npos )
+		begin = 2;
+	else
+		begin++;
+
+	return( line.substr( begin ) );
+}
+
+// DIARY =======================================================================
 Diary::Diary( const std::string &path )
-:	m_path( path ), m_passphrase( "" ),
-	m_ptr2chapter_ctg_cur( NULL ), m_default_theme( &Theme::system_theme ),
-	m_option_spellcheck( false ), m_option_sorting_criteria( SC_DATE ),
+:	DiaryElement( this ), m_path( path ), m_passphrase( "" ),
+	m_ptr2chapter_ctg_cur( NULL ), m_topics( this, "topics" ),
+	m_default_theme( ThemeSystem::get() ), m_startup_elem( HOME_CURRENT_ELEM ),
+	m_last_elem( DEID_MIN ),
+	m_option_spellcheck( false ), m_option_sorting_criteria( SC_DATE ), m_read_version( 0 ),
 	m_filter_text( "" ), m_filter_tag( NULL ), m_filtering_status( FS_CLEAR )
 {
-	// "touch" new lock
+	// "touch" new lock:
 	std::string path_lock = m_path + LOCK_SUFFIX;
 	FILE *fp = fopen( path_lock.c_str(), "a+" );
 	if( fp )
 		fclose( fp );
+
+	// add system theme to the pool where it will remain till the diary is destroyed:
+	m_themes.insert( PoolThemes::value_type( m_default_theme->get_name(), m_default_theme ) );
 }
 
 // derived classes should use this
 Diary::Diary( void )
-:	m_path( "" ), m_passphrase( "" ),
-	m_ptr2chapter_ctg_cur( NULL ), m_default_theme( &Theme::system_theme ),
-	m_option_spellcheck( false ), m_option_sorting_criteria( SC_DATE ),
+:	DiaryElement( this ), m_path( "" ), m_passphrase( "" ),
+	m_ptr2chapter_ctg_cur( NULL ), m_topics( this, "topics" ),
+	m_default_theme( ThemeSystem::get() ), m_startup_elem( HOME_CURRENT_ELEM ),
+	m_last_elem( DEID_MIN ),
+	m_option_spellcheck( false ), m_option_sorting_criteria( SC_DATE ), m_read_version( 0 ),
 	m_filter_text( "" ), m_filter_tag( NULL ), m_filtering_status( FS_CLEAR )
 {
+	// add system theme to the pool where it will remain till the diary is destroyed:
+	m_themes.insert( PoolThemes::value_type( m_default_theme->get_name(), m_default_theme ) );
 }
 
 Diary::~Diary( void )
@@ -74,6 +114,52 @@ Diary::~Diary( void )
 	}
 }
 
+void
+Diary::init_new( const std::string &path )
+{
+	// this function must only be called on a reset diary
+	set_path( path );
+	add_today();
+	// every diary must at least have one chapter category:
+	m_ptr2chapter_ctg_cur = create_chapter_ctg( _( DEFAULT_CHAPTER_CTG_NAME ) );
+}
+
+void
+Diary::clear( void )
+{
+	if( ! m_path.empty() )
+	{
+		std::string path_lock = m_path + LOCK_SUFFIX;
+		remove( path_lock.c_str() );
+		m_path.clear();
+	}
+	m_read_version = 0;
+	DiaryBase::clear();			// clear IDs
+	create_new_id( this );		// add DEID_MIN back to IDs pool
+	create_new_id( &m_topics );	// FIXME: do topics really need a deid?
+	m_entries.clear();
+	m_tags.clear();
+	m_tag_categories.clear();
+	m_chapter_categories.clear();
+	m_topics.clear();
+	m_themes.clear();
+
+	m_ptr2chapter_ctg_cur = NULL;
+	m_default_theme = ThemeSystem::get();
+	m_startup_elem = HOME_CURRENT_ELEM;
+	m_last_elem = DEID_MIN;
+
+	m_passphrase.clear();
+
+	m_filter_text.clear();
+	m_filter_tag = NULL;
+	m_filtering_status = FS_CLEAR;
+
+	// NOTE: only reset body options here:
+	m_option_spellcheck = true;
+	m_option_sorting_criteria = SC_DATE;
+}
+
 Glib::RefPtr< Gdk::Pixbuf >&
 Diary::get_icon( void ) const
 {
@@ -174,17 +260,14 @@ Diary::read_header()
 		switch( line[ 0 ] )
 		{
 			case 'V':
-			{
-				int version = convert_string( line.substr( 2 ) );
-				if( version < LIFEO::DB_FILE_VERSION_INT_MIN ||
-					version > LIFEO::DB_FILE_VERSION_INT )
+				m_read_version = convert_string( line.substr( 2 ) );
+				if( m_read_version < LIFEO::DB_FILE_VERSION_INT_MIN ||
+					m_read_version > LIFEO::DB_FILE_VERSION_INT )
 				{
 					file.close();
 					return LIFEO::INCOMPATIBLE_FILE;
 				}
-				m_flag_v56 = ( version == 56 );
 				break;
-			}
 			case 'E':
 				if( line[ 2 ] != 'n' )
 					// passphrase is set to a dummy value to indicate that diary
@@ -234,9 +317,9 @@ Diary::read_plain()
 
 	if( readsize <= 0 )
 	{
-		print_info( "empty diary" );
+		print_info( "diary has no body" );
 		file.close();
-		return LIFEO::EMPTY_DATABASE;
+		return LIFEO::CORRUPT_FILE;
 	}
 
 	file.seekg( BodyPosition );
@@ -251,7 +334,7 @@ Diary::read_plain()
 	file.close();
 
 	std::stringstream stream( readbuffer );
-	LIFEO::Result result = m_flag_v56 ? parse_db_body_text_56( stream ) :
+	LIFEO::Result result = m_read_version == 56 ? parse_db_body_text_56( stream ) :
 										parse_db_body_text( stream );
 	delete[] readbuffer;
 
@@ -294,7 +377,7 @@ Diary::read_encrypted()
 		if( size <= 3 )
 		{
 			file.close();
-			return LIFEO::EMPTY_DATABASE;
+			return LIFEO::CORRUPT_FILE;
 		}
 		buffer = new unsigned char[ size + 1 ];
 		// TODO(<0.4.0): ex handling here!!!
@@ -322,7 +405,7 @@ Diary::read_encrypted()
 	buffer -= 2;    // restore pointer to the start of the buffer before deletion
 	delete[] buffer;
 
-	return( m_flag_v56 ? parse_db_body_text_56( stream ) :
+	return( m_read_version == 56 ? parse_db_body_text_56( stream ) :
 						 parse_db_body_text( stream ) );
 }
 
@@ -335,12 +418,18 @@ Diary::parse_db_body_text( std::stringstream &stream )
 	Chapter				*ptr2chapter = NULL;
 	CategoryTags		*ptr2tag_ctg = NULL;
 	Theme				*ptr2theme = NULL;
+	bool				flag_first_paragraph( false );
 
 	// TAG DEFINITIONS & CHAPTERS
 	while( getline( stream, line ) )
 	{
 		if( line[ 0 ] == 0 )	// end of section
+		{
+			// every diary must at least have one chapter category:
+			if( m_chapter_categories.empty() )
+				m_ptr2chapter_ctg_cur = create_chapter_ctg( _( DEFAULT_CHAPTER_CTG_NAME ) );
 			break;
+		}
 		else
 		if( line.size() < 3 )
 			continue;
@@ -348,34 +437,40 @@ Diary::parse_db_body_text( std::stringstream &stream )
 		{
 			switch( line[ 0 ] )
 			{
-				case 'O':	// options
-					if( line.size() < 4 )
-						break;
-					m_option_spellcheck = ( line[ 2 ] == 's' );
-					m_option_sorting_criteria = line[ 3 ];
-					break;
+				case 'I':
+					set_force_id( LIFEO::convert_string( line.substr( 2 ) ) );
+				break;
 				case 'T':	// tag category
 					ptr2tag_ctg = create_tag_ctg( line.substr( 2 ) );
 					ptr2tag_ctg->set_expanded( line[ 1 ] == 'e' );
 					break;
 				case 't':	// tag
-					m_tags.create( line.substr( 2 ), ptr2tag_ctg );
+					create_tag( line.substr( 2 ), ptr2tag_ctg );
 					break;
 				case 'C':	// chapter category
 					ptr2chapter_ctg = create_chapter_ctg( line.substr( 2 ) );
-					if( line[ 1 ] == 'c' )
+					if( line[ 1 ] == 'c' || m_ptr2chapter_ctg_cur == NULL )	// for v73
 						m_ptr2chapter_ctg_cur = ptr2chapter_ctg;
 					break;
+				case 'o':	// ordinal chapter (topic)
+					ptr2chapter = m_topics.create_chapter(
+							get_db_line_name( line ), get_db_line_date( line ) );
+					ptr2chapter->set_expanded( line[ 1 ] == 'e' );
+					break;
 				case 'c':	// chapter
 					if( ptr2chapter_ctg == NULL )
 					{
 						print_error( "No chapter category defined" );
 						break;
 					}
-					ptr2chapter = ptr2chapter_ctg->add_chapter( line.substr( 2 ) );
-					ptr2chapter->set_expanded( line[ 1 ] == 'e' );
+						if( m_read_version < 74 )
+							ptr2chapter = new Chapter( this, line.substr( 2 ) );
+						else
+							ptr2chapter = ptr2chapter_ctg->create_chapter(
+								get_db_line_name( line ), get_db_line_date( line ) );
+						ptr2chapter->set_expanded( line[ 1 ] == 'e' );
 					break;
-				case 'd':	// chapter begin date
+				case 'd':	// chapter begin date (v73)
 					if( ptr2chapter != NULL )	// chapter begin date
 					{
 						Date date( LIFEO::convert_string( line.substr( 2 ) ) );
@@ -415,9 +510,21 @@ Diary::parse_db_body_text( std::stringstream &stream )
 							break;
 					}
 					break;
+				case 'S':	// startup action
+					m_startup_elem = LIFEO::convert_string( line.substr( 2 ) );
+					break;
+				case 'O':	// options
+					if( line.size() < 4 )
+						break;
+					m_option_spellcheck = ( line[ 2 ] == 's' );
+					m_option_sorting_criteria = line[ 3 ];
+					break;
+				case 'L':
+					m_last_elem = LIFEO::convert_string( line.substr( 2 ) );
+					break;
 				default:
 					print_error( "unrecognized line:\n" + line );
-					return LIFEO::FAILURE;
+					return LIFEO::CORRUPT_FILE;
 			}
 		}
 	}
@@ -430,14 +537,18 @@ Diary::parse_db_body_text( std::stringstream &stream )
 
 		switch( line[ 0 ] )
 		{
+			case 'I':
+				set_force_id( LIFEO::convert_string( line.substr( 2 ) ) );
+				break;
 			case 'E':	// new entry
-				entry_new = new Entry( Date( LIFEO::convert_string( line.substr( 2 ) ) ),
+				entry_new = new Entry( this, Date( LIFEO::convert_string( line.substr( 2 ) ) ),
 									   line[ 1 ] == 'f' );
-				// chapters
-				place_entry_in_chapters( entry_new );
+				if( m_read_version < 74 )
+					entry_new->m_date.m_date++;	// entry orders should start from 1
 				// TODO: per entry spellcheck options
 				//entry_new->set_spellcheck( opt_spell );
 				m_entries[ entry_new->m_date.m_date ] = entry_new;
+				flag_first_paragraph = true;
 				break;
 			case 'D':	// creation & change dates (optional)
 				if( entry_new == NULL )
@@ -480,23 +591,40 @@ Diary::parse_db_body_text( std::stringstream &stream )
 					print_error( "No entry declared" );
 					break;
 				}
-				if( ! entry_new->m_text.empty() )
+				if( flag_first_paragraph )
+				{
+					if( line.size() > 2 )
+						entry_new->m_text = line.substr( 2 );
+					entry_new->m_name = entry_new->m_text;
+					flag_first_paragraph = false;
+				}
+				else
+				{
 					entry_new->m_text += "\n";
-				entry_new->m_text += line.substr( 2 );
+					entry_new->m_text += line.substr( 2 );
+				}
 				break;
 			default:
 				print_error( "unrecognized line:\n" + line );
-				return LIFEO::FAILURE;
+				return LIFEO::CORRUPT_FILE;
 				break;
 		}
 	}
 
+	if( m_startup_elem > HOME_FIXED_ELEM )
+		if( get_element( m_startup_elem ) == NULL )
+		{
+			print_error( "startup element cannot be found in db" );
+			m_startup_elem = DEID_MIN;
+		}
+
 	if( m_entries.size() )
 	{
 		return LIFEO::SUCCESS;
 	}
 	else
 	{
+		add_today();
 		print_info( "empty diary" );
 		return LIFEO::EMPTY_DATABASE;
 	}
@@ -519,7 +647,12 @@ Diary::parse_db_body_text_56( std::stringstream &stream )
 	while( getline( stream, line ) )
 	{
 		if( line[ 0 ] == 0 )
+		{
+			// every diary must at least have one chapter category:
+			if( m_chapter_categories.empty() )
+				m_ptr2chapter_ctg_cur = create_chapter_ctg( _( DEFAULT_CHAPTER_CTG_NAME ) );
 			break;
+		}
 		else
 		if( line.size() < 3 )
 			continue;
@@ -530,7 +663,7 @@ Diary::parse_db_body_text_56( std::stringstream &stream )
 				case 'T':	// tag
 					try
 					{
-						Tag *tmp_tag = m_tags.create( line.substr( 2 ) );
+						Tag *tmp_tag = create_tag( line.substr( 2 ) );
 						tmp_vector_tags.push_back( tmp_tag );
 						// TODO: this is a temporary solution. will be fixed in new diary version
 					}
@@ -559,8 +692,7 @@ Diary::parse_db_body_text_56( std::stringstream &stream )
 					{
 						try
 						{
-							ptr2chapter = ptr2chapter_ctg->add_chapter(
-									line.substr( 2 ) );
+							ptr2chapter = new Chapter( this, line.substr( 2 ) );
 						}
 						catch( std::exception &ex )
 						{
@@ -571,14 +703,14 @@ Diary::parse_db_body_text_56( std::stringstream &stream )
 				case 'd':	// begin date
 					if( ptr2chapter != NULL )	// chapter begin date
 					{
-						ptr2chapter_ctg->set_chapter_date( ptr2chapter, line.substr( 2 ) );
+						ptr2chapter_ctg->set_chapter_date( ptr2chapter, Date( line.substr( 2 ) ) );
 					}
 					break;
 				case 'O':	// options
 					m_option_spellcheck = ( line[2] == 's' );
 					break;
 				case 'M':
-					if( m_default_theme == &Theme::system_theme )
+					if( m_default_theme == ThemeSystem::get() )
 					{
 						m_default_theme = create_theme( _( "theme" ) );
 					}
@@ -607,7 +739,7 @@ Diary::parse_db_body_text_56( std::stringstream &stream )
 					break;
 				default:
 					print_error( "unrecognized line:\n" + line );
-					return LIFEO::FAILURE;
+					return LIFEO::CORRUPT_FILE;
 			}
 		}
 	}
@@ -620,10 +752,10 @@ Diary::parse_db_body_text_56( std::stringstream &stream )
 			case 'A':	// date
 			{	// FIXME: temporary (for compatibility)
 				Date::date_t date_new = LIFEO::convert_string( line.substr( 2 ) );
-				if( date_new == ( date.m_date >> 10 ) )
+				if( date_new == ( ( date.m_date >> 10 ) + 1 ) )
 					date.m_date++;
 				else
-					date.m_date = ( date_new << 10 );
+					date.m_date = ( date_new << 10 ) + 1;
 				break;
 			}
 			case 'C':    // content
@@ -643,7 +775,7 @@ Diary::parse_db_body_text_56( std::stringstream &stream )
 				// erase redundant new line:
 				content.erase( content.size() - 1 );
 
-				entry_new = new Entry( date, content, flag_favorite );
+				entry_new = new Entry( this, date, content, flag_favorite );
 
 				// tags
 				std::vector< Tag* >::const_iterator iter_tag = tmp_vector_tags.begin();
@@ -661,9 +793,6 @@ Diary::parse_db_body_text_56( std::stringstream &stream )
 
 					++iter_tag;
 				}
-
-				// chapters
-				place_entry_in_chapters( entry_new );
 				// TODO: per entry spellcheck options
 				//entry_new->set_spellcheck( opt_spell );
 				m_entries[ date.m_date ] = entry_new;
@@ -683,7 +812,7 @@ Diary::parse_db_body_text_56( std::stringstream &stream )
 				break;
 			default:
 				print_error( "unrecognized line:\n" + line );
-				return LIFEO::FAILURE;
+				return LIFEO::CORRUPT_FILE;
 				break;
 		}
 	}
@@ -694,6 +823,7 @@ Diary::parse_db_body_text_56( std::stringstream &stream )
 	}
 	else
 	{
+		add_today();
 		print_info( "empty diary" );
 		return LIFEO::EMPTY_DATABASE;
 	}
@@ -905,48 +1035,62 @@ Diary::create_db_body_text( std::stringstream &output )
 	output << "O " << ( m_option_spellcheck ? 's' : '-' ) <<
 					  m_option_sorting_criteria << '\n';
 
+	// STARTUP ACTION (HOME ITEM)
+	output << "S " << m_startup_elem << '\n';
+	output << "L " << m_last_elem << '\n';
+
 	// ROOT TAGS
 	for( PoolTags::const_iterator iter = m_tags.begin();
 		 iter != m_tags.end();
 		 ++iter )
 	{
 		if( iter->second->get_category() == NULL )
-			output << "t " << ( *iter ).first << '\n';
+			output << "ID" << iter->second->get_id() << "\nt " << iter->first << '\n';
 	}
 	// CATEGORIZED TAGS
 	for( PoolCategoriesTags::const_iterator iter = m_tag_categories.begin();
 		 iter != m_tag_categories.end();
 		 ++iter )
 	{
-		output << "T" <<
+		output << "ID" << iter->second->get_id() << "\nT" <<
 				( iter->second->get_expanded() ? 'e' : ' ' ) <<
-				( *iter ).first << '\n';	// tag category
+				iter->first << '\n';	// tag category
 		CategoryTags *category( iter->second );
 		for( CategoryTags::const_iterator iter_tag = category->begin();
 			 iter_tag != category->end();
 			 ++iter_tag )
 		{
-			output << "t " << ( *iter_tag )->get_name() << '\n';
+			output << "ID" << ( *iter_tag )->get_id() << "\nt " <<
+					( *iter_tag )->get_name() << '\n';
 		}
 	}
+	// TOPICS
+	for( CategoryChapters::iterator iter_chapter = m_topics.begin();
+		 iter_chapter != m_topics.end();
+		 ++iter_chapter )
+	{
+		output << "ID" << iter_chapter->second->get_id() << "\no" <<
+				( iter_chapter->second->get_expanded() ? 'e' : ' ' ) <<
+				iter_chapter->first << '\t' <<
+				iter_chapter->second->get_name() << '\n';
+	}
 	// CHAPTERS
 	for( PoolCategoriesChapters::iterator iter = m_chapter_categories.begin();
 		 iter != m_chapter_categories.end();
 		 ++iter )
 	{
-		output << "C" <<
+		output << "ID" << iter->second->get_id() << "\nC" <<
 				( iter->second == m_ptr2chapter_ctg_cur ? 'c' : ' ' ) <<
 				iter->first << '\n';
 		CategoryChapters *category( iter->second );
 		for( CategoryChapters::iterator iter_chapter = category->begin();
 			 iter_chapter != category->end();
-			 iter_chapter++ )
+			 ++iter_chapter )
 		{
-			output << "c" <<
-					( ( *iter_chapter )->get_expanded() ? 'e' : ' ' ) <<
-					( *iter_chapter )->get_name() << '\n';
-			if( ( *iter_chapter )->is_initialized() )
-				output << "d " << ( *iter_chapter )->get_date().m_date << '\n';
+			output << "ID" << iter_chapter->second->get_id() << "\nc" <<
+					( iter_chapter->second->get_expanded() ? 'e' : ' ' ) <<
+					iter_chapter->first << '\t' <<
+					iter_chapter->second->get_name() << '\n';
 		}
 	}
 	// THEMES
@@ -955,8 +1099,12 @@ Diary::create_db_body_text( std::stringstream &output )
 		 ++iter )
 	{
 		Theme *theme( iter->second );
-		output << "M" << ( theme == m_default_theme ? 'd' : ' ' ) <<
-						 theme->m_name << '\n';
+
+		if( theme->is_system() )
+			continue;
+
+		output << "ID" << theme->get_id() << "\nM" <<
+		( theme == m_default_theme ? 'd' : ' ' ) << theme->m_name << '\n';
 		output << "mf" << theme->font.to_string() << '\n';
 		output << "mb" << theme->color_base.to_string() << '\n';
 		output << "mt" << theme->color_text.to_string() << '\n';
@@ -978,15 +1126,15 @@ Diary::create_db_body_text( std::stringstream &output )
 		if( entry->m_text.size() < 1 && entry->m_tags.empty() ) continue;
 
 		// ENTRY DATE
-		output << "E" <<
+		output << "ID" << entry->get_id() << "\nE" <<
 				( entry->m_option_favored ? 'f' : ' ' ) <<
 				entry->m_date.m_date << '\n';
 		output << "Dr" << entry->m_date_created << '\n';
 		output << "Dh" << entry->m_date_changed << '\n';
 
 		// THEME
-		if( entry->m_ptr2theme != NULL )
-			output << "M " << entry->m_ptr2theme->get_name() << '\n';
+		if( entry->get_theme_is_set() )
+			output << "M " << entry->get_theme()->get_name() << '\n';
 
 		// TAGS
 		for( Tagset::const_iterator iter_tag = entry->m_tags.begin();
@@ -1026,31 +1174,65 @@ Diary::create_db_body_text( std::stringstream &output )
 	return true;
 }
 
-void
-Diary::clear( void )
+DiaryElement*
+Diary::get_element( DEID id )
 {
-	if( ! m_path.empty() )
+	PoolDEIDs::iterator iter( m_ids.find( id ) );
+	return( iter == m_ids.end() ? NULL : iter->second );
+}
+
+DiaryElement*
+Diary::get_startup_elem( void )
+{
+	switch( m_startup_elem )
 	{
-		std::string path_lock = m_path + LOCK_SUFFIX;
-		remove( path_lock.c_str() );
-		m_path.clear();
+		case HOME_CURRENT_ELEM:
+			return get_most_current_elem();
+		break;
+		case HOME_LAST_ELEM:
+			return get_element( m_last_elem );
+		break;
+		case DEID_UNSET:
+			return NULL;
+		default:
+			return get_element( m_startup_elem );
 	}
-	m_entries.clear();
-	m_tags.clear();
-	m_tag_categories.clear();
-	m_chapter_categories.clear();
-	m_ptr2chapter_ctg_cur = NULL;
-	m_themes.clear();
-	m_default_theme = &Theme::system_theme;
+}
 
-	m_passphrase.clear();
-	m_filter_text.clear();
-	m_filter_tag = NULL;
-	m_filtering_status = FS_CLEAR;
+DiaryElement*
+Diary::get_most_current_elem( void )
+{
+	Glib::Date gdate;
+	gdate.set_time_current();
+	Date date( gdate.get_year(), gdate.get_month(), gdate.get_day() );
+	Date::date_t diff1( Date::ORDINAL_FLAG );
+	long diff2;
+	DiaryElement *elem( this );
+	bool descending( false );
+	for( Entryiter iter = m_entries.begin(); iter != m_entries.end(); ++iter )
+	{
+		if( ! iter->second->get_filtered_out() && ! iter->second->get_date().is_ordinal() )
+		{
+			diff2 = iter->second->get_date().m_date - date.m_date;
+			if( diff2 < 0 ) diff2 *= -1;
+			if( static_cast< unsigned long >( diff2 ) < diff1 )
+			{
+				diff1 = diff2;
+				elem = iter->second;
+				descending = true;
+			}
+			else
+			if( descending )
+				break;
+		}
+	}
+	return elem;
+}
 
-	// NOTE: only reset body options here:
-	m_option_spellcheck = true;
-	m_option_sorting_criteria = SC_DATE;
+DiaryElement*
+Diary::get_prev_session_elem( void )
+{
+	return get_element( m_last_elem );
 }
 
 // ENTRIES =====================================================================
@@ -1211,16 +1393,15 @@ Entry*
 Diary::create_entry( Date date, const Glib::ustring &content, bool flag_favorite )
 {
 	// make it the last entry of its day:
-	date.reset_order();
+	date.reset_order_1();
 	while( m_entries.find( date.m_date ) != m_entries.end() )
 		++date.m_date;
 
-	Entry *entry = new Entry( date, content, flag_favorite );
+	Entry *entry = new Entry( this, date, content, flag_favorite );
 	entry->m_date_created = time( NULL );
 	entry->m_date_changed = entry->m_date_created;
 
 	m_entries[ date.m_date ] = entry;
-	place_entry_in_chapters( entry );
 
 	return( entry );
 }
@@ -1230,8 +1411,9 @@ Diary::dismiss_entry( Entry *entry )
 {
 	Date::date_t date = entry->get_date().m_date;
 
-	// remove from chapters:
-	remove_entry_from_chapters( entry );
+	// fix startup element
+	if( m_startup_elem == entry->get_id() )
+		m_startup_elem = DEID_MIN;
 
 	// remove from tags:
 	for( Tagset::iterator iter = entry->get_tags().begin();
@@ -1250,7 +1432,7 @@ Diary::dismiss_entry( Entry *entry )
 	{
 		Entry *entry2fix = iter->second;
 		m_entries.erase( iter );
-		entry2fix->get_date().m_date--;
+		entry2fix->m_date.m_date--;
 		m_entries[ entry2fix->get_date().m_date ] = entry2fix;
 		++i;
 	}
@@ -1259,7 +1441,7 @@ Diary::dismiss_entry( Entry *entry )
 	return true;
 }
 
-int
+/*int
 Diary::list_entries( void )
 {
 	int number_of_entries( 0 );
@@ -1270,8 +1452,52 @@ Diary::list_entries( void )
 	}
 	std::cout << number_of_entries << " entries total." << std::endl;
 	return( number_of_entries );
+}*/
+
+void
+Diary::set_entry_date( Entry *entry, const Date &date )
+{
+	Entryiter iter;
+	Date::date_t d( entry->m_date.m_date );
+	m_entries.erase( d );
+
+	for( iter = m_entries.find( ++d );
+		 iter != m_entries.end();
+		 iter = m_entries.find( ++d ) )
+	{
+		Entry *entry_shift( iter->second );
+		m_entries.erase( d );
+		entry_shift->set_date( Date( d - 1 ) );
+		m_entries[ d - 1 ] = entry_shift;
+	}
+
+	// find the last entry in the date/order
+	d = date.m_date;
+	bool flag_replace( false );
+	while( m_entries.find( d ) != m_entries.end() )
+	{
+		flag_replace = true;
+		d++;
+	}
+
+	if( flag_replace )
+	{
+		for( iter = m_entries.find( --d );
+			 d >= date.m_date;
+			 iter = m_entries.find( --d ) )
+		{
+			Entry *entry_shift( iter->second );
+			m_entries.erase( d );
+			entry_shift->set_date( Date( d + 1 ) );
+			m_entries[ d + 1 ] = entry_shift;
+		}
+	}
+
+	entry->set_date( date );
+	m_entries[ date.m_date ] = entry;
 }
 
+// FILTERING ===================================================================
 void
 Diary::set_filter_text( const Glib::ustring &filter )
 {
@@ -1350,6 +1576,20 @@ Diary::replace_text( const Glib::ustring &newtext )
 }
 
 // TAGS ========================================================================
+Tag*
+Diary::create_tag( const Glib::ustring &name, CategoryTags *category )
+{
+	PoolTags::iterator iter = m_tags.find( name );
+	if( iter != m_tags.end() )
+	{
+		PRINT_DEBUG( "Tag already exists: " + name );
+		return( iter->second );
+	}
+	Tag *tag( new Tag( this, name, category ) );
+	m_tags.insert( PoolTags::value_type( name, tag ) );
+	return tag;
+}
+
 void
 Diary::dismiss_tag( Tag *tag )
 {
@@ -1378,7 +1618,7 @@ CategoryTags*
 Diary::create_tag_ctg( void )
 {
 	Glib::ustring name = create_unique_name_for_map( m_tag_categories, _( "New category" ) );
-	CategoryTags *new_category = new CategoryTags( name );
+	CategoryTags *new_category = new CategoryTags( this, name );
 	m_tag_categories.insert( PoolCategoriesTags::value_type( name, new_category ) );
 
 	return new_category;
@@ -1389,7 +1629,7 @@ Diary::create_tag_ctg( const Glib::ustring &name )	// used while reading diary f
 {
 	try
 	{
-		CategoryTags *new_category = new CategoryTags( name );
+		CategoryTags *new_category = new CategoryTags( this, name );
 		m_tag_categories.insert( PoolCategoriesTags::value_type( name, new_category ) );
 		return new_category;
 	}
@@ -1420,7 +1660,7 @@ CategoryChapters*
 Diary::create_chapter_ctg( void )
 {
 	Glib::ustring name = create_unique_name_for_map( m_chapter_categories, _( "New category" ) );
-	CategoryChapters *category = new CategoryChapters( name );
+	CategoryChapters *category = new CategoryChapters( this, name );
 	m_chapter_categories.insert(
 			PoolCategoriesChapters::value_type( name, category ) );
 
@@ -1433,7 +1673,7 @@ Diary::create_chapter_ctg( const Glib::ustring &name )
 	// TODO: check string's availability
 	try
 	{
-		CategoryChapters *category = new CategoryChapters( name );
+		CategoryChapters *category = new CategoryChapters( this, name );
 		m_chapter_categories.insert(
 				PoolCategoriesChapters::value_type( name, category ) );
 		return category;
@@ -1448,12 +1688,96 @@ void
 Diary::dismiss_chapter_ctg( CategoryChapters *category )
 {
 	if( category == m_ptr2chapter_ctg_cur )
-		m_ptr2chapter_ctg_cur = NULL;
+		m_ptr2chapter_ctg_cur = m_chapter_categories.begin()->second;
 
 	delete category;
 	m_chapter_categories.erase( category->get_name() );
 }
 
+void
+Diary::dismiss_chapter( Chapter *chapter )
+{
+	// TODO: add an option to also delete entries within the chapter
+	if( chapter->is_ordinal() )
+	{
+		// ORDER SHIFTING TO PRESERVE CONTINUITY
+		if( m_topics.size() == 1 )	// must be handled separately
+		{
+			m_topics.erase( chapter->get_date().m_date );
+			delete chapter;
+			return;
+		}
+
+		// CALCULATE HELPER VALUES
+		Date d_chapter( chapter->get_date() );
+		Date::date_t d_chapter_next( d_chapter.m_date + Date::ORDINAL_STEP );
+		bool flag_last_chapter( m_topics.find( d_chapter_next ) == m_topics.end() );
+
+		int last_order_of_prev_chapter( 0 );
+		if( flag_last_chapter )
+		{
+			Chapter *chapter_prev( m_topics.get_chapter(
+					Date( d_chapter.m_date - Date::ORDINAL_STEP ) ) );
+			Date d_first_free( chapter_prev->get_free_order() );
+			last_order_of_prev_chapter = d_first_free.get_order() - 1;
+		}
+		else
+		{
+			Date d_first_free( chapter->get_free_order() );
+			last_order_of_prev_chapter = d_first_free.get_order() - 1;
+		}
+
+		// ACTUALLY DISMISS THE TOPIC
+		m_topics.erase( chapter->get_date().m_date );
+		delete chapter;
+
+		// SHIFT TOPICS
+		for( Date::date_t d = d_chapter.m_date + Date::ORDINAL_STEP; ;
+			 d += Date::ORDINAL_STEP )
+		{
+			CategoryChapters::iterator iter( m_topics.find( d ) );
+			if( iter == m_topics.end() )
+				break;
+			Chapter *chpt( iter->second );
+			m_topics.erase( d );
+			chpt->set_date( Date( d - Date::ORDINAL_STEP ) );
+			m_topics[ chpt->get_date().m_date ] = chpt;
+		}
+
+		// SHIFT ENTRIES
+		if( m_entries.size() > 0 )
+		{
+			for( Date::date_t d = d_chapter.m_date + 1; d <= m_entries.begin()->first; )
+			{
+				Entryiter iter( m_entries.find( d ) );
+				if( iter == m_entries.end() )
+				{
+					d += Date::ORDINAL_STEP;
+					Date::reset_order_1( d );
+					continue;
+				}
+
+				Entry *entry( iter->second );
+				int order_diff( entry->get_date().get_ordinal_order() -
+								d_chapter.get_ordinal_order() );
+
+				if( order_diff > 0 || ( order_diff == 0 && flag_last_chapter ) )
+				{
+					m_entries.erase( d );
+					Date d_new( d - Date::ORDINAL_STEP );
+					if( order_diff == 1 || ( order_diff == 0 && flag_last_chapter ) )
+						d_new.m_date += last_order_of_prev_chapter;
+					entry->set_date( d_new );
+					m_entries[ d_new.m_date ] = entry;
+				}
+				d++;
+			}
+		}
+	}
+	else
+		m_ptr2chapter_ctg_cur->dismiss_chapter( chapter );
+}
+
 bool
 Diary::rename_chapter_ctg( CategoryChapters *category, const Glib::ustring &name )
 {
@@ -1468,9 +1792,19 @@ Diary::rename_chapter_ctg( CategoryChapters *category, const Glib::ustring &name
 	return true;
 }
 
-void
-Diary::place_entry_in_chapters( Entry *entry )
+Chapter*
+Diary::add_topic( const Glib::ustring &name )
 {
+	Chapter *c( m_topics.create_chapter( name, m_topics.get_free_order_ordinal() ) );
+	c->set_expanded( true );
+	return c;
+}
+
+/*
+void
+Diary::place_entry_in_chapters( DiaryElement *entry )
+{
+	
 	for( PoolCategoriesChapters::const_iterator iter_set = m_chapter_categories.begin();
 		 iter_set != m_chapter_categories.end();
 		 ++iter_set )
@@ -1480,36 +1814,43 @@ Diary::place_entry_in_chapters( Entry *entry )
 			 ++iter_chapter )
 		{
 			if( ( * iter_chapter )->is_initialized() )
+			{
 				if( entry->get_date() >= ( * iter_chapter )->get_date() )
 				{
 					( * iter_chapter )->get_items()->insert( entry );
 					break;
 				}
+			}
 		}
 	}
-}
+}*/
 
-void
-Diary::update_entries_in_chapter_ctg( CategoryChapters *set )
+/*void
+Diary::update_entries_in_chapters( CategoryChapters *set )
 {
+	// if set==NULL update current
+	if( set == NULL )
+	{
+		if( m_ptr2chapter_ctg_cur != NULL )
+			set = m_ptr2chapter_ctg_cur;
+		else
+			return;
+	}
 	Entryiter	iter_last = m_entries.end();
-	bool		flag_early;
 
-	// CLEAR ALL SETS
-	for(	CategoryChapters::const_iterator iter_chapter = set->begin();
-			iter_chapter != set->end();
-			++iter_chapter )
+	// CLEAR ALL CHAPTERS
+	for( CategoryChapters::const_iterator iter_chapter = set->begin();
+		 iter_chapter != set->end();
+		 ++iter_chapter )
 	{
 		( * iter_chapter )->clear();
 	}
-	//set->get_earlier()->clear();
 
 	// INSERT ENTRIES AGAIN
 	for(	Entryiter iter_entry = m_entries.begin();
 			iter_entry != iter_last;
 			++iter_entry )
 	{
-		flag_early = true;
 		// TODO: efficiency: do not start form the beginning each time:
 		for(	CategoryChapters::const_iterator iter_chapter = set->begin();
 				iter_chapter != set->end();
@@ -1520,19 +1861,17 @@ Diary::update_entries_in_chapter_ctg( CategoryChapters *set )
 				if( iter_entry->second->get_date().get_pure()
 					>= ( * iter_chapter )->get_date().m_date )
 				{
-					( * iter_chapter )->get_items()->insert( iter_entry->second );
-					flag_early = false;
+					DiaryElement *elem( iter_entry->second );
+					( * iter_chapter )->get_items()->insert( elem );
 					break;
 				}
 			}
 		}
-		//if( flag_early )
-			//set->get_earlier()->get_items()->insert( iter_entry->second );
 	}
 }
 
 void
-Diary::remove_entry_from_chapters( Entry *entry )
+Diary::remove_entry_from_chapters( DiaryElement *entry )
 {
 	for( PoolCategoriesChapters::const_iterator iter_set = m_chapter_categories.begin();
 		 iter_set != m_chapter_categories.end();
@@ -1542,9 +1881,46 @@ Diary::remove_entry_from_chapters( Entry *entry )
 			 iter_chapter != iter_set->second->end();
 			 ++iter_chapter )
 		{
-			( * iter_chapter )->get_items()->erase( entry );
+			SetDiaryElements *elements( ( * iter_chapter )->get_items() );
+			if( elements->find( entry ) != elements->end() )
+			{
+				PRINT_DEBUG( "element erased" );
+				elements->erase( entry );
+				break;
+			}
 		}
 	}
+}*/
+
+Date
+Diary::get_free_chapter_order_temporal( void )
+{
+	time_t t = time( NULL );
+	struct tm *ti = localtime( &t );
+	Date date( ti->tm_year + 1900, ti->tm_mon + 1, ti->tm_mday );
+
+	while( m_ptr2chapter_ctg_cur->get_chapter( date ) )
+	{
+		date.forward_day();
+	}
+
+	return date;
+}
+
+Date
+Diary::get_free_chapter_order_ordinal( void )
+{
+	return m_ptr2chapter_ctg_cur->get_free_order_ordinal();
+}
+
+bool
+Diary::make_free_entry_order( Date &date )
+{
+	date.reset_order_1();
+	while( m_entries.find( date.m_date ) != m_entries.end() )
+		++date.m_date;
+
+	return true;	// reserved for bounds checking
 }
 
 // THEMES ======================================================================
@@ -1554,7 +1930,7 @@ Diary::create_theme( const Glib::ustring& name )
 	PoolThemes::iterator iter = m_themes.find( name );
 	if( iter != m_themes.end() )
 		return( iter->second );
-	Theme *theme = new Theme( name );
+	Theme *theme = new Theme( this, name );
 	m_themes.insert( PoolThemes::value_type( name, theme ) );
 	return theme;
 }
@@ -1576,7 +1952,7 @@ Diary::dismiss_theme( Theme *theme )
 	// change default theme if necessary
 	if( m_default_theme == theme )
 	{
-		m_default_theme = &Theme::system_theme;
+		m_default_theme = ThemeSystem::get();
 	}
 	// fix entries
 	for( Entryiter iter = m_entries.begin(); iter != m_entries.end(); ++iter )
@@ -1598,7 +1974,7 @@ Diary::show( void )
 bool
 Diary::import_tag( Tag *tag )
 {
-	return( m_tags.create( tag->get_name() ) != NULL );
+	return( create_tag( tag->get_name() ) != NULL );
 }
 
 bool
@@ -1610,19 +1986,20 @@ Diary::import_entryset( const PoolEntries &set,
 	Tag *tag_all = NULL;
 
 	if( ! tag_all_str.empty() )
-		tag_all = m_tags.create( tag_all_str );
+		tag_all = create_tag( tag_all_str );
 
 	for( Entryciter iter = set.begin();
 		 iter != set.end();
 		 ++iter )
 	{
 		entry_ext = iter->second;
-		entry_new = new Entry( entry_ext->m_date,
+		entry_new = new Entry( this,
+							   entry_ext->m_date,
 							   entry_ext->m_text,
 							   entry_ext->m_option_favored );
 
 		// fix order:
-		entry_new->m_date.reset_order();
+		entry_new->m_date.reset_order_1();
 		while( m_entries.find( entry_new->m_date.m_date ) != m_entries.end() )
 			entry_new->m_date.m_date++;
 
@@ -1632,7 +2009,6 @@ Diary::import_entryset( const PoolEntries &set,
 
 		// insert it into the diary:
 		m_entries[ entry_new->m_date.m_date ] = entry_new;
-		place_entry_in_chapters( entry_new );
 
 		if( flag_import_tags )
 		{
