@@ -1,6 +1,6 @@
 /***********************************************************************************
 
-	Copyright (C) 2007-2009 Ahmet Öztürk (aoz_2@yahoo.com)
+	Copyright (C) 2007-2011 Ahmet Öztürk (aoz_2@yahoo.com)
 
 	This file is part of Lifeograph.
 
@@ -32,11 +32,9 @@
 namespace LIFEO
 {
 const char						DB_FILE_HEADER[] = "LIFEOGRAPHDB";
-const int						DB_FILE_VERSION_INT( 73 );
+const int						DB_FILE_VERSION_INT( 74 );
 const int						DB_FILE_VERSION_INT_MIN( 56 );
 const std::string::size_type	PASSPHRASE_MIN_SIZE( 4 );
-const unsigned int				YEAR_MAX = 2099;
-const unsigned int				YEAR_MIN = 1900;
 const char						LOCK_SUFFIX[] = ".~LOCK~";
 
 
@@ -65,7 +63,7 @@ typedef std::vector< Entry* >				Entryvector;
 typedef std::vector< Entry* >::iterator		Entryiterv;
 
 
-class Diary : public DiaryElement
+class Diary : public DiaryBase, public DiaryElement
 {
 	public:
 		typedef unsigned int FilteringStatus;
@@ -86,7 +84,9 @@ class Diary : public DiaryElement
 		static Diary			*d;
 
 		bool					is_old( void )
-		{ return m_flag_v56; }
+		{ return( m_read_version < DB_FILE_VERSION_INT ); }
+
+		void					init_new( const std::string& );
 
 		// DISK I/O
 		LIFEO::Result			read_body( void );
@@ -103,12 +103,16 @@ class Diary : public DiaryElement
 		{ return m_entries.size(); }
 		Glib::ustring			get_name( void ) const
 		{ return Glib::filename_display_basename( m_path ); }
-		//Date					get_date( void ) const; // maybe earliest entries date
 		Type					get_type( void ) const
 		{ return IT_DIARY; }
 		Glib::RefPtr< Gdk::Pixbuf >&
 								get_icon( void ) const;
 
+		Glib::ustring			get_list_str( void ) const
+		{ return Glib::ustring::compose( "<b>%1</b>",
+				Glib::Markup::escape_text(
+						Glib::filename_display_basename( m_path ) ) ); }
+
 		void					set_path( const std::string& );
 		const std::string&		get_path( void ) const;
 		bool					is_path_set( void ) const;
@@ -119,6 +123,11 @@ class Diary : public DiaryElement
 		bool					compare_passphrase( const std::string& ) const;
 		bool					is_passphrase_set( void ) const;
 
+		DiaryElement*			get_element( DEID );
+		DiaryElement*			get_startup_elem( void );
+		DiaryElement*			get_most_current_elem( void );
+		DiaryElement*			get_prev_session_elem( void );
+
 		Entry*					get_entry( const Date& );
 		Entryvector*			get_entries( Date::date_t );
 		PoolEntries&			get_entries( void )
@@ -127,7 +136,9 @@ class Diary : public DiaryElement
 		Entry*					get_entry_today( void );
 		Entry*					get_entry_nextinday( const Date& );
 		Entry*					get_entry_first( void );
+		void					set_entry_date( Entry*, const Date& );
 		//int						get_entrydayorder( const Entry * const ) const;
+
 		bool					is_first( const Entry * const ) const;
 		bool					is_last( const Entry * const ) const;
 		SortingCriteria			get_sorting_criteria( void ) const
@@ -161,6 +172,7 @@ class Diary : public DiaryElement
 		CategoryTags*			create_tag_ctg( void );
 		CategoryTags*			create_tag_ctg( const Glib::ustring& );
 		void					dismiss_tag_ctg( CategoryTags* );
+		Tag*					create_tag( const Glib::ustring&, CategoryTags* = NULL );
 		void					dismiss_tag( Tag* );
 
 		const PoolCategoriesChapters*
@@ -171,10 +183,16 @@ class Diary : public DiaryElement
 		CategoryChapters*		create_chapter_ctg( void );
 		CategoryChapters*		create_chapter_ctg( const Glib::ustring& );
 		void					dismiss_chapter_ctg( CategoryChapters* );
+		void					dismiss_chapter( Chapter* );
 		bool					rename_chapter_ctg( CategoryChapters*, const Glib::ustring& );
-		void					place_entry_in_chapters( Entry* );
-		void					update_entries_in_chapter_ctg( CategoryChapters* );
-		void					remove_entry_from_chapters( Entry* );
+		const CategoryChapters*	get_topics( void )
+		{ return &m_topics; }
+		Chapter*				add_topic( const Glib::ustring& );
+
+		Date					get_free_chapter_order_temporal( void );
+		Date					get_free_chapter_order_ordinal( void );
+
+		bool					make_free_entry_order( Date& );
 
 		bool					get_spellcheck( void ) const
 		{ return m_option_spellcheck; }
@@ -211,12 +229,16 @@ class Diary : public DiaryElement
 		PoolCategoriesTags		m_tag_categories;
 		PoolCategoriesChapters	m_chapter_categories;
 		CategoryChapters		*m_ptr2chapter_ctg_cur;
+		CategoryChapters		m_topics;	// ordinal chapters
 		PoolThemes				m_themes;
 		Theme					*m_default_theme;
+
+		DEID					m_startup_elem;
+		DEID					m_last_elem;
 		// options & flags
 		bool					m_option_spellcheck;
 		SortingCriteria			m_option_sorting_criteria;
-		bool					m_flag_v56;
+		int						m_read_version;
 		// filtering
 		Glib::ustring			m_filter_text;
 		const Tag				*m_filter_tag;
@@ -242,6 +264,7 @@ class Diary : public DiaryElement
 	friend class ThemeView;
 	friend class PanelDiary;
 	friend class TagPanel;
+	friend class WidgetPanelExp;
 	friend class DialogImport;
 };
 
