@@ -1,6 +1,6 @@
 /***********************************************************************************
 
-	Copyright (C) 2007-2010 Ahmet Öztürk (aoz_2@yahoo.com)
+	Copyright (C) 2007-2011 Ahmet Öztürk (aoz_2@yahoo.com)
 
 	This file is part of Lifeograph.
 
@@ -38,14 +38,97 @@ static const SortingCriteria SC_DATE	= 'd';
 static const SortingCriteria SC_SIZE	= 's';
 //static const SortingCriteria SC_CHANGE_DATE	= 'c';
 
+typedef unsigned long DEID;	// unique diary element id
+static const DEID	DEID_MIN = 10000;	// reserved for Diary itself
+static const DEID	DEID_UNSET = 404;		// :)
+static const DEID	HOME_CURRENT_ELEM = 1;	// element shown at startup
+static const DEID	HOME_LAST_ELEM = 2;		// element shown at startup
+static const DEID	HOME_FIXED_ELEM = 3;	// element shown at startup (defines boundary)
+
+class DiaryElement;	// forward declaration
+
+
+// DIARYBASE (FOR COMMUNICATION)
+class DiaryBase
+{
+	public:
+		typedef std::map< DEID, DiaryElement* >
+									PoolDEIDs;
+
+									DiaryBase( void )
+		:	m_current_id( DEID_MIN ), m_force_id( DEID_UNSET ) {}
+
+		DEID						create_new_id( DiaryElement *element )
+		{
+			DEID retval;
+			if( m_force_id == DEID_UNSET )
+				retval = m_current_id;
+			else
+			{
+				retval = m_force_id;
+				if( m_force_id > m_current_id )
+					m_current_id = m_force_id;
+				m_force_id = DEID_UNSET;
+			}
+			m_ids[ retval ] = element;
+
+			do
+			{
+				m_current_id++;
+			}
+			while( m_ids.find( m_current_id ) != m_ids.end() );
+
+			return retval;
+		}
+
+		bool						set_force_id( DEID id )
+		{
+			if( m_ids.find( id ) != m_ids.end() || id <= DEID_MIN )
+				return false;
+			m_force_id = id;
+			return true;
+		}
+
+		void						clear( void )
+		{
+			m_force_id = DEID_UNSET;
+			m_current_id = DEID_MIN;
+			m_ids.clear();
+		}
+
+		virtual bool				make_free_entry_order( Date& ) = 0;
+
+	protected:
+		DEID						m_current_id;
+		DEID						m_force_id;
+		PoolDEIDs					m_ids;
+};
+
 
 class ListData;	// forward declaration
 
 // DIARYELEMENT
-class DiaryElement
+class NamedElement
 {
 	public:
-		static bool 			FLAG_ALLOCATE_GUI_FOR_DIARY;
+								NamedElement( void )
+								:	m_name( "" ) {}
+								NamedElement( const Glib::ustring &name )
+								:	m_name( name ) {}
+
+		virtual Glib::ustring	get_name( void ) const
+		{ return m_name; }
+		void					set_name( const Glib::ustring &name )
+		{ m_name = name; }
+
+	protected:
+		Glib::ustring			m_name;
+};
+
+class DiaryElement : public NamedElement
+{
+	public:
+		static bool				FLAG_ALLOCATE_GUI_FOR_DIARY;
 
 		enum Type
 		{
@@ -54,7 +137,8 @@ class DiaryElement
 			IT_DIARY, IT_EARLIER, IT_CHAPTER, IT_ENTRY, IT_DATE
 		};
 
-								DiaryElement( void );
+								DiaryElement( DiaryBase * const,
+											  const Glib::ustring& = "" );
 		virtual					~DiaryElement() {}
 
 		virtual Date			get_date( void ) const
@@ -64,34 +148,23 @@ class DiaryElement
 		virtual Glib::RefPtr< Gdk::Pixbuf >&
 								get_icon( void ) const = 0;
 
-		virtual	Glib::ustring	get_1st_str( void ) const
-		{ return ""; }
-		virtual	Glib::ustring	get_2nd_str( void ) const
+		virtual	Glib::ustring	get_list_str( void ) const
 		{ return ""; }
 
+		DEID					get_id( void ) const
+		{ return m_id; }
+
 		virtual void			show( void ) = 0;
 		virtual void			prepare_for_hiding( void ) {}
 		virtual bool			get_filtered_out( void )
 		{ return false; }
 
 		ListData				*m_list_data;
-};
 
-class DiaryElementNamed : public DiaryElement
-{
-	public:
-								DiaryElementNamed( void )
-								:	m_name( "" ) {}
-								DiaryElementNamed( const Glib::ustring &name )
-								:	m_name( name ) {}
-
-		Glib::ustring			get_name( void ) const
-		{ return m_name; }
-		void					set_name( const Glib::ustring &name )
-		{ m_name = name; }
+		DiaryBase * const		m_ptr2diary;
 
 	protected:
-		Glib::ustring			m_name;
+		const DEID				m_id;
 };
 
 class ListData
@@ -103,14 +176,12 @@ class ListData
 				Colrec()
 				{
 					add( ptr );
-					add( id );
 					add( info );
 					add( icon );
 				}
 				// HIDDEN COLUMNS
 				Gtk::TreeModelColumn< DiaryElement* >					ptr;
 				// COLUMNS
-				Gtk::TreeModelColumn< Glib::ustring >					id;
 				Gtk::TreeModelColumn< Glib::ustring >					info;
 				Gtk::TreeModelColumn< Glib::RefPtr< Gdk::Pixbuf > >		icon;
 		};
@@ -120,12 +191,12 @@ class ListData
 };
 
 bool compare_listitems( DiaryElement*, DiaryElement* );
-bool compare_listitems_by_name( DiaryElementNamed*, DiaryElementNamed* );
+bool compare_listitems_by_name( DiaryElement*, DiaryElement* );
 bool compare_names( const Glib::ustring&, const Glib::ustring& );
 
 typedef bool( *FuncCompareStrings )( const Glib::ustring&, const Glib::ustring& ) ;
 typedef bool( *FuncCompareDiaryElem )( DiaryElement*, DiaryElement* ) ;
-typedef bool( *FuncCompareDiaryElemNamed )( DiaryElementNamed*, DiaryElementNamed* ) ;
+typedef bool( *FuncCompareDiaryElemNamed )( DiaryElement*, DiaryElement* ) ;
 typedef std::set< DiaryElement*, FuncCompareDiaryElem > SetDiaryElements;
 
 template< class T >
@@ -144,15 +215,15 @@ create_unique_name_for_map( const std::map< Glib::ustring, T*, FuncCompareString
 	return name;
 }
 
-class DiaryElementContainer : public DiaryElementNamed
+class DiaryElementContainer : public DiaryElement
 {
 // NOTE: this class and its derivatives are not responsible for handling of...
 // ...its children's allocation and deletion
 	public:
-								DiaryElementContainer( void )
-								:	m_items( compare_listitems ) {}
-								DiaryElementContainer( const Glib::ustring &name )
-								:	DiaryElementNamed( name ), m_items( compare_listitems ) {}
+								DiaryElementContainer( DiaryBase * const d )
+								:	DiaryElement( d, "" ), m_items( compare_listitems ) {}
+								DiaryElementContainer( DiaryBase * const d, const Glib::ustring &name )
+								:	DiaryElement( d, name ), m_items( compare_listitems ) {}
 
 		SetDiaryElements*		get_items( void )
 		{ return &m_items; }
@@ -162,8 +233,6 @@ class DiaryElementContainer : public DiaryElementNamed
 		void					clear( void )
 		{ m_items.clear(); }
 
-		DiaryElement*			add_to_list( Glib::RefPtr< Gtk::TreeStore >&,
-											 const Gtk::TreeRow& );
 
 	protected:
 		SetDiaryElements		m_items;
@@ -175,13 +244,13 @@ class Tag;	// forward declaration
 
 // TAG CATEGORY
 // TODO: should be a DiaryElementContainer as it is not responsible for memory handling
-class CategoryTags : public DiaryElementNamed, public std::set< Tag*, FuncCompareDiaryElemNamed >
+class CategoryTags : public DiaryElement, public std::set< Tag*, FuncCompareDiaryElemNamed >
 {
 	public:
 		struct Shower { virtual void show( CategoryTags& ) = 0; };
 		static Shower			*shower;
 
-								CategoryTags( const Glib::ustring& );
+								CategoryTags( DiaryBase * const, const Glib::ustring& );
 
 		void					show( void );
 
@@ -192,8 +261,8 @@ class CategoryTags : public DiaryElementNamed, public std::set< Tag*, FuncCompar
 		Glib::RefPtr< Gdk::Pixbuf >&
 								get_icon( void ) const;
 
-		Glib::ustring			get_2nd_str( void ) const
-		{ return m_name; }
+		Glib::ustring			get_list_str( void ) const
+		{ return Glib::Markup::escape_text( m_name ); }
 
 		bool					get_expanded( void ) const
 		{ return m_flag_expanded; }
@@ -226,8 +295,10 @@ class Tag : public DiaryElementContainer
 		struct Shower { virtual void show( Tag& ) = 0; };
 		static Shower			*shower;
 
-								Tag( void ) : m_ptr2category( NULL ) {}
-		explicit				Tag( const Glib::ustring&, CategoryTags* );
+								Tag( DiaryBase * const d )
+								:	DiaryElementContainer( d ),
+									m_ptr2category( NULL ) {}
+		explicit				Tag( DiaryBase * const, const Glib::ustring&, CategoryTags* );
 		virtual					~Tag( void );
 
 		void					show( void );
@@ -263,8 +334,6 @@ class PoolTags : public std::map< Glib::ustring, Tag*, FuncCompareStrings >
 								~PoolTags();
 
 		bool					handle_tag_changed( );
-		Tag*					create( const Glib::ustring&,
-										CategoryTags* = NULL );
 		bool					rename( Tag*, const Glib::ustring& );
 		Tag*					get_tag( unsigned int );
 		Tag*					get_tag( const Glib::ustring& );
@@ -290,14 +359,14 @@ class Tagset : public std::set< Tag*, FuncCompareDiaryElemNamed >
 };
 
 // CHAPTER =====================================================================
-class Chapter : public DiaryElementContainer
+class Chapter : public DiaryElement
 {
 	public:
 		struct Shower { virtual void show( Chapter& ) = 0; };
 		static Shower			*shower;
 
-		explicit				Chapter( const Glib::ustring& );
-								Chapter( const Glib::ustring&, Date );
+		explicit				Chapter( DiaryBase * const, const Glib::ustring& );
+								Chapter( DiaryBase * const, const Glib::ustring&, Date );
 		// this class should have a virtual dtor for some reason because
 		// if it does not, its derived classes cannot be deleted
 		virtual					~Chapter() {}
@@ -307,30 +376,40 @@ class Chapter : public DiaryElementContainer
 		void					set_name( const Glib::ustring& );
 		Date					get_date( void ) const;
 		Glib::ustring			get_date_str( void ) const;
-		void					set_date( Date );
+		void					set_date( const Date& );
 		Type					get_type( void ) const
 		{ return IT_CHAPTER; }
 		Glib::RefPtr< Gdk::Pixbuf >&
 								get_icon( void ) const;
+		Glib::RefPtr< Gdk::Pixbuf >&
+								get_icon_32( void ) const;
+		virtual int				get_size( void ) const	// TODO: stub!
+		{ return m_size; }
 
-		Glib::ustring			get_1st_str( void ) const
-		{ return Glib::ustring::compose( "<b>%1</b>", Glib::Markup::escape_text(
-			m_date_begin.format_string() ) ); }
-		Glib::ustring			get_2nd_str( void ) const
-		//{ return Glib::ustring::compose( "<b>%1</b>", Glib::Markup::escape_text( m_name ) ); }
-		{ return m_name; }
+		Date					get_free_order( void ) const;
+
+		Glib::ustring			get_list_str( void ) const
+		{ return Glib::ustring::compose( "<b>%1 -  %2</b>", m_date_begin.format_string(),
+				Glib::Markup::escape_text( m_name ) ); }
 
 		bool					get_expanded( void ) const
 		{ return m_flag_expanded; }
 		void					set_expanded( bool flag_expanded )
 		{ m_flag_expanded = flag_expanded; }
 
-		bool					is_initialized( void )
-		{ return m_flag_initialized; }
+		bool					is_initialized( void ) const
+		{ return( m_date_begin.m_date != Date::NOTSET ); }
+		bool					is_ordinal( void ) const
+		{ return m_date_begin.is_ordinal(); }
+
+		void					reset_size( void )
+		{ m_size = 0; }
+		void					increase_size()
+		{ m_size++; }
 
 	protected:
 		Date					m_date_begin;
-		bool					m_flag_initialized;
+		int						m_size;
 		bool					m_flag_expanded;
 
 	friend class CategoryChapters;
@@ -338,10 +417,10 @@ class Chapter : public DiaryElementContainer
 };
 
 class CategoryChapters :
-		public DiaryElementNamed, public std::set< Chapter*, FuncCompareDiaryElem >
+		public DiaryElement, public std::map< Date::date_t, Chapter*, FuncCompareDates >
 {
 	public:
-		explicit				CategoryChapters( const Glib::ustring& );
+		explicit				CategoryChapters( DiaryBase * const d, const Glib::ustring& );
 		virtual					~CategoryChapters();
 
 		virtual int				get_size( void ) const
@@ -352,14 +431,17 @@ class CategoryChapters :
 								get_icon( void ) const;
 		void					show( void ) {}	// not used
 
-		Chapter*				add_chapter( const Glib::ustring& );
-		void					remove_chapter( Chapter* );
+		Chapter*				get_chapter( const Date& );
+
+		Chapter*				create_chapter( const Glib::ustring&, const Date& );
+		void					dismiss_chapter( Chapter* );
 		bool					rename_chapter( Chapter*, const Glib::ustring& );
-		bool					set_chapter_date( Chapter*, const Glib::ustring& );
 		bool					set_chapter_date( Chapter*, const Date& );
 
 		void					clear( void );
 
+		Date					get_free_order_ordinal( void ) const;
+
 	protected:
 
 };
@@ -376,15 +458,16 @@ class PoolCategoriesChapters :
 };
 
 // THEMES
-class Theme : public DiaryElementNamed
+class Theme : public DiaryElement
 {
 	public:
 		// Shower sub-class is for communication between Theme and...
 		// ...ThemeView classes. we may resort to the sigc::signals if needed:
 		struct Shower { virtual void show( Theme& ) = 0; };
 
-									Theme( const Glib::ustring& );
-									Theme( const Glib::ustring&,
+									Theme( DiaryBase * const, const Glib::ustring& );
+									Theme( DiaryBase * const,
+										   const Glib::ustring&,
 										   const Glib::ustring&,
 										   const std::string&,
 										   const std::string&,
@@ -401,9 +484,12 @@ class Theme : public DiaryElementNamed
 		Glib::RefPtr< Gdk::Pixbuf >&
 									get_icon( void ) const;
 
-		Glib::ustring				get_1st_str( void ) const
+		Glib::ustring				get_list_str( void ) const
 		{ return Glib::Markup::escape_text( m_name ); }
 
+		virtual bool				is_system( void ) const
+		{ return false; }
+
 		Pango::FontDescription		font;
 		Gdk::Color					color_base;
 		Gdk::Color					color_text;
@@ -411,13 +497,27 @@ class Theme : public DiaryElementNamed
 		Gdk::Color					color_subheading;
 		Gdk::Color					color_highlight;
 
-		static Theme				system_theme;
-
 		static Shower				*shower;
 
 	friend class Diary;
 };
 
+class ThemeSystem : public Theme
+{
+	public:
+		static ThemeSystem*			get( void );
+		bool						is_system( void ) const
+		{ return true; }
+
+	protected:
+									ThemeSystem( const Glib::ustring&,
+												 const std::string&,
+												 const std::string&,
+												 const std::string&,
+												 const std::string&,
+												 const std::string& );
+};
+
 class PoolThemes : public std::map< Glib::ustring, Theme*, FuncCompareStrings >
 {
 	public:
