@@ -1,6 +1,6 @@
 /***********************************************************************************
 
-	Copyright (C) 2007-2008 Ahmet Öztürk (aoz_2@yahoo.com)
+	Copyright (C) 2007-2011 Ahmet Öztürk (aoz_2@yahoo.com)
 
 	Parts of this file are loosely based on an example gcrypt program
 	on http://punkroy.drque.net/
@@ -42,6 +42,26 @@ Error::Error( const Glib::ustring &error_message )
 }
 
 // DATE
+Date::Date( const Glib::ustring &str_date )
+	:	m_date( 0 )
+{
+	Glib::Date glib_date;
+
+	glib_date.set_parse( str_date );
+
+	if( ! glib_date.valid() )
+		throw LIFEO::Error( "Date format not accepted" );
+
+	guint16 year = glib_date.get_year();
+
+	if( year < YEAR_MIN || year > YEAR_MAX )
+		throw LIFEO::Error( "Date out of acceptable range" );
+
+	m_date = ( year << 19 ) +
+			 ( glib_date.get_month() << 15 ) +
+			 ( glib_date.get_day() << 10 );
+}
+
 Glib::Date::Month
 Date::get_month_glib( void ) const
 {
@@ -80,10 +100,18 @@ Date::get_month_glib( void ) const
 Glib::ustring
 Date::format_string( void ) const
 {
-	return Glib::ustring::compose( "%1.%2.%3",
-			Glib::ustring::format( std::setfill( L'0' ), std::setw( 2 ), get_year() ),
-			Glib::ustring::format( std::setfill( L'0' ), std::setw( 2 ), get_month() ),
-			Glib::ustring::format( std::setfill( L'0' ), std::setw( 2 ), get_day() ) );
+	if( m_date & ORDINAL_FLAG )
+	{
+		return Glib::ustring::compose( get_order() ? "%1.%2" : "%1",
+				get_ordinal_order() + 1, get_order() );
+	}
+	else
+	{
+		return Glib::ustring::compose( "%1.%2.%3",
+				Glib::ustring::format( std::setfill( L'0' ), std::setw( 2 ), get_year() ),
+				Glib::ustring::format( std::setfill( L'0' ), std::setw( 2 ), get_month() ),
+				Glib::ustring::format( std::setfill( L'0' ), std::setw( 2 ), get_day() ) );
+	}
 }
 
 Glib::ustring
@@ -98,6 +126,17 @@ Date::format_string( const time_t *time )
 			Glib::ustring::format( std::setfill( L'0' ), std::setw( 2 ), timeinfo->tm_min ) );
 }
 
+// date only
+Glib::ustring
+Date::format_string_do( const time_t *time )
+{
+	struct tm *timeinfo = localtime( time );
+	return Glib::ustring::compose( "%1.%2.%3",
+			1900 + timeinfo->tm_year,
+			Glib::ustring::format( std::setfill( L'0' ), std::setw( 2 ), timeinfo->tm_mon + 1 ),
+			Glib::ustring::format( std::setfill( L'0' ), std::setw( 2 ), timeinfo->tm_mday ) );
+}
+
 Glib::ustring
 Date::get_year_str( void ) const
 {
@@ -150,6 +189,55 @@ Date::get_ctime( void ) const
 	return( mktime( timeinfo ) );
 }
 
+void
+Date::forward_months( int months )
+{
+	months += ( ( m_date & 0x78000 ) >> 15 ); // get month
+	m_date &= 0x7FF80000;	// isolate year
+	const int mod_months = months % 12;
+	if( mod_months == 0 )
+	{
+		m_date += make_year( ( months / 12 ) - 1);
+		m_date |= 0x60000;	// make month 12
+	}
+	else
+	{
+		m_date += make_year( months / 12 );
+		m_date |= make_month( mod_months );
+	}
+}
+
+void
+Date::forward_month( void )
+{
+	int months = get_month() + 1;
+	m_date &= 0x7FF80000;	// isolate year
+	const int mod_months = months % 12;
+	if( mod_months == 0 )
+	{
+		m_date += make_year( ( months / 12 ) - 1);
+		m_date |= 0x60000;	// make month 12
+	}
+	else
+	{
+		m_date += make_year( months / 12 );
+		m_date |= make_month( mod_months );
+	}
+}
+
+void
+Date::forward_day( void )
+{
+	int day = get_day();
+	if( day >= Glib::Date::get_days_in_month( get_month_glib(), get_year() ) )
+	{
+		set_day( 1 );
+		forward_month();
+	}
+	else
+		set_day( day + 1 );
+}
+
 // CONSOLE MESSAGES
 void
 LIFEO::print_error( const Glib::ustring &description )
@@ -168,14 +256,14 @@ long
 LIFEO::convert_string( const std::string &str )
 {
 	//TODO: add negative number support
-	long f_result=0;	// result
-	for (unsigned int i=0;
-		 i<str.size() && i<10 && int (str[i])>='0' && int (str[i])<='9';
-		 i++)
+	long result( 0 );	// result
+	for( unsigned int i = 0;
+		 i < str.size() && i < 10 && int ( str[ i ] ) >= '0' && int ( str[ i ] ) <= '9';
+		 i++ )
 	{
-		f_result = (f_result * 10) + int (str[i]) - '0';
+		result = ( result * 10 ) + int ( str[ i ] ) - '0';
 	}
-	return (f_result);
+	return( result );
 }
 
 // COLOR PROCESSING
