@@ -1,6 +1,6 @@
 /***********************************************************************************
 
-	Copyright (C) 2007-2009 Ahmet Öztürk (aoz_2@yahoo.com)
+	Copyright (C) 2007-2011 Ahmet Öztürk (aoz_2@yahoo.com)
 
     This file is part of Lifeograph.
 
@@ -63,10 +63,18 @@ static const int		RESPONSE_GO			= 1;
 // order: 10 bits
 // day:	   5 bits
 // month:  4 bits
-// year:  13 bits
+// year:  12 bits
+// ordinal flag:  1 bit (32nd bit)
+
 class Date
 {
 	public:
+		const static unsigned int	YEAR_MAX		= 2199;
+		const static unsigned int	YEAR_MIN		= 1900;
+		const static unsigned long	ORDINAL_FLAG	= 0x80000000;
+		const static unsigned long	NOTSET			= 0xFFFFFFFF;
+		const static unsigned long	ORDINAL_STEP	= 0x400;
+
 		typedef unsigned long date_t;
 
 		void operator=( const Date &date )
@@ -93,16 +101,20 @@ class Date
 			:	m_date( date ) {}
 									Date( unsigned int y, unsigned int m, unsigned int d,
 										  unsigned int o = 0 )
-			:	m_date( ( y << 19 ) + ( m << 15 ) + ( d << 10 ) + o ) {}
+			:	m_date( ( y << 19 ) | ( m << 15 ) | ( d << 10 ) | o ) {}
+		explicit					Date( const Glib::ustring& );
+		// ORDINAL C'TOR
+									Date( unsigned int o1, unsigned int o2 )
+			:	m_date( ORDINAL_FLAG | ( o1 << 10 ) | o2 ) {}
 
 		date_t						get_pure( void ) const
 		{ return( m_date & 0xFFFFFC00 ); }
 
 		date_t						get_yearmonth( void ) const
-		{ return( m_date & 0xFFFF8000 ); }
+		{ return( m_date & 0x7FFF8000 ); }
 
 		unsigned int				get_year( void ) const
-		{ return ( ( m_date & 0xFFF80000 ) >> 19 ); }
+		{ return ( ( m_date & 0x7FF80000 ) >> 19 ); }
 
 		Glib::ustring				get_year_str( void ) const;
 		Glib::ustring				get_month_str( void ) const;
@@ -120,13 +132,16 @@ class Date
 		unsigned int				get_order( void ) const
 		{ return( m_date & 0x3FF ); }
 
+		unsigned int				get_ordinal_order( void ) const
+		{ return( ( m_date & 0x7FFFFC00) >> 10 ); }
+
 		Glib::Date					get_glib( void ) const
 		{ return Glib::Date( get_day(), get_month_glib(), get_year() ); }
 		time_t						get_ctime( void ) const;
 
 		void						set_year( unsigned int y )
 		{
-			m_date &= 0x74444;
+			m_date &= 0x8007FFFF;
 			m_date |= ( y << 19 );
 		}
 		void						set_month( unsigned int m )
@@ -139,52 +154,34 @@ class Date
 			m_date &= 0xFFFF83FF;
 			m_date |= ( d << 10 );
 		}
-		void						reset_order( void )
-		{
-			m_date &= 0xFFFFFC00;
-		}
+		void						reset_order_0( void )
+		{ m_date &= 0xFFFFFC00; }
+		void						reset_order_1( void )
+		{ m_date |= 0x1; m_date &= 0xFFFFFC01; }
+		
+		static void					reset_order_1( Date::date_t &date )
+		{ date |= 0x1; date &= 0xFFFFFC01; }
 
 		Glib::ustring				format_string( void ) const;
 		static Glib::ustring		format_string( const time_t* );
+		static Glib::ustring		format_string_do( const time_t* );
 
 		bool						is_valid( void ) const
 		{
 			return( Glib::Date::get_days_in_month( get_month_glib(), get_year() ) >= get_day() );
 		}
 
-		void						forward_months( int months )
-		{
-			months += ( ( m_date & 0x78000 ) >> 15 ); // get month
-			m_date &= 0xFFF80000;	// truncate months
-			const int mod_months = months % 12;
-			if( mod_months == 0 )
-			{
-				m_date += make_year( ( months / 12 ) - 1);
-				m_date |= 0x60000;	// make month 12
-			}
-			else
-			{
-				m_date += make_year( months / 12 );
-				m_date |= make_month( mod_months );
-			}
-		}
+		bool						is_ordinal( void ) const
+		{ return( m_date & ORDINAL_FLAG ); }
 
-		void						forward_month( void )
-		{
-			int months = get_month() + 1;
-			m_date &= 0xFFF80000;	// truncate months
-			const int mod_months = months % 12;
-			if( mod_months == 0 )
-			{
-				m_date += make_year( ( months / 12 ) - 1);
-				m_date |= 0x60000;	// make month 12
-			}
-			else
-			{
-				m_date += make_year( months / 12 );
-				m_date |= make_month( mod_months );
-			}
-		}
+		void						forward_months( int months );
+		void						forward_month( void );
+		void						forward_day( void );
+
+		void						backward_ordinal_order( void )
+		{ if( get_ordinal_order() > 0 ) m_date -= ORDINAL_STEP; }
+		void						forward_ordinal_order( void )
+		{ m_date += ORDINAL_STEP; }
 
 		static date_t				make_year( unsigned int y )
 		{ return( y << 19 ); }
