@@ -1,6 +1,6 @@
 /***********************************************************************************
 
-	Copyright (C) 2007-2009 Ahmet Öztürk (aoz_2@yahoo.com)
+	Copyright (C) 2007-2011 Ahmet Öztürk (aoz_2@yahoo.com)
 
 	This file is part of Lifeograph.
 
@@ -38,7 +38,12 @@ using namespace LIFEO;
 Lifeograph::Lifeograph( int argc, char *argv[] )
 :	m_about_dialog( NULL ), m_menu_link( NULL ), m_vbox_login( NULL ), m_hpaned_main( NULL ),
 	m_theme_view( NULL ),
-	m_programpath( argv[ 0 ] ), m_loginstatus( LOGGED_OUT ), m_flag_ignore_locks( false )
+	m_programpath( argv[ 0 ] ), m_loginstatus( LOGGED_OUT ),
+#ifdef LIFEOGRAPH_DEBUG_BUILD
+	m_flag_ignore_locks( true )
+#else
+	m_flag_ignore_locks( false )
+#endif
 {
 	Cipher::init();
 
@@ -90,7 +95,6 @@ Lifeograph::Lifeograph( int argc, char *argv[] )
 		if( ! strcmp( argv[ i ], "--ignore-locks" ) )
 		{
 			m_flag_ignore_locks = true;
-			PRINT_DEBUG( "file locks will be ignored" );
 		}
 		else
 		if( Glib::file_test( argv[ i ], Glib::FILE_TEST_EXISTS ) )
@@ -173,8 +177,11 @@ Lifeograph::on_expose_event( GdkEventExpose *event )
 {
 	if( m_loginstatus != LOGGED_IN && bool( pixbuf_background ) )
 	{
+		Glib::RefPtr< Gdk::GC > gc( get_style()->get_bg_gc( Gtk::STATE_NORMAL ) );
+		Gdk::Rectangle rect( &event->area );
+		gc->set_clip_rectangle( rect );
 		get_window()->draw_pixbuf(
-			get_style()->get_bg_gc( Gtk::STATE_NORMAL ),
+			gc,
 			pixbuf_background,
 			0, 0, 0, 0, -1, -1,
 			Gdk::RGB_DITHER_NONE, 0, 0 );
@@ -246,45 +253,46 @@ Lifeograph::update_login_file( void )
 		return;
 
 	std::string path_lock( path + LOCK_SUFFIX );
-	if( ! m_flag_ignore_locks && Glib::file_test( path_lock, Glib::FILE_TEST_EXISTS ) )
+	if( Glib::file_test( path_lock, Glib::FILE_TEST_EXISTS ) )
 	{
-		m_label_startupmessage->set_text( _( STRING::DIARY_LOCKED ) );
-		return;
+		if( m_flag_ignore_locks )
+			print_info( "ignored file lock" );
+		else
+		{
+			m_label_startupmessage->set_text( _( STRING::DIARY_LOCKED ) );
+			return;
+		}
 	}
 
 	Diary::d->set_path( path );
 
-	Result result = Diary::d->read_header();
-	if( result == SUCCESS )
+	switch( Diary::d->read_header() )
 	{
-		if( Diary::d->is_encrypted() )
-		{
-			m_entry_password->set_sensitive( true );
-			if( m_loginstatus == LOGGED_TIME_OUT )
-				m_label_startupmessage->set_text( _( STRING::ENTER_PASSWORD_TIMEOUT ) );
+		case SUCCESS:
+			if( Diary::d->is_encrypted() )
+			{
+				m_entry_password->set_sensitive( true );
+				if( m_loginstatus == LOGGED_TIME_OUT )
+					m_label_startupmessage->set_text( _( STRING::ENTER_PASSWORD_TIMEOUT ) );
+				else
+					m_label_startupmessage->set_text( _( STRING::ENTER_PASSWORD ) );
+				m_button_opendiary->set_sensitive( m_entry_password->get_text_length() > 0 );
+			}
 			else
-				m_label_startupmessage->set_text( _( STRING::ENTER_PASSWORD ) );
-			m_button_opendiary->set_sensitive( m_entry_password->get_text_length() > 0 );
-		}
-		else
-		{
-			m_label_startupmessage->set_text( _( STRING::DB_IS_NOT_ENCRYPTED ) );
-			m_button_opendiary->set_sensitive( true );
-		}
-	}
-	else
-	if( result == INCOMPATIBLE_FILE )
-	{
-		m_label_startupmessage->set_text( _( STRING::INCOMPATIBLE_DB ) );
-	}
-	else
-	if( result == CORRUPT_FILE )
-	{
-		m_label_startupmessage->set_text( _( STRING::CORRUPT_DB ) );
-	}
-	else
-	{
-		m_label_startupmessage->set_text( _( STRING::FAILED_TO_OPEN_DB ) );
+			{
+				m_label_startupmessage->set_text( _( STRING::DB_IS_NOT_ENCRYPTED ) );
+				m_button_opendiary->set_sensitive( true );
+			}
+			break;
+		case INCOMPATIBLE_FILE:
+			m_label_startupmessage->set_text( _( STRING::INCOMPATIBLE_DB ) );
+			break;
+		case CORRUPT_FILE:
+			m_label_startupmessage->set_text( _( STRING::CORRUPT_DB ) );
+			break;
+		default:
+			m_label_startupmessage->set_text( _( STRING::FAILED_TO_OPEN_DB ) );
+			break;
 	}
 
 	// FOCUS ADJUSTMENT
@@ -339,10 +347,7 @@ Lifeograph::initialize_new_diary( void )
 
 	if( dialogsavediary->run() == Gtk::RESPONSE_ACCEPT )
 	{
-		Diary::d->set_path( dialogsavediary->get_filename() );
-		Diary::d->clear_passphrase();
-		Diary::d->add_today();
-		// TODO: check for validity of file?
+		Diary::d->init_new( dialogsavediary->get_filename() );
 		draw_editscreen();
 	}
 
@@ -394,9 +399,7 @@ Lifeograph::handle_button_opendb_clicked( void )
 
 	switch( Diary::d->read_body() )
 	{
-		case EMPTY_DATABASE:
-			Diary::d->add_today();
-			// continues:
+		case EMPTY_DATABASE:	// no special treatment for now:
 		case SUCCESS:
 			draw_editscreen();
 			break;
@@ -408,8 +411,8 @@ Lifeograph::handle_button_opendb_clicked( void )
 			m_entry_password->set_sensitive( true );
 			m_entry_password->grab_focus();
 			break;
-		case FAILURE:
-			m_label_startupmessage->set_text( _( STRING::INCOMPATIBLE_DB ) );
+		case CORRUPT_FILE:
+			m_label_startupmessage->set_text( _( STRING::CORRUPT_DB ) );
 			Diary::d->clear();	// clear partially read content if any
 			break;
 		default:
@@ -418,60 +421,7 @@ Lifeograph::handle_button_opendb_clicked( void )
 }
 
 void
-Lifeograph::handle_calendar_doubleclicked( void )
-{
-	guint year, month, day;
-
-	m_calendar->get_date( year, month, day );
-
-	Entry *entry = Diary::d->create_entry( Date( year, month + 1, day ) );
-
-	// FIXME: replace these with more direct functions like add_entry()
-	// (after treeview becomes a class of its own)
-	update_entry_list();
-	update_calendar();
-
-	entry->show();
-
-	// this is the only way i could find to give focus to textview
-	//FIXME
-//	Glib::signal_timeout().connect_once(
-//			sigc::mem_fun( m_textviewdiary, &Gtk::Widget::grab_focus ), 200 );
-}
-
-void
-Lifeograph::handle_calendar_dayselected( void )
-{
-	if( m_internaloperation ) return;
-
-	guint year, month, day;
-	m_calendar->get_date( year, month, day );
-	Date date( year, month + 1, day );
-	Entry *entry;
-	if( date.get_pure() == ( m_panel_main->get_cur_elem()->get_date().get_pure() ) )
-	{
-		//date = m_panel_main->get_cur_elem()->get_date();
-		entry = Diary::d->get_entry_nextinday( date );
-	}
-	else
-	{
-		entry = Diary::d->get_entry( date );
-	}
-
-	if( entry )
-		entry->show();
-}
-
-void
-Lifeograph::handle_calendar_monthchanged( void )
-{
-	if( m_internaloperation ) return;
-
-	update_calendar();
-}
-
-void
-Lifeograph::handle_textview_link_activated( Date date )
+Lifeograph::handle_textview_link_activated( const Date &date )
 {
 	Entryvector *entries = Diary::d->get_entries( date.get_pure() );
 
@@ -497,7 +447,7 @@ Lifeograph::handle_textview_link_activated( Date date )
 			Entry *entry = ( *iter );
 			m_menu_link->items().push_back(
 					Gtk::Menu_Helpers::MenuElem(
-							entry->get_title(),
+							entry->get_name(),
 							sigc::mem_fun( entry, &Entry::show ) ) );
 		}
 
@@ -518,12 +468,18 @@ Lifeograph::handle_textview_link_activated( Date date )
 }
 
 void
-Lifeograph::handle_entry_title_changed( DiaryElement *element, const Glib::ustring &title )
+Lifeograph::handle_entry_title_changed( DiaryElement *element )
 {
 	if( m_internaloperation ) return;
 
 	Gtk::TreeRow row = * m_panel_diary->get_row( element->m_list_data->treepath );
-	row[ ListData::colrec->info ] = title;
+	row[ ListData::colrec->info ] = element->get_list_str();
+}
+
+void
+Lifeograph::handle_settings_changed( void )
+{
+	m_entry_view->update_formatting_toolbar();
 }
 
 void
@@ -545,6 +501,7 @@ Lifeograph::finish_editing( bool opt_save )
 	m_settings.position_pane = m_hpaned_main->get_position();
 	m_settings.position_pane_tags = m_hpaned_entry->get_position();
 	sync_entry();
+	Diary::d->m_last_elem = get_cur_elem()->get_id();
 
 	if( opt_save )
 	{
@@ -649,35 +606,42 @@ Lifeograph::sync_entry( void )
 	m_entry_view->sync();
 }
 
-void
-Lifeograph::dismiss_entry( Entry *entry )
+bool
+Lifeograph::confirm_dismiss_element( void )
 {
-	m_panel_main->remove_element_from_history( entry );
+	Gtk::MessageDialog message( *Lifeobase::base, "", false,
+							    Gtk::MESSAGE_WARNING, Gtk::BUTTONS_NONE, true );
+	message.set_message( _( "Are you sure, you want to dismiss?" ) );
+	message.set_secondary_text( _( "This operation cannot be undone!" ) );
+	message.add_button( Gtk::Stock::CANCEL, Gtk::RESPONSE_CANCEL );
+	message.add_button( _( "_DISMISS!" ), Gtk::RESPONSE_ACCEPT );
 
-	Diary::d->dismiss_entry( entry );
-
-	m_internaloperation++;
-
-	update_calendar();
-	update_entry_list();
-
-	//Diary::d->show();
-
-	m_internaloperation--;
+	if( message.run() != Gtk::RESPONSE_ACCEPT )
+		return false;
+	else
+		return true;
 }
 
 void
+Lifeograph::remove_element_from_history( DiaryElement *element )
+{
+	m_panel_main->remove_element_from_history( element );
+}
+
+bool
 Lifeograph::change_current_element( DiaryElement *element, Gtk::Widget *widget )
 {
+	if( m_panel_main->get_cur_elem() == element )
+		return false;
+
 	m_internaloperation++;
 
 	m_panel_main->show( element, widget );
-	if( element->get_type() >= DiaryElement::IT_DIARY )
-		m_panel_diary->m_treeview_entries->present_row( element->m_list_data->treepath );
-	else
-		m_panel_diary->m_treeview_entries->get_selection()->unselect_all();
+	m_panel_diary->show( element );
 
 	m_internaloperation--;
+
+	return true;
 }
 
 DiaryElement*
@@ -692,6 +656,12 @@ Lifeograph::get_cur_elem_type( void )
 	return m_panel_main->get_cur_elem_type();
 }
 
+DiaryElement*
+Lifeograph::get_dragged_elem( void )
+{
+	return m_panel_diary->m_treeview_entries->get_dragged_elem();
+}
+
 Gtk::TreeRow
 Lifeograph::get_element_row( DiaryElement *element )
 {
@@ -715,9 +685,9 @@ Lifeograph::update_entry_list( void )
 }
 
 void
-Lifeograph::update_tag_list( const Glib::ustring &filter )
+Lifeograph::update_tag_list( void )
 {
-	m_panel_tag->populate_main( filter );
+	m_panel_tag->populate_main();
 }
 
 void
@@ -767,34 +737,7 @@ Lifeograph::update_toolbar()
 void
 Lifeograph::update_calendar( void )
 {
-	guint year, month, day;
-
-	m_calendar->get_date( year, month, day );
-	month++;	// fix month
-	Entry *entry;
-	m_calendar->clear_marks();
-
-	for(	Entryiter itr_entry = Diary::d->get_entries().begin();
-			itr_entry != Diary::d->get_entries().end();
-			++itr_entry )
-	{
-		entry = itr_entry->second;
-		if( entry->get_filtered_out() ) continue;
-
-		if( entry->get_date().get_year() == year )
-		{
-			if( entry->get_date().get_month() == month )
-			{
-				m_calendar->mark_day( entry->get_date().get_day() );
-			}
-			else
-			if( entry->get_date().get_month() < month )
-				break;
-		}
-		else
-		if( entry->get_date().get_year() < year )
-			break;
-	}
+	m_panel_diary->update_calendar();
 }
 
 void
@@ -841,19 +784,7 @@ Lifeograph::focus_tag( void )
 void
 Lifeograph::select_date_in_calendar( const Date &date )
 {
-	++m_internaloperation;
-
-	// CALENDAR
-	// the current day has to be deselected first to avoid gtk's complaints
-	m_calendar->select_day( 0 );
-	m_calendar->select_month( date.get_month() - 1, date.get_year() );
-	update_calendar();
-	if( date.get_day() )
-	{
-		m_calendar->select_day( date.get_day() );
-	}
-
-	--m_internaloperation;
+	m_panel_diary->select_date_in_calendar( date );
 }
 
 // MODE DRAWING FUNCTIONS
@@ -1036,7 +967,6 @@ Lifeograph::draw_editscreen( void )
 
 			// TIME BASED NAVIGATION PANE
 			Lifeobase::builder->get_widget( "toolbar_diary_prefs", toolbar_diary_prefs );
-			Lifeobase::builder->get_widget( "calendar_main", m_calendar );
 
 			Lifeobase::builder->get_widget( "hpaned_second", m_hpaned_entry );
 			Lifeobase::builder->get_widget( "label_gui_elem", m_label_gui_elem );
@@ -1119,13 +1049,6 @@ Lifeograph::draw_editscreen( void )
 		m_toolbutton_logout->signal_clicked().connect(
 				sigc::bind( sigc::mem_fun( *this, &Lifeograph::logout ), true ) );
 
-		m_calendar->signal_day_selected_double_click().connect(
-				sigc::mem_fun( *this, &Lifeograph::handle_calendar_doubleclicked ) );
-		m_calendar->signal_month_changed().connect(
-				sigc::mem_fun( *this, &Lifeograph::handle_calendar_monthchanged ) );
-		m_calendar->signal_day_selected().connect(
-				sigc::mem_fun( *this, &Lifeograph::handle_calendar_dayselected ) );
-
 		LinkEntry::signal_activated().connect(
 				sigc::mem_fun( *this, &Lifeograph::handle_textview_link_activated ) );
 	}
@@ -1151,8 +1074,8 @@ Lifeograph::draw_editscreen( void )
 	m_panel_tag->m_button_remove_filter->set_visible( false );
 	m_widget_panel_exp->populate_themes();
 	m_tag_view->update_combobox_categories();
-	m_diary_view->update_combobox_chapter_ctg();
-	m_entry_view->m_textviewdiary->m_buffer->handle_login();
+	m_diary_view->handle_login();
+	m_entry_view->handle_login();
 
 	m_internaloperation--;
 	m_loginstatus = LOGGED_IN;
@@ -1229,6 +1152,13 @@ Lifeograph::create_actions( void )
 					"", "",		// no grachical interface yet!
 					Gtk::AccelKey( GDK_j, Gdk::CONTROL_MASK ),
 					sigc::mem_fun( m_panel_diary, &PanelDiary::present_current_row ) );
+	
+	create_action(	m_action_show_prev_session_elem,
+					"PrevSessElem",
+					Gtk::Stock::JUMP_TO,
+					"", "",		// no grachical interface yet!
+					Gtk::AccelKey( GDK_l, Gdk::CONTROL_MASK ),
+					sigc::mem_fun( m_panel_diary, &PanelDiary::show_prev_session_elem ) );
 
 	create_action(	m_action_focusfilter,
 					"FocusFilter",
