@@ -1,6 +1,6 @@
 /***********************************************************************************
 
-	Copyright (C) 2007-2010 Ahmet Öztürk (aoz_2@yahoo.com)
+	Copyright (C) 2007-2011 Ahmet Öztürk (aoz_2@yahoo.com)
 
     This file is part of Lifeograph.
 
@@ -122,7 +122,7 @@ class Lifeograph : public Lifeobase
 		void				update_login_file( void );
 		void				update_indices( void );
 		void				update_entry_list();
-		void				update_tag_list( const Glib::ustring& );
+		void				update_tag_list( void );
 		void				update_tag_categories( void );
 		void				update_filter_tag( void );
 		void				update_tag_icon( Tag* );
@@ -133,10 +133,12 @@ class Lifeograph : public Lifeobase
 		void				update_toolbar( void );
 
 		void				sync_entry( void );
-		void				dismiss_entry( Entry* );
-		void				change_current_element( DiaryElement*, Gtk::Widget* );
+		bool				confirm_dismiss_element( void );
+		void				remove_element_from_history( DiaryElement* );
+		bool				change_current_element( DiaryElement*, Gtk::Widget* );
 		DiaryElement*		get_cur_elem( void );
 		DiaryElement::Type	get_cur_elem_type( void );
+		DiaryElement*		get_dragged_elem( void );
 		Gtk::TreeRow		get_element_row( DiaryElement* );
 
 		void				focus_filter( void );
@@ -156,12 +158,9 @@ class Lifeograph : public Lifeobase
 		void				handle_button_quit_clicked( void );
 
 		void				handle_textview_entry_changed( void );
-		void				handle_textview_link_activated( Date );
-		void				handle_entry_title_changed( DiaryElement*, const Glib::ustring& );
-
-		void				handle_calendar_doubleclicked( void );
-		void				handle_calendar_monthchanged( void );
-		void				handle_calendar_dayselected( void );
+		void				handle_textview_link_activated( const Date& );
+		void				handle_entry_title_changed( DiaryElement* );
+		void				handle_settings_changed( void );
 
 		void				select_date_in_calendar( const Date& );
 
@@ -183,7 +182,6 @@ class Lifeograph : public Lifeobase
 
 		Menutoolbutton				*m_menubutton_list;
 
-		Gtk::Calendar				*m_calendar;
 		Gtk::Menu					*m_menu_link;
 		PanelMain					*m_panel_main;
 		PanelDiary					*m_panel_diary;
@@ -216,6 +214,7 @@ class Lifeograph : public Lifeobase
 		Glib::RefPtr< Gtk::Action >	m_action_next_match;
 		Glib::RefPtr< Gtk::Action >	m_action_today;
 		Glib::RefPtr< Gtk::Action >	m_action_jump2current;
+		Glib::RefPtr< Gtk::Action >	m_action_show_prev_session_elem;
 		Glib::RefPtr< Gtk::Action >	m_action_focusfilter;
 		Glib::RefPtr< Gtk::Action >	m_action_focustag;
 
