@@ -1,6 +1,6 @@
 /***********************************************************************************
 
-	Copyright (C) 2007-2010 Ahmet Öztürk (aoz_2@yahoo.com)
+	Copyright (C) 2007-2011 Ahmet Öztürk (aoz_2@yahoo.com)
 
 	This file is part of Lifeograph.
 
@@ -38,7 +38,7 @@ PanelDiary::PanelDiary( void )
 {
 	try
 	{
-		//Lifeobase::builder->get_widget( "calendar_main", m_calendar );	// TODO
+		Lifeobase::builder->get_widget( "calendar_main", m_calendar );
 		Lifeobase::builder->get_widget_derived( "treeview_main", m_treeview_entries );
 		Lifeobase::builder->get_widget_derived( "entry_filter", m_entry_filter );
 		Lifeobase::builder->get_widget_derived( "entry_replace", m_entry_replace );
@@ -59,6 +59,15 @@ PanelDiary::PanelDiary( void )
 	m_entry_replace->set_idle_text( _( "Replace" ) );
 
 	// SIGNALS
+	m_calendar->signal_day_selected_double_click().connect(
+			sigc::mem_fun( this, &PanelDiary::handle_calendar_doubleclicked ) );
+	m_calendar->signal_month_changed().connect(
+			sigc::mem_fun( this, &PanelDiary::handle_calendar_monthchanged ) );
+	m_calendar->signal_day_selected().connect(
+			sigc::mem_fun( this, &PanelDiary::handle_calendar_dayselected ) );
+	m_calendar->signal_button_press_event().connect_notify(
+			sigc::mem_fun( this, &PanelDiary::handle_calendar_button_pressed ) );
+
 	m_entry_filter->signal_changed().connect(
 			sigc::mem_fun( this, &PanelDiary::handle_entry_filter_changed ) );
 	m_entry_filter->signal_activate().connect(
@@ -90,15 +99,69 @@ PanelDiary::PanelDiary( void )
 			sigc::mem_fun( this, &PanelDiary::handle_filter_favorites_toggled ) );
 }
 
+void
+PanelDiary::add_chapter_category_to_list( const CategoryChapters *chapters )
+{
+	Gtk::TreeRow	row_container;
+	Chapter			*chapter = NULL;
+
+	for( CategoryChapters::const_iterator iter_chapter = chapters->begin();
+		 iter_chapter != chapters->end();
+		 ++iter_chapter )
+	{
+		chapter = iter_chapter->second;
+		if( ! chapter->is_initialized() )
+			continue;
+
+		// show all folders even when they are empty:
+		row_container = * m_treeview_entries->m_treestore_entries->append( l_row_diary.children() );
+		row_container[ ListData::colrec->ptr ] = chapter;	// ptr must be first:
+		row_container[ ListData::colrec->info ] = chapter->get_list_str();
+		row_container[ ListData::colrec->icon ] = chapter->get_icon();
+		chapter->m_list_data->treepath = m_treeview_entries->m_treestore_entries->get_path( row_container );
+
+		chapter->reset_size();
+
+		if( l_entry == NULL )
+			continue;
+
+		while( l_entry->get_date() >= chapter->get_date() )
+		{
+			chapter->increase_size();
+			if( l_entry->get_filtered_out() == false )
+			{
+				Lifeobase::m_entrycount++;
+				l_row_entry = * m_treeview_entries->m_treestore_entries->append( row_container.children() );
+				l_row_entry[ ListData::colrec->ptr ] = l_entry;
+				l_row_entry[ ListData::colrec->info ] = l_entry->get_list_str();
+				l_row_entry[ ListData::colrec->icon ] = l_entry->get_icon();
+
+				l_entry->m_list_data->treepath = m_treeview_entries->m_treestore_entries->get_path( l_row_entry );
+			}
+
+			++l_itr_entry;
+			if( l_itr_entry != Diary::d->m_entries.end() )
+				l_entry = l_itr_entry->second;
+			else
+			{
+				l_entry = NULL;
+				break;
+			}
+		}
+
+		if( chapter->get_expanded() )
+			m_treeview_entries->expand_element( chapter );
+	}
+}
+
 void
 PanelDiary::update_entry_list( void )
 {
+	PRINT_DEBUG( "update_entry_list()" );
 	Lifeobase::m_internaloperation++;
 
-	Gtk::TreeRow	row_entry;
-	Entry			*entry;
-	Entryiter		iter_entry = Diary::d->m_entries.begin();
-	Chapter			*chapter = NULL;
+	l_itr_entry = Diary::d->m_entries.begin();
+	l_entry = ( l_itr_entry != Diary::d->m_entries.end() ? l_itr_entry->second : NULL );
 
 	Lifeobase::base->sync_entry();
 
@@ -106,67 +169,45 @@ PanelDiary::update_entry_list( void )
 	Lifeobase::m_entrycount = 0;
 
 	// DIARY ITEM
-	Gtk::TreeRow row_diary = * m_treeview_entries->m_treestore_entries->append();
-	row_diary[ ListData::colrec->ptr ] = Diary::d; // ptr must be first
-	row_diary[ ListData::colrec->id ] =  Glib::ustring::compose( "<b>%1</b>", _( "Diary" ) );
-	row_diary[ ListData::colrec->info ] = Diary::d->get_name();
-	row_diary[ ListData::colrec->icon ] = Lifeobase::icons->diary_16;
+	l_row_diary = * m_treeview_entries->m_treestore_entries->append();
+	l_row_diary[ ListData::colrec->ptr ] = Diary::d; // ptr must be first
+	l_row_diary[ ListData::colrec->info ] =
+			Glib::ustring::compose( "<b>%1</b>", Diary::d->get_name() );
+	l_row_diary[ ListData::colrec->icon ] = Lifeobase::icons->diary_16;
 
 	Diary::d->m_list_data->treepath =
-			m_treeview_entries->m_treestore_entries->get_path( row_diary );
-
+			m_treeview_entries->m_treestore_entries->get_path( l_row_diary );
 
 	// CHAPTERS
-	CategoryChapters *chapters = Diary::d->get_current_chapter_ctg();
+	//CategoryChapters *chapters = Diary::d->get_current_chapter_ctg();
 
-	if( Diary::d->get_sorting_criteria() == SC_DATE && chapters )
+	if( Diary::d->get_sorting_criteria() == SC_DATE )
 	{
-		DiaryElement *element( NULL ), *element_ret;
-
-		for( CategoryChapters::const_iterator iter_chapter = chapters->begin();
-			 iter_chapter != chapters->end();
-			 ++iter_chapter )
-		{
-			chapter = * iter_chapter;
-			element_ret = chapter->add_to_list(
-					m_treeview_entries->m_treestore_entries, row_diary );
-			if( element_ret )	// skip empty folders
-			{
-				element = element_ret;
-				if( chapter->get_expanded() )
-					m_treeview_entries->expand_element( chapter );
-			}
-		}
-		if( element )
-		{
-			iter_entry = Diary::d->m_entries.find( element->get_date().m_date );
-			if( iter_entry != Diary::d->m_entries.end() )
-				++iter_entry;
-		}
+		add_chapter_category_to_list( Diary::d->get_topics() );
+		add_chapter_category_to_list( Diary::d->get_current_chapter_ctg() );
 	}
 
 	for( ;
-		 iter_entry != Diary::d->m_entries.end();
-		 ++iter_entry )
+		 l_itr_entry != Diary::d->m_entries.end();
+		 ++l_itr_entry )
 	{
-		entry = ( *iter_entry ).second;
-		if( entry->get_filtered_out() == false )
+		l_entry = ( *l_itr_entry ).second;
+		if( l_entry->get_filtered_out() == false )
 		{
-			//if( entry->get_date() >= ( *iter_chapter )->get_date() )
-
 			Lifeobase::m_entrycount++;
 
-			row_entry = *m_treeview_entries->m_treestore_entries->append(
-					row_diary.children() );
-			row_entry[ ListData::colrec->ptr ] = entry; // ptr must be first
-			row_entry[ ListData::colrec->id ] = entry->get_1st_str();
-			row_entry[ ListData::colrec->info ] = entry->get_2nd_str();
-			row_entry[ ListData::colrec->icon ] = entry->get_icon();
+			l_row_entry = *m_treeview_entries->m_treestore_entries->append(
+					l_row_diary.children() );
+			l_row_entry[ ListData::colrec->ptr ] = l_entry; // ptr must be first
+			l_row_entry[ ListData::colrec->info ] = l_entry->get_list_str();
+			l_row_entry[ ListData::colrec->icon ] = l_entry->get_icon();
 
-			entry->m_list_data->treepath =
-					m_treeview_entries->m_treestore_entries->get_path( row_entry );
+			l_entry->m_list_data->treepath =
+					m_treeview_entries->m_treestore_entries->get_path( l_row_entry );
 		}
 	}
+	// always expand diary
+	m_treeview_entries->expand_element( Diary::d );
 
 	// UPDATE DIARY ELEM IF LAST ONE IS NOT IN THE LIST ANYMORE
 	DiaryElement *elem = Lifeobase::base->get_cur_elem();
@@ -182,19 +223,53 @@ PanelDiary::update_entry_list( void )
 			flag_show_elem = false;
 	}
 	else	// the case at the start-up
-		elem = Diary::d->get_entry_first();
+		elem = Diary::d->get_startup_elem();
 
 	if( flag_show_elem )
 		elem->show();
-	else
+	/*else
 	if( elem->get_type() >= DiaryElement::IT_DIARY )	// only entry list elements
-		m_treeview_entries->present_row( elem->m_list_data->treepath );
+		m_treeview_entries->present_row( elem->m_list_data->treepath );*/
 
 	Diary::d->set_filtering_status_applied();
 
 	Lifeobase::m_internaloperation--;
 }
 
+void
+PanelDiary::update_calendar( void )
+{
+	guint year, month, day;
+
+	m_calendar->get_date( year, month, day );
+	month++;	// fix month
+	Entry *entry;
+	m_calendar->clear_marks();
+
+	for( Entryiter itr_entry = Diary::d->get_entries().begin();
+		 itr_entry != Diary::d->get_entries().end();
+		 ++itr_entry )
+	{
+		entry = itr_entry->second;
+		if( entry->get_filtered_out() || entry->get_date().is_ordinal() )
+			continue;
+
+		if( entry->get_date().get_year() == year )
+		{
+			if( entry->get_date().get_month() == month )
+			{
+				m_calendar->mark_day( entry->get_date().get_day() );
+			}
+			else
+			if( entry->get_date().get_month() < month )
+				break;
+		}
+		else
+		if( entry->get_date().get_year() < year )
+			break;
+	}
+}
+
 void
 PanelDiary::handle_login( void )
 {
@@ -217,6 +292,30 @@ PanelDiary::handle_logout( void )
 	m_treeview_entries->m_treestore_entries->clear();
 }
 
+void
+PanelDiary::show( const DiaryElement *element )
+{
+	if( element->get_type() >= DiaryElement::IT_DIARY )
+		m_treeview_entries->present_element( element );
+	else
+		m_treeview_entries->get_selection()->unselect_all();
+
+	if( element->get_type() > DiaryElement::IT_DIARY )
+		select_date_in_calendar( element->get_date() );
+	else
+		m_calendar->select_day( 0 );
+}
+
+void
+PanelDiary::show_prev_session_elem( void )
+{
+	DiaryElement *elem( Diary::d->get_prev_session_elem() );
+	if( elem != NULL )
+		elem->show();
+	else
+		print_info( "Previous sesion element cannot be found!" );
+}
+
 void
 PanelDiary::present_current_row( void )
 {
@@ -225,11 +324,7 @@ PanelDiary::present_current_row( void )
 		return;
 	Gtk::TreePath path = m_treeview_entries->m_treemodelsort_entries->get_path( iter );
 
-	Lifeobase::m_internaloperation++;
-
-	m_treeview_entries->present_row( path );
-
-	Lifeobase::m_internaloperation--;
+	m_treeview_entries->scroll_to_row( path );
 }
 
 void
@@ -265,8 +360,7 @@ PanelDiary::handle_entry_filter_changed( void )
 		m_entry_filter->set_tooltip_text( _( "Enter text to be filtered" ) );
 	}
 
-	// TODO:
-	// update_calendar();
+	update_calendar();
 
 	if( Lifeobase::m_entrycount < 1 || filterstring.size() < 1 )
 	{
@@ -396,6 +490,131 @@ PanelDiary::go_down( bool flag_entry_operation )
 	listitem->show();
 }
 
+void
+PanelDiary::handle_calendar_doubleclicked( void )
+{
+	guint year, month, day;
+
+	m_calendar->get_date( year, month, day );
+
+	Entry *entry = Diary::d->create_entry( Date( year, month + 1, day ) );
+
+	// FIXME: replace these with more direct functions like add_entry()
+	// (after treeview becomes a class of its own)
+	update_entry_list();
+	update_calendar();
+
+	entry->show();
+
+	// this is the only way i could find to give focus to textview
+	//FIXME
+//	Glib::signal_timeout().connect_once(
+//			sigc::mem_fun( m_textviewdiary, &Gtk::Widget::grab_focus ), 200 );
+}
+
+void
+PanelDiary::handle_calendar_dayselected( void )
+{
+	if( Lifeobase::m_internaloperation ) return;
+
+	guint year, month, day;
+	m_calendar->get_date( year, month, day );
+	Date date( year, month + 1, day );
+	Entry *entry;
+	if( date.get_pure() == ( Lifeobase::base->get_cur_elem()->get_date().get_pure() ) )
+	{
+		//date = m_panel_main->get_cur_elem()->get_date();
+		entry = Diary::d->get_entry_nextinday( date );
+	}
+	else
+	{
+		entry = Diary::d->get_entry( date );
+	}
+
+	if( entry )
+		entry->show();
+}
+
+void
+PanelDiary::handle_calendar_monthchanged( void )
+{
+	if( Lifeobase::m_internaloperation ) return;
+
+	update_calendar();
+}
+
+void
+PanelDiary::create_chapter( const Date &date )
+{
+	// CAUTION: nohing is checked here as everything should have been checked before
+
+	Chapter *c = Diary::d->get_current_chapter_ctg()->create_chapter(
+			_( "Untitled chapter" ), date );
+
+	Lifeobase::base->update_entry_list();
+	c->show();
+}
+
+void
+PanelDiary::handle_calendar_button_pressed( GdkEventButton *event )
+{
+	if( event->button != 3 )
+		return;
+
+	static Gtk::Menu *m_menu( NULL );
+
+	if( m_menu )
+	{
+		delete m_menu;
+		m_menu = NULL;
+	}
+
+	m_menu = new Gtk::Menu;
+
+	m_menu->items().push_back(
+			Gtk::Menu_Helpers::MenuElem( _( "Create an _Entry" ),
+			sigc::mem_fun( this, &PanelDiary::handle_calendar_doubleclicked ) ) );
+
+	CategoryChapters *cc = Diary::d->get_current_chapter_ctg();
+	if( cc != NULL )
+	{
+		guint year, month, day;
+		m_calendar->get_date( year, month, day );
+		Date date_selected( year, month + 1, day );
+		if( cc->get_chapter( date_selected ) == NULL )
+		{
+			m_menu->items().push_back(
+					Gtk::Menu_Helpers::MenuElem( _( "Create a _Chapter" ),
+					sigc::bind( sigc::mem_fun( this, &PanelDiary::create_chapter ),
+								date_selected ) ) );
+		}
+	}
+
+	m_menu->accelerate( *m_calendar );
+	m_menu->popup( event->button, event->time );
+
+	return;
+}
+
+void
+PanelDiary::select_date_in_calendar( const Date &date )
+{
+	++Lifeobase::m_internaloperation;
+
+	// CALENDAR
+	// the current day has to be deselected first to avoid gtk's complaints
+	m_calendar->select_day( 0 );
+	if( ! date.is_ordinal() )
+	{
+		m_calendar->select_month( date.get_month() - 1, date.get_year() );
+		update_calendar();
+		if( date.get_day() )
+			m_calendar->select_day( date.get_day() );
+	}
+
+	--Lifeobase::m_internaloperation;
+}
+
 void
 PanelDiary::go_prev_match( void )
 {
