@@ -38,8 +38,17 @@ class PanelDiary
 								PanelDiary( void );
 
 		void					update_entry_list( void );
+		void					update_calendar( void );
 
+		void					show( const DiaryElement* );
 		void					present_current_row( void );
+		void					show_prev_session_elem( void );
+
+		void					handle_calendar_doubleclicked( void );
+		void					handle_calendar_monthchanged( void );
+		void					handle_calendar_dayselected( void );
+		void					handle_calendar_button_pressed( GdkEventButton* );
+		void					select_date_in_calendar( const Date& );
 
 		void					handle_filter_favorites_toggled( void );
 		void					handle_login( void );
@@ -62,9 +71,11 @@ class PanelDiary
 															  const Gtk::TreePath& );
 
 	protected:
+		void					create_chapter( const Date& );
+
 		WidgetEntryList			*m_treeview_entries;
 
-		//Gtk::Calendar			*m_calendar;	// TODO
+		Gtk::Calendar			*m_calendar;
 		EntryIdletext			*m_entry_filter;
 		EntryIdletext			*m_entry_replace;
 		Gtk::ToolButton			*m_button_match_prev;
@@ -77,6 +88,13 @@ class PanelDiary
 		Gtk::RadioMenuItem		*m_menuitem_sort_by_size;
 		Gtk::CheckMenuItem		*m_menuitem_filter_favorites;
 
+		// LISTING TMP VARIABLES
+		void					add_chapter_category_to_list( const CategoryChapters* );
+		Entryiter				l_itr_entry;
+		Gtk::TreeRow			l_row_diary;
+		Gtk::TreeRow			l_row_entry;
+		Entry					*l_entry;
+
 	friend class Lifeograph;
 };
 
