@@ -30,7 +30,7 @@ using namespace LIFEO;
 
 
 Settings::Settings( void )
-:	autologout( true ), idletime( IDLETIME_DEFAULT ),
+:	autologout( true ), idletime( IDLETIME_DEFAULT ), show_formatting_toolbar( true ),
 	width( WIDTH_DEFAULT ), height( HEIGHT_DEFAULT ),
 	position_x( POSITION_NOTSET ), position_y( POSITION_NOTSET ),
 	position_pane( PANEPOS_DEFAULT ), position_pane_tags( PANEPOS_TAGS_DEFAULT )
@@ -77,10 +77,10 @@ Settings::read( void )
 			case 'Y':
 				position_y = convert_string( line.substr( 2 ) );
 				break;
-			case 'P':
+			case 'P':	// uppercase
 				position_pane = convert_string( line.substr( 2 ) );
 				break;
-			case 'p':
+			case 'p':	// lowercase
 				position_pane_tags =  convert_string( line.substr( 2 ) );
 				break;
 			case 'L':
@@ -94,6 +94,9 @@ Settings::read( void )
 			case 'B':
 				background = line.substr( 2 );
 				break;
+			case 'F':
+				show_formatting_toolbar = ( line[ 2 ] == 'y' || line[ 2 ] == 'Y' );
+				break;
 
 			case 0:		// empty line
 			case '#':	// comment
@@ -126,13 +129,14 @@ Settings::write( void )
 	file << "\np " << position_pane_tags;
 
 	// RECENT FILES
-	for(	std::list< std::string >::iterator iter = recentfiles.begin();
-			iter != recentfiles.end(); ++iter )
+	for( std::list< std::string >::iterator iter = recentfiles.begin();
+		 iter != recentfiles.end(); ++iter )
 		file << "\nR " << *iter;
 
 	file << "\nL " << ( autologout ? 'y' : 'n' );
 	file << "\nI " << idletime;
 	file << "\nB " << background;
+	file << "\nF " << ( show_formatting_toolbar ? 'y' : 'n' );
 
 	file.close();
 	return true;
