@@ -1,6 +1,6 @@
 /***********************************************************************************
 
-	Copyright (C) 2007-2010 Ahmet Öztürk (aoz_2@yahoo.com)
+	Copyright (C) 2007-2011 Ahmet Öztürk (aoz_2@yahoo.com)
 
 	This file is part of Lifeograph.
 
@@ -41,14 +41,19 @@ EntryView::EntryView( void )
 	try
 	{
 		Lifeobase::builder->get_widget( "vbox_entry", m_vbox_entry );
-		Lifeobase::builder->get_widget( "togglebutton_entry_details", m_button_toggle_details );
 		Lifeobase::builder->get_widget( "togglebutton_entry_favorite", m_button_toggle_favorite );
 		Lifeobase::builder->get_widget( "button_entry_reset_theme", m_button_reset_theme );
 		Lifeobase::builder->get_widget( "button_entry_dismiss", m_button_dismiss );
+		Lifeobase::builder->get_widget( "button_entry_bold", m_button_bold );
+		Lifeobase::builder->get_widget( "button_entry_italic", m_button_italic );
+		Lifeobase::builder->get_widget( "button_entry_strikethrough", m_button_strikethrough );
+		Lifeobase::builder->get_widget( "button_entry_highlight", m_button_highlight );
+		Lifeobase::builder->get_widget( "label_entry_highlight", m_label_highlight );
+		Lifeobase::builder->get_widget( "evntb_entry_highlight", m_eventbox_highlight );
 		Lifeobase::builder->get_widget( "button_entry_add_bullet", m_button_add_bullet );
 		Lifeobase::builder->get_widget( "button_entry_add_checkbox", m_button_add_checkbox );
 		Lifeobase::builder->get_widget_derived( "textview_entry", m_textviewdiary );
-		Lifeobase::builder->get_widget_derived( "entry_tag_filter", m_entry_tag );
+		Lifeobase::builder->get_widget_derived( "entry_tag_filter", m_tag_widget );
 		Lifeobase::builder->get_widget( "button_tag_operation", m_button_tag_operation );
 		Lifeobase::builder->get_widget( "iconview_current_tags", m_iconview_tags );
 		Lifeobase::builder->get_widget( "alignment_entry_toolbar", m_alignment_toolbar );
@@ -57,7 +62,9 @@ EntryView::EntryView( void )
 	catch( ... ) {}
 
 	// TEXT VIEW
-	m_textviewdiary->drag_dest_set_target_list( Lifeobase::base->drag_target_list );
+	m_textviewdiary->drag_dest_get_target_list()->add( Lifeobase::base->drag_targets_tag );
+	m_textviewdiary->drag_dest_get_target_list()->add( Lifeobase::base->drag_targets_theme );
+	m_textviewdiary->drag_dest_get_target_list()->add( Lifeobase::base->drag_targets_entry );
 
 	// TAG LIST
 	m_liststore_tags = Gtk::ListStore::create( *TagPanel::colrec );
@@ -72,15 +79,13 @@ EntryView::EntryView( void )
 	m_iconview_tags->modify_base(
 			Gtk::STATE_NORMAL, Lifeobase::base->get_style()->get_bg( Gtk::STATE_NORMAL ) );
 
-	// TAG WIDGET
-	m_entry_tag->set_idle_text( _( "Tag name" ) );
+	// TOOLBAR
+	m_hbox_editing_toolbar->set_visible( Lifeobase::m_settings.show_formatting_toolbar );
 
 	// SIGNALS
 	TextviewDiary::m_buffer->signal_changed().connect(
 			sigc::mem_fun( this, &LIFEO::EntryView::handle_textview_changed ) );
 
-	m_button_toggle_details->signal_toggled().connect(
-			sigc::mem_fun( this, &LIFEO::EntryView::toggle_details ) );
 	m_button_dismiss->signal_clicked().connect(
 			sigc::mem_fun( this, &LIFEO::EntryView::dismiss_entry ) );
 	m_button_reset_theme->signal_clicked().connect(
@@ -88,19 +93,26 @@ EntryView::EntryView( void )
 	m_button_toggle_favorite->signal_toggled().connect(
 			sigc::mem_fun( this, &LIFEO::EntryView::toggle_favoredness ) );
 
+	m_button_bold->signal_clicked().connect(
+			sigc::mem_fun( m_textviewdiary->m_buffer, &TextbufferDiary::toggle_bold ) );
+	m_button_italic->signal_clicked().connect(
+			sigc::mem_fun( m_textviewdiary->m_buffer, &TextbufferDiary::toggle_italic ) );
+	m_button_strikethrough->signal_clicked().connect(
+			sigc::mem_fun( m_textviewdiary->m_buffer, &TextbufferDiary::toggle_strikethrough ) );
+	m_button_highlight->signal_clicked().connect(
+			sigc::mem_fun( m_textviewdiary->m_buffer, &TextbufferDiary::toggle_highlight ) );
+
 	m_button_add_bullet->signal_clicked().connect(
-			sigc::mem_fun( m_textviewdiary->m_buffer,
-			&TextbufferDiary::add_bullet ) );
+			sigc::mem_fun( m_textviewdiary->m_buffer, &TextbufferDiary::add_bullet ) );
 	m_button_add_checkbox->signal_clicked().connect(
-			sigc::mem_fun( m_textviewdiary->m_buffer,
-			&TextbufferDiary::add_checkbox ) );
+			sigc::mem_fun( m_textviewdiary->m_buffer, &TextbufferDiary::add_checkbox ) );
 
 	m_textviewdiary->signal_drag_data_received().connect(
 			sigc::mem_fun( this, &EntryView::handle_drag_data_received ) );
 
-	m_entry_tag->signal_changed().connect(
+	m_tag_widget->signal_changed().connect(
 			sigc::mem_fun( this, &EntryView::handle_entry_tag_changed ) );
-	m_entry_tag->signal_activate().connect(
+	m_tag_widget->signal_activate().connect(
 			sigc::mem_fun( this, &EntryView::handle_button_tag_clicked ) );
 	m_button_tag_operation->signal_clicked().connect(
 			sigc::mem_fun( this, &EntryView::handle_button_tag_clicked ) );
@@ -165,7 +177,7 @@ void
 EntryView::focus_tag( void )
 {
 	//m_hbox_tagtools->show();
-	m_entry_tag->grab_focus();
+	m_tag_widget->grab_focus();
 }
 
 void
@@ -195,6 +207,13 @@ EntryView::handle_hboxtagtools_enternotify( GdkEventCrossing *event )
 	return false;
 }*/
 
+void
+EntryView::handle_login( void )
+{
+	m_textviewdiary->m_buffer->handle_login();
+	m_tag_widget->populate();
+}
+
 void
 EntryView::handle_textview_changed( void )
 {
@@ -235,11 +254,18 @@ EntryView::handle_drag_data_received(
 		}
 		case DRAG_TARGET_THEME_INFO:
 		{
-			Theme *theme = ( name == Theme::system_theme.get_name() ) ?
-					&Theme::system_theme : Diary::d->m_themes.find( name )->second;
+			Theme *theme = Diary::d->m_themes.find( name )->second;
 			m_ptr2entry->set_theme( theme );
 			TextviewDiary::m_buffer->set_theme( theme );
 			m_button_reset_theme->set_visible( true );
+			update_highlight_button();
+			break;
+		}
+		case DRAG_TARGET_ENTRY_INFO:
+		{
+			DiaryElement *element = Lifeobase::base->get_dragged_elem();
+			if( element != NULL )
+				TextviewDiary::m_buffer->insert_link( element );
 			break;
 		}
 	}
@@ -259,31 +285,34 @@ EntryView::toggle_favoredness( void )
 void
 EntryView::reset_theme( void )
 {
-	m_ptr2entry->set_theme( NULL );
-	TextviewDiary::m_buffer->set_theme( NULL );
+	m_ptr2entry->unset_theme();
+	TextviewDiary::m_buffer->set_theme( Diary::d->get_default_theme() );
 	m_button_reset_theme->set_visible( false );
+	update_highlight_button();
 }
 
 void
 EntryView::dismiss_entry( void )
 {
-	Gtk::MessageDialog message( *Lifeobase::base, "", false,
-							    Gtk::MESSAGE_WARNING, Gtk::BUTTONS_NONE, true );
-	message.set_message( _( "Are you sure, you want to dismiss?" ) );
-	message.set_secondary_text( _( "This operation cannot be undone!" ) );
-	message.add_button( Gtk::Stock::CANCEL, Gtk::RESPONSE_CANCEL );
-	message.add_button( _( "_DISMISS!" ), Gtk::RESPONSE_ACCEPT );
-
-	if( message.run() != Gtk::RESPONSE_ACCEPT )
+	if( ! Lifeobase::base->confirm_dismiss_element() )
 		return;
 
-	Lifeobase::base->dismiss_entry( m_ptr2entry );
+	Lifeobase::base->remove_element_from_history( m_ptr2entry );
+
+	Diary::d->dismiss_entry( m_ptr2entry );
+
+	Lifeobase::m_internaloperation++;
+
+	Lifeobase::base->update_calendar();
+	Lifeobase::base->update_entry_list();
+
+	Lifeobase::m_internaloperation--;
 }
 
 void
 EntryView::handle_entry_tag_changed( void )
 {
-	Glib::ustring filter = m_entry_tag->get_text();
+	Glib::ustring filter = m_tag_widget->get_text();
 
 	if( filter.empty() )
 	{
@@ -313,15 +342,13 @@ EntryView::handle_entry_tag_changed( void )
 			}
 		}
 	}
-
-	Lifeobase::base->update_tag_list( filter.lowercase() );
 }
 
 void
 EntryView::handle_button_tag_clicked( void )
 {
 	Tag *tag;
-	Glib::ustring filter = m_entry_tag->get_text();
+	Glib::ustring filter = m_tag_widget->get_text();
 
 	switch( m_tag_operation_cur )
 	{
@@ -332,8 +359,9 @@ EntryView::handle_button_tag_clicked( void )
 			m_ptr2entry->remove_tag( tag );
 			break;
 		case TO_CREATE_AND_ADD:
-			tag = Diary::d->get_tags()->create( filter );
+			tag = Diary::d->create_tag( filter );
 			m_ptr2entry->add_tag( tag );
+			Lifeobase::base->update_tag_list();
 			break;
 		case TO_ADD:
 			tag = Diary::d->get_tags()->get_tag( filter );
@@ -342,7 +370,7 @@ EntryView::handle_button_tag_clicked( void )
 	}
 
 	populate_tags();
-	m_entry_tag->set_text( "" );
+	m_tag_widget->set_text( "" );
 }
 
 void
@@ -403,31 +431,36 @@ EntryView::handle_iconview_tags_button_press( GdkEventButton *event )
 void
 EntryView::show( Entry &entry )
 {
-	Lifeobase::m_internaloperation++;
-
-	Lifeobase::base->change_current_element( &entry, m_vbox_entry );
+	// do nothing is entry is already the current element:
+	if( ! Lifeobase::base->change_current_element( &entry, m_vbox_entry ) )
+		return;
 
 	m_ptr2entry = &entry;
 
 	// HEADER
-	Lifeobase::m_image_gui_elem->set( ICONDIR "/entry-32.png" );
+	Lifeobase::m_image_gui_elem->set( Lifeobase::icons->entry_32 );
 	Lifeobase::base->set_header_texts(
 			entry.m_date.format_string(),
-			entry.m_date.get_weekday_str(),
-			Glib::ustring::compose( "%1: <b>%2</b>\n%3: <b>%4</b>",
+			entry.m_date.is_ordinal() ? Date::format_string_do( &entry.m_date_created )
+					: entry.m_date.get_weekday_str(),
+			Glib::ustring::compose( "%1: <b>%2</b>\n%3: <b>%4</b>\nDEID:%5",
 // TRANSLATORS: label of creation date
 									_( "Created"), entry.get_date_created_str(),
 // TRANSLATORS: label of last change date
-									_( "Changed"), entry.get_date_changed_str() ) );
+									_( "Changed"), entry.get_date_changed_str(),
+									entry.m_id ) );
+
 	Lifeobase::set_gelem_toolbar( m_alignment_toolbar );
 
-	Lifeobase::base->select_date_in_calendar( entry.m_date );
+	// TOOLBAR
+	update_highlight_button();
 
+	Lifeobase::m_internaloperation++;
 	// BODY
 	TextviewDiary::m_buffer->set_richtext( &entry );
 	populate_tags();
 	m_button_toggle_favorite->set_active( entry.m_option_favored );
-	m_button_reset_theme->set_visible( entry.get_theme() != NULL );
+	m_button_reset_theme->set_visible( entry.get_theme_is_set() );
 
 	Lifeobase::m_internaloperation--;
 
@@ -438,16 +471,24 @@ void
 EntryView::prepare_for_hiding( Entry &entry )
 {
 	sync();
-	m_entry_tag->set_text( "" );
+	m_tag_widget->set_text( "" );
 }
 
 void
-EntryView::toggle_details( void )
+EntryView::update_formatting_toolbar( void )
 {
-	if( m_button_toggle_details->get_active() )
-		//m_vbox_entry->pack_start( *m_vbox_details, false, false );
-		m_hbox_editing_toolbar->set_visible( true );
-	else
-		//m_vbox_entry->remove( *m_vbox_details );
-		m_hbox_editing_toolbar->set_visible( false );
+	m_hbox_editing_toolbar->set_visible( Lifeobase::m_settings.show_formatting_toolbar );
+}
+
+void
+EntryView::update_highlight_button( void )
+{
+	const Theme *theme = m_ptr2entry->get_theme_is_set() ?
+			m_ptr2entry->get_theme() : Diary::d->get_default_theme();
+	m_label_highlight->modify_fg( Gtk::STATE_NORMAL, theme->color_text );
+	m_label_highlight->modify_fg( Gtk::STATE_PRELIGHT, theme->color_text );
+	m_label_highlight->modify_fg( Gtk::STATE_ACTIVE, theme->color_text );
+	m_eventbox_highlight->modify_bg( Gtk::STATE_NORMAL, theme->color_highlight );
+	m_eventbox_highlight->modify_bg( Gtk::STATE_PRELIGHT, theme->color_highlight );
+	m_eventbox_highlight->modify_bg( Gtk::STATE_ACTIVE, theme->color_highlight );
 }
