@@ -1,6 +1,6 @@
 /***********************************************************************************
 
-	Copyright (C) 2007-2010 Ahmet Öztürk (aoz_2@yahoo.com)
+	Copyright (C) 2007-2011 Ahmet Öztürk (aoz_2@yahoo.com)
 
 	This file is part of Lifeograph.
 
@@ -185,30 +185,18 @@ TagView::update_combobox_categories( void )
 void
 TagView::show( Tag &tag )
 {
-	Lifeobase::m_internaloperation++;
-
-	Lifeobase::base->change_current_element( &tag, m_vbox );
+	// do nothing is entry is already the current element:
+	if( ! Lifeobase::base->change_current_element( &tag, m_vbox ) )
+		return;
 
 	m_ptr2tag = &tag;
 
+	Lifeobase::m_internaloperation++;
 	// HEADER
-	Lifeobase::m_image_gui_elem->set( ICONDIR "/tag-32.png" );
-	int timespan = 0;
-	//Glib::ustring begins, ends;
-	if( ! tag.m_items.empty() )
-	{
-		Glib::Date gdate_begin = ( * tag.m_items.rbegin() )->get_date().get_glib();
-		Glib::Date gdate_end = ( * tag.m_items.begin() )->get_date().get_glib();
-		timespan = gdate_begin.days_between( gdate_end );
-		//begins = gdate_begin.format_string( "%Y.%m.%d" );
-		//ends = gdate_end.format_string( "%Y.%m.%d" );
-	}
+	Lifeobase::m_image_gui_elem->set( Lifeobase::icons->tag_32 );
 	Lifeobase::base->set_header_texts(
 			tag.m_name,
-			Glib::ustring::compose(
-					_( "%1 entries spanning %2 days" ),
-					tag.m_items.size(),
-					timespan ) );
+			Glib::ustring::compose( _( "%1 entries" ), tag.m_items.size() ) );
 	Lifeobase::set_gelem_toolbar( m_alignment_toolbar );
 
 	// BODY
@@ -264,16 +252,11 @@ TagView::show( Tag &tag )
 void
 TagView::dismiss_tag( void )
 {
-	Gtk::MessageDialog message( *Lifeobase::base, "", false,
-								Gtk::MESSAGE_WARNING, Gtk::BUTTONS_NONE, true );
-	message.set_message( _( "Are you sure, you want to dismiss?" ) );
-	message.set_secondary_text( _( "This operation cannot be undone!" ) );
-	message.add_button( Gtk::Stock::CANCEL, Gtk::RESPONSE_CANCEL );
-	message.add_button( _( "_DISMISS!" ), Gtk::RESPONSE_ACCEPT );
-
-	if( message.run() != Gtk::RESPONSE_ACCEPT )
+	if( ! Lifeobase::base->confirm_dismiss_element() )
 		return;
 
+	Lifeobase::base->remove_element_from_history( m_ptr2tag );
+
 	// clear filter if necessary:
 	if( Diary::d->get_filter_tag() == m_ptr2tag )
 	{
@@ -325,12 +308,14 @@ CategoryTagsView::CategoryTagsView( void )
 void
 CategoryTagsView::show( CategoryTags &category )
 {
-	Lifeobase::base->change_current_element( &category, m_vbox );
+	// do nothing is entry is already the current element:
+	if( ! Lifeobase::base->change_current_element( &category, m_vbox ) )
+		return;
 
 	m_ptr2category = &category;
 
 	// HEADER
-	Lifeobase::m_image_gui_elem->set( ICONDIR "/category_tags-32.png" );
+	Lifeobase::m_image_gui_elem->set( Lifeobase::icons->tag_category_32 );
 	Lifeobase::base->set_header_texts(
 			category.m_name,
 			Glib::ustring::compose( _( "%1 tags" ), category.size() ) );
@@ -338,8 +323,6 @@ CategoryTagsView::show( CategoryTags &category )
 
 	// BODY
 	m_entry_name->set_text( category.m_name );
-
-	//Lifeobase::base->select_date_in_calendar( chapter.m_date_begin );
 }
 
 void
@@ -366,17 +349,11 @@ CategoryTagsView::handle_name_applied( void )
 void
 CategoryTagsView::dismiss_category( void )
 {
-	// TODO: this code is repeated in every view, so it should be unified
-	Gtk::MessageDialog message( *Lifeobase::base, "", false,
-								Gtk::MESSAGE_WARNING, Gtk::BUTTONS_NONE, true );
-	message.set_message( _( "Are you sure, you want to dismiss?" ) );
-	message.set_secondary_text( _( "This operation cannot be undone!" ) );
-	message.add_button( Gtk::Stock::CANCEL, Gtk::RESPONSE_CANCEL );
-	message.add_button( _( "_DISMISS!" ), Gtk::RESPONSE_ACCEPT );
-
-	if( message.run() != Gtk::RESPONSE_ACCEPT )
+	if( Lifeobase::base->confirm_dismiss_element() )
 		return;
 
+	Lifeobase::base->remove_element_from_history( m_ptr2category );
+
 	// TODO: offer merging with a category
 	Diary::d->show();
 	Diary::d->dismiss_tag_ctg( m_ptr2category );
@@ -389,23 +366,74 @@ ChapterView::ChapterView( void )
 {
 	Chapter::shower = this;
 
+	Gtk::Button		*button_new_entry;
+
 	try
 	{
 		Lifeobase::builder->get_widget( "vbox_chapter", m_vbox );
+		Lifeobase::builder->get_widget( "alignment_chapter_toolbar", m_alignment_toolbar );
+		Lifeobase::builder->get_widget( "button_chapter_dismiss", m_button_dismiss );
 		Lifeobase::builder->get_widget( "entry_chapter_name", m_entry_name );
 		Lifeobase::builder->get_widget( "button_chapter_name_apply", m_button_name_apply );
+
+		Lifeobase::builder->get_widget( "label_chapter_period", m_label_period );
+		Lifeobase::builder->get_widget( "hbox_chapter_period", m_hbox_period );
 		Lifeobase::builder->get_widget( "entry_chapter_begins", m_entry_begins );
+		Lifeobase::builder->get_widget( "button_chapter_date_apply", m_button_date_apply );
 		Lifeobase::builder->get_widget( "label_chapter_ends", m_label_ends );
+
+		Lifeobase::builder->get_widget( "frame_chapter_actions", m_frame_actions );
+		Lifeobase::builder->get_widget( "button_chapter_new_entry", button_new_entry );
+
+		//m_liststore_chapter_type = Gtk::ListStore::create( colrec_chapter_type );
 	}
 	catch( ... ) {}
 
+	// TYPE
+	//m_combo_type->set_model( m_liststore_chapter_type );
+	//m_combo_type->pack_start( colrec_chapter_type.name );
+	//Gtk::TreeRow row( * m_liststore_chapter_type->append() );
+	//row[ colrec_chapter_type.name ] = _( "Time Based" );
+	//row[ colrec_chapter_type.type ] = CT_TEMPORAL;
+	//row = * m_liststore_chapter_type->append();
+	//row[ colrec_chapter_type.name ] = _( "Time Independent" );
+	//row[ colrec_chapter_type.type ] = CT_ORDINAL;
+
+	
 	// SIGNALS
+	m_button_dismiss->signal_clicked().connect(
+			sigc::mem_fun( this, &ChapterView::dismiss_chapter ) );
+
 	m_entry_name->signal_changed().connect(
 			sigc::mem_fun( *this, &ChapterView::handle_name_changed ) );
 	m_entry_name->signal_activate().connect(
 			sigc::mem_fun( *m_button_name_apply, &Gtk::Button::clicked ) );
 	m_button_name_apply->signal_clicked().connect(
 			sigc::mem_fun( *this, &ChapterView::handle_name_applied ) );
+
+	m_entry_begins->signal_changed().connect(
+			sigc::mem_fun( *this, &ChapterView::handle_date_changed ) );
+	m_entry_begins->signal_activate().connect(
+			sigc::mem_fun( *m_button_date_apply, &Gtk::Button::clicked ) );
+	m_button_date_apply->signal_clicked().connect(
+			sigc::mem_fun( *this, &ChapterView::handle_date_applied ) );
+
+	button_new_entry->signal_clicked().connect(
+			sigc::mem_fun( *this, &ChapterView::add_new_entry ) );
+}
+
+void
+ChapterView::dismiss_chapter( void )
+{
+	if( ! Lifeobase::base->confirm_dismiss_element() )
+		return;
+
+	Lifeobase::base->remove_element_from_history( m_ptr2chapter );
+
+	Diary::d->show();
+
+	Diary::d->dismiss_chapter( m_ptr2chapter );
+	Lifeobase::base->update_entry_list();
 }
 
 void
@@ -427,42 +455,106 @@ ChapterView::handle_name_applied( void )
 	m_button_name_apply->set_sensitive( false );
 	Lifeobase::base->m_label_gui_elem->set_label( name );
 	Gtk::TreeRow row = * Lifeobase::base->get_element_row( m_ptr2chapter );
-	row[ ListData::colrec->info ] = m_ptr2chapter->get_2nd_str();
+	row[ ListData::colrec->info ] = m_ptr2chapter->get_list_str();
+}
+
+void
+ChapterView::handle_date_changed( void )
+{
+	try
+	{
+		Date date( m_entry_begins->get_text() );
+		m_button_date_apply->set_sensitive( 
+				! Diary::d->get_current_chapter_ctg()->get_chapter( date ) );
+	}
+	catch( LIFEO::Error &er )
+	{
+		m_button_date_apply->set_sensitive( false );
+	}
+}
+
+void
+ChapterView::handle_date_applied( void )
+{
+	// a caution against calls by an entry activation event:
+	if( ! m_button_date_apply->is_sensitive() )
+		return;
+	if( Diary::d->get_current_chapter_ctg()->set_chapter_date(
+			m_ptr2chapter, Date( m_entry_begins->get_text() ) ) )
+	{
+		m_button_date_apply->set_sensitive( false );
+		Lifeobase::base->update_entry_list();
+		update_labels();
+	}
+}
+
+void
+ChapterView::update_labels( void )
+{
+	if( m_ptr2chapter->m_date_begin.is_ordinal() )
+	{
+		Lifeobase::base->set_header_texts(
+				Glib::ustring::compose( "%1. %2", m_ptr2chapter->get_date().format_string(),
+									    m_ptr2chapter->m_name ),
+				Glib::ustring::compose( _( "%1 entries" ), m_ptr2chapter->get_size() ) );
+	}
+	else
+	{
+		int timespan( 0 );
+
+		m_entry_begins->set_text( m_ptr2chapter->m_date_begin.format_string() );
+
+		CategoryChapters::iterator iter( Diary::d->get_current_chapter_ctg()->find(
+												m_ptr2chapter->get_date().m_date ) );
+		if( iter == Diary::d->get_current_chapter_ctg()->begin() )
+			m_label_ends->set_text( _( "Unlimited" ) );
+		else
+		{
+			--iter;
+			m_label_ends->set_text( iter->second->m_date_begin.format_string() );
+			Glib::Date gdate_begin( m_ptr2chapter->m_date_begin.get_glib() );
+			Glib::Date gdate_end( iter->second->m_date_begin.get_glib() );
+			timespan = gdate_begin.days_between( gdate_end );
+		}
+
+		Lifeobase::base->set_header_texts(
+				m_ptr2chapter->m_name,
+				Glib::ustring::compose(
+						timespan ? _( "%1 entries in %2 days" ) : _( "%1 entries" ),
+						m_ptr2chapter->get_size(), timespan ) );
+	}
+}
+
+void
+ChapterView::add_new_entry( void )
+{
+	Diary::d->create_entry( m_ptr2chapter->get_free_order() );
+
+	Lifeobase::base->update_entry_list();
+	Lifeobase::base->update_calendar();
 }
 
 void
 ChapterView::show( Chapter &chapter )
 {
-	Lifeobase::base->change_current_element( &chapter, m_vbox );
+	// do nothing is entry is already the current element:
+	if( ! Lifeobase::base->change_current_element( &chapter, m_vbox ) )
+		return;
 
 	m_ptr2chapter = &chapter;
 
-	// HEADER
-	int timespan = 0;
-	Glib::ustring begins, ends;
-	if( ! chapter.m_items.empty() )
-	{
-		Glib::Date gdate_begin = ( * chapter.m_items.rbegin() )->get_date().get_glib();
-		Glib::Date gdate_end = ( * chapter.m_items.begin() )->get_date().get_glib();
-		timespan = gdate_begin.days_between( gdate_end );
-		begins = chapter.m_date_begin.format_string();
-		// TODO: replace this one with the begin date of the next chapter if any:
-		ends = gdate_end.format_string( "%Y.%m.%d" );
-	}
+	Lifeobase::m_image_gui_elem->set( chapter.get_icon_32() );
+	Lifeobase::set_gelem_toolbar( m_alignment_toolbar );
 
-	Lifeobase::m_image_gui_elem->set( ICONDIR "/chapter-32.png" );
-	Lifeobase::base->set_header_texts(
-			chapter.m_name,
-			Glib::ustring::compose( _( "%1 entries spanning %2 days" ),
-					chapter.m_items.size(), timespan ) );
-	Lifeobase::set_gelem_toolbar( NULL );
-
-	Lifeobase::base->select_date_in_calendar( chapter.m_date_begin );
-
-	// BODY
+	Lifeobase::m_internaloperation++;
+	update_labels();
 	m_entry_name->set_text( chapter.m_name );
-	m_entry_begins->set_text( begins );
-	m_label_ends->set_text( ends );
+	Lifeobase::m_internaloperation--;
+
+	bool flag_temporal( ! chapter.m_date_begin.is_ordinal() );
+	m_label_period->set_visible( flag_temporal );
+	m_hbox_period->set_visible( flag_temporal );
+	m_frame_actions->set_visible( ! flag_temporal );
 }
 
 // THEME =======================================================================
@@ -522,15 +614,16 @@ ThemeView::ThemeView( void )
 void
 ThemeView::show( Theme &theme )
 {
-	Lifeobase::base->change_current_element( &theme, m_vbox );
+	// do nothing is entry is already the current element:
+	if( ! Lifeobase::base->change_current_element( &theme, m_vbox ) )
+		return;
 
 	m_ptr2theme = &theme;
 
 	// HEADER
 	Lifeobase::m_image_gui_elem->set( Lifeobase::icons->theme_32 );
 	Lifeobase::base->set_header_texts(
-			theme.get_name(),
-			_( "Custom theme" ) );
+			theme.get_name(), theme.is_system() ? _( "System theme" ) : _( "Custom theme" ) );
 	Lifeobase::set_gelem_toolbar( m_alignment_toolbar );
 
 	// BODY
@@ -542,8 +635,8 @@ ThemeView::show( Theme &theme )
 	m_colorbutton_subheading->set_color( theme.color_subheading );
 	m_colorbutton_highlight->set_color( theme.color_highlight );
 
-	m_vbox->set_sensitive( &theme != &Theme::system_theme );
-	m_button_dismiss->set_sensitive( &theme != &Theme::system_theme );
+	m_vbox->set_sensitive( ! theme.is_system() );
+	m_button_dismiss->set_sensitive( ! theme.is_system() );
 	m_button_make_default->set_visible( &theme != Diary::d->get_default_theme() );
 }
 
@@ -602,19 +695,13 @@ ThemeView::make_default( void )
 void
 ThemeView::dismiss_theme( void )
 {
-	// TODO: this code is repeated in every view, so it should be unified
-	Gtk::MessageDialog message( *Lifeobase::base, "", false,
-								Gtk::MESSAGE_WARNING, Gtk::BUTTONS_NONE, true );
-	message.set_message( _( "Are you sure, you want to dismiss?" ) );
-	message.set_secondary_text( _( "This operation cannot be undone!" ) );
-	message.add_button( Gtk::Stock::CANCEL, Gtk::RESPONSE_CANCEL );
-	message.add_button( _( "_DISMISS!" ), Gtk::RESPONSE_ACCEPT );
-
-	if( message.run() != Gtk::RESPONSE_ACCEPT )
+	if( ! Lifeobase::base->confirm_dismiss_element() )
 		return;
 
+	Lifeobase::base->remove_element_from_history( m_ptr2theme );
+
 	Theme *theme = m_ptr2theme; // always be on the safe side
-	Theme::system_theme.show();
+	ThemeSystem::get()->show();
 	Diary::d->dismiss_theme( theme );
 	Lifeobase::base->update_theme_list();
 }
@@ -628,18 +715,18 @@ DiaryView::DiaryView( void )
 
 	Gtk::Button *button_import( NULL );
 	Gtk::Button *button_export( NULL );
-	Gtk::Button *button_chapters( NULL );
+	Gtk::Button *button_add_topic( NULL );
 	try
 	{
 		Lifeobase::builder->get_widget( "vbox_diary", m_vbox_diary );
+		Lifeobase::builder->get_widget( "alignment_diary_toolbar", m_alignment_toolbar );
 		Lifeobase::builder->get_widget( "label_path", m_label_path );
 		Lifeobase::builder->get_widget( "label_encryption", m_label_encryption );
-		Lifeobase::builder->get_widget( "button_log_out_wosaving",
-				m_button_logoutwosaving );
-		Lifeobase::builder->get_widget( "button_import", button_import );
-		Lifeobase::builder->get_widget( "button_export", button_export );
-		Lifeobase::builder->get_widget( "button_chapters", button_chapters );
-		Lifeobase::builder->get_widget( "button_password", m_button_password );
+
+		Lifeobase::builder->get_widget( "cmb_startup_type", m_combobox_startup_type );
+		Lifeobase::builder->get_widget( "label_startup_elem", m_label_startup_elem );
+		Lifeobase::builder->get_widget( "label_startup_elem_drop", m_label_startup_elem_drop );
+		Lifeobase::builder->get_widget( "button_startup_elem_go", m_button_startup_elem_go );
 
 		Lifeobase::builder->get_widget( "combobox_chapter_ctg",
 				m_combobox_chapter_ctg );
@@ -648,10 +735,35 @@ DiaryView::DiaryView( void )
 		Lifeobase::builder->get_widget( "button_chapter_ctg_rename",
 				m_button_chapter_ctg_rename );
 
+		Lifeobase::builder->get_widget( "button_add_topic", button_add_topic );
+
+		Lifeobase::builder->get_widget( "button_import", button_import );
+		Lifeobase::builder->get_widget( "button_export", button_export );
+		Lifeobase::builder->get_widget( "button_password", m_button_password );
+
+		Lifeobase::builder->get_widget( "button_log_out_wosaving",
+				m_button_logoutwosaving );
+
+		m_liststore_startup_type = Gtk::ListStore::create( colrec_startup_type );
 		m_liststore_chapter_ctg = Gtk::ListStore::create( colrec_chapter_ctg );
 	}
 	catch( ... ) { }
 
+	// STARTUP
+	m_combobox_startup_type->set_model( m_liststore_startup_type );
+	m_combobox_startup_type->pack_start( colrec_startup_type.name );
+	Gtk::TreeRow row( * m_liststore_startup_type->append() );
+	row[ colrec_startup_type.name ] = _( "Show Most Current Entry" );
+	row[ colrec_startup_type.type ] = HOME_CURRENT_ELEM;
+	row = * m_liststore_startup_type->append();
+	row[ colrec_startup_type.name ] = _( "Remember Last Item" );
+	row[ colrec_startup_type.type ] = HOME_LAST_ELEM;
+	row = * m_liststore_startup_type->append();
+	row[ colrec_startup_type.name ] = _( "Always Show a Fixed Item" );
+	row[ colrec_startup_type.type ] = DEID_MIN;
+
+	m_label_startup_elem_drop->drag_dest_set( Lifeobase::base->drag_targets_entry );
+
 	// CHAPTER CATEGORIES
 	m_combobox_chapter_ctg->set_model( m_liststore_chapter_ctg );
 	m_combobox_chapter_ctg->set_text_column( colrec_chapter_ctg.name );
@@ -661,25 +773,33 @@ DiaryView::DiaryView( void )
 	// SIGNALS
 	m_button_password->signal_clicked().connect(
 			sigc::mem_fun( *this, &DiaryView::start_dialog_password ) );
-	button_import->signal_clicked().connect(
-			sigc::mem_fun( *this, &DiaryView::start_dialog_import ) );
-	button_export->signal_clicked().connect(
-			sigc::mem_fun( *this, &DiaryView::start_dialog_export ) );
-	button_chapters->signal_clicked().connect(
-			sigc::mem_fun( *this, &DiaryView::start_dialog_chaptereditor ) );
-	m_button_logoutwosaving->signal_clicked().connect(
-			sigc::bind( sigc::mem_fun( *Lifeobase::base, &Lifeobase::logout ), false ) );
+
+	m_combobox_startup_type->signal_changed().connect(
+			sigc::mem_fun( this, &DiaryView::handle_startup_type_changed ) );
+	m_label_startup_elem_drop->signal_drag_data_received().connect(
+			sigc::mem_fun( this, &DiaryView::handle_startup_elem_dropped ) );
+	m_button_startup_elem_go->signal_clicked().connect(
+			sigc::mem_fun( *this, &DiaryView::go_to_startup_elem ) );
 
 	m_combobox_chapter_ctg->signal_changed().connect(
 			sigc::mem_fun( this, &DiaryView::handle_cur_chapter_ctg_changed ) );
 	m_combobox_chapter_ctg->get_entry()->signal_activate().connect(
 			sigc::mem_fun( this, &DiaryView::rename_cur_chapter_ctg ) );
-
 	m_button_chapter_ctg_rename->signal_clicked().connect(
 			sigc::mem_fun( this, &DiaryView::rename_cur_chapter_ctg ) );
-
 	m_button_chapter_ctg_dismiss->signal_clicked().connect(
 			sigc::mem_fun( this, &DiaryView::dismiss_cur_chapter_ctg ) );
+
+	button_add_topic->signal_clicked().connect(
+			sigc::mem_fun( this, &DiaryView::add_topic ) );
+
+	button_import->signal_clicked().connect(
+			sigc::mem_fun( *this, &DiaryView::start_dialog_import ) );
+	button_export->signal_clicked().connect(
+			sigc::mem_fun( *this, &DiaryView::start_dialog_export ) );
+
+	m_button_logoutwosaving->signal_clicked().connect(
+			sigc::bind( sigc::mem_fun( *Lifeobase::base, &Lifeobase::logout ), false ) );
 }
 
 DiaryView::~DiaryView( void )
@@ -705,30 +825,55 @@ add_group( const Glib::ustring& str_label, Gtk::Widget& content )
 	return frame;
 }*/
 
+void
+DiaryView::handle_login( void )
+{
+	Lifeobase::m_internaloperation++;
+
+	update_combobox_chapter_ctg();
+	if( Diary::d->m_startup_elem < HOME_FIXED_ELEM )
+		m_combobox_startup_type->set_active( Diary::d->m_startup_elem - 1 );
+	else
+		m_combobox_startup_type->set_active( 2 );
+
+	Lifeobase::m_internaloperation--;
+}
+
 void
 DiaryView::show( Diary &diary )
 {
-	Lifeobase::base->change_current_element( &diary, m_vbox_diary );
+	// do nothing is entry is already the current element:
+	if( ! Lifeobase::base->change_current_element( &diary, m_vbox_diary ) )
+		return;
 
 	// not necessary for now as there is only one diary to show in the view:
 	// m_ptr2diary = &diary;
 
 	// HEADER
-	int timespan = 0;
+	int timespan( 0 );
 	if( ! diary.m_entries.empty() )
 	{
-		Glib::Date gdate_begin = diary.m_entries.rbegin()->second->get_date().get_glib();
-		Glib::Date gdate_end = diary.m_entries.begin()->second->get_date().get_glib();
-		timespan = gdate_begin.days_between( gdate_end );
+		for( Entryiter iter = diary.m_entries.begin();
+			 iter != diary.m_entries.end();
+			 ++iter )
+		{
+			if( ! iter->second->get_date().is_ordinal() )
+			{
+				Glib::Date gdate_begin( diary.m_entries.rbegin()->second->get_date().get_glib() );
+				Glib::Date gdate_end( iter->second->get_date().get_glib() );
+				timespan = gdate_begin.days_between( gdate_end );
+				break;
+			}
+		}
 	}
 
-	Lifeobase::m_image_gui_elem->set( ICONDIR "/diary-32.png" );
+	Lifeobase::m_image_gui_elem->set( Lifeobase::icons->diary_32 );
 	Lifeobase::base->set_header_texts( diary.get_name(),
 									   Glib::ustring::compose(
 												_( "%1 entries spanning %2 days" ),
 												diary.m_entries.size(),
 												timespan ) );
-	Lifeobase::set_gelem_toolbar( NULL );
+	Lifeobase::set_gelem_toolbar( m_alignment_toolbar );
 
 	// BODY
 	m_button_password->set_label( diary.m_passphrase.empty() ? _( "Encrypt..." ) :
@@ -778,23 +923,6 @@ DiaryView::start_dialog_password( void )
 	m_dialog_password = NULL;
 }
 
-#include "dialog_chaptereditor.hpp"
-
-void
-DiaryView::start_dialog_chaptereditor( void )
-{
-	DialogChaptereditor *dialog_chaptereditor = new DialogChaptereditor( m_ptr2diary );
-
-	dialog_chaptereditor->run();
-
-	if( m_ptr2diary->m_option_sorting_criteria == SC_DATE )
-	{
-		Lifeobase::base->update_entry_list();
-	}
-
-	delete dialog_chaptereditor;
-}
-
 void
 DiaryView::start_dialog_import( void )
 {
@@ -831,30 +959,68 @@ DiaryView::start_dialog_export( void )
 //	gtk_show_uri( NULL, uri.c_str(), GDK_CURRENT_TIME, &err);
 //}
 
+void
+DiaryView::handle_startup_type_changed( void )
+{
+	if( ! Lifeobase::m_internaloperation )
+	{
+		Gtk::TreeRow row( * m_combobox_startup_type->get_active() );
+		Diary::d->m_startup_elem = row[ colrec_startup_type.type ];
+	}
+	bool flag_show_fixed_item( Diary::d->m_startup_elem > HOME_FIXED_ELEM );
+	m_label_startup_elem->set_visible( flag_show_fixed_item );
+	m_label_startup_elem_drop->set_visible( flag_show_fixed_item );
+	m_button_startup_elem_go->set_visible( flag_show_fixed_item );
+	if( flag_show_fixed_item )
+		m_label_startup_elem_drop->set_label( Diary::d->get_startup_elem()->get_name() );
+}
+
+void
+DiaryView::handle_startup_elem_dropped(
+				const Glib::RefPtr< Gdk::DragContext > &context,
+				int x, int y,
+				const Gtk::SelectionData &selection_data,
+				uint info,
+				guint time )
+{
+	context->drag_finish( true, false, time );
+
+	Glib::ustring name = selection_data.get_data_as_string();
+
+	if( name.empty() )	// should never happen
+		return;
+
+	switch( info )
+	{
+		case DRAG_TARGET_ENTRY_INFO:
+		{
+			DiaryElement *element = Lifeobase::base->get_dragged_elem();
+			if( element != NULL )
+			{
+				m_label_startup_elem_drop->set_label( element->get_name() );
+				Diary::d->m_startup_elem = element->get_id();
+			}
+			break;
+		}
+		default:
+			break;
+	}
+}
+
+void
+DiaryView::go_to_startup_elem( void )
+{
+	DiaryElement *element( Diary::d->get_startup_elem() );
+	if( element != NULL && element != Diary::d )
+		element->show();
+}
+
 void
 DiaryView::update_combobox_chapter_ctg( void )
 {
 	m_liststore_chapter_ctg->clear();
 
-	Gtk::TreeRow row = * m_liststore_chapter_ctg->append();
-
-	row[ colrec_chapter_ctg.name ] = _( "None" );
-	row[ colrec_chapter_ctg.ptr ] = NULL;
-	row[ colrec_chapter_ctg.type ] = CCCT_NONE;
-
-	// setting current item:
-	if( m_ptr2diary->m_ptr2chapter_ctg_cur == NULL )
-	{
-		m_combobox_chapter_ctg->set_active( row );
-		m_button_chapter_ctg_rename->set_visible( false );
-		m_button_chapter_ctg_dismiss->set_visible( true );
-		m_button_chapter_ctg_dismiss->set_sensitive( false );
-		m_combobox_chapter_ctg->get_child()->set_sensitive( false );
-	}
-
-	// separator 1:
-	row = * m_liststore_chapter_ctg->append();
-	row[ colrec_chapter_ctg.type ] = CCCT_SEPARATOR;
+	Gtk::TreeRow row;
 
 	for( PoolCategoriesChapters::const_iterator iter =
 				m_ptr2diary->m_chapter_categories.begin();
@@ -871,15 +1037,13 @@ DiaryView::update_combobox_chapter_ctg( void )
 
 		// setting current item:
 		if( m_ptr2diary->m_ptr2chapter_ctg_cur == category )
-		{
 			m_combobox_chapter_ctg->set_active( row );
-			m_button_chapter_ctg_rename->set_visible( false );
-			m_button_chapter_ctg_dismiss->set_visible( true );
-			m_button_chapter_ctg_dismiss->set_sensitive( true );
-			m_combobox_chapter_ctg->get_child()->set_sensitive( true );
-		}
 	}
 
+	m_button_chapter_ctg_rename->set_visible( false );
+	m_button_chapter_ctg_dismiss->set_visible( true );
+	m_button_chapter_ctg_dismiss->set_sensitive( m_ptr2diary->m_chapter_categories.size() > 1 );
+
 	// separator 2:
 	if( m_ptr2diary->m_chapter_categories.size() > 0 )
 	{
@@ -890,8 +1054,7 @@ DiaryView::update_combobox_chapter_ctg( void )
 	row = * m_liststore_chapter_ctg->append();
 
 	row[ colrec_chapter_ctg.name ] = _( "Create New..." );
-	// a dummy value to differentiate it from "None":
-	row[ colrec_chapter_ctg.ptr ] = ( CategoryChapters* ) 0x1;
+	row[ colrec_chapter_ctg.ptr ] = NULL;
 	row[ colrec_chapter_ctg.type ] = CCCT_NEW;
 }
 
@@ -901,7 +1064,7 @@ DiaryView::handle_cur_chapter_ctg_changed( void )
 	if( Lifeobase::m_internaloperation ) return;
 
 	Gtk::TreeIter iter = m_combobox_chapter_ctg->get_active();
-	bool flag_renamed = ! ( iter ), flag_existing_ctg = false;
+	bool flag_renamed = ! ( iter ), flag_existing_ctg( false );
 
 	if( flag_renamed )
 	{
@@ -921,7 +1084,6 @@ DiaryView::handle_cur_chapter_ctg_changed( void )
 				break;
 			case CCCT_CATEGORY:
 				flag_existing_ctg = true;
-			case CCCT_NONE:
 				m_ptr2diary->m_ptr2chapter_ctg_cur = row[ colrec_chapter_ctg.ptr ];
 				Lifeobase::base->update_entry_list();
 				break;
@@ -934,7 +1096,6 @@ DiaryView::handle_cur_chapter_ctg_changed( void )
 
 	m_button_chapter_ctg_rename->set_visible( flag_renamed );
 	m_button_chapter_ctg_dismiss->set_visible( ! flag_renamed );
-	m_combobox_chapter_ctg->get_child()->set_sensitive( flag_existing_ctg || flag_renamed );
 }
 
 void
@@ -950,14 +1111,7 @@ DiaryView::rename_cur_chapter_ctg( void )
 void
 DiaryView::dismiss_cur_chapter_ctg( void )
 {
-	Gtk::MessageDialog message( *Lifeobase::base, "", false,
-								Gtk::MESSAGE_WARNING, Gtk::BUTTONS_NONE, true );
-	message.set_message( _( "Are you sure, you want to dismiss?" ) );
-	message.set_secondary_text( _( "This operation cannot be undone!" ) );
-	message.add_button( Gtk::Stock::CANCEL, Gtk::RESPONSE_CANCEL );
-	message.add_button( _( "_DISMISS!" ), Gtk::RESPONSE_ACCEPT );
-
-	if( message.run() != Gtk::RESPONSE_ACCEPT )
+	if( ! Lifeobase::base->confirm_dismiss_element() )
 		return;
 
 	m_ptr2diary->dismiss_chapter_ctg( m_ptr2diary->m_ptr2chapter_ctg_cur );
@@ -971,3 +1125,11 @@ DiaryView::is_row_separator( const Glib::RefPtr< Gtk::TreeModel > &model,
 	Gtk::TreeRow row = * iter;
 	return ( row[ colrec_chapter_ctg.type ] == CCCT_SEPARATOR );
 }
+
+void
+DiaryView::add_topic( void )
+{
+	Diary::d->add_topic( _( "New topic" ) );
+	Lifeobase::base->update_entry_list();
+	// show the new topic?
+}
