@@ -1,6 +1,6 @@
 /***********************************************************************************
 
-	Copyright (C) 2007-2010 Ahmet Öztürk (aoz_2@yahoo.com)
+	Copyright (C) 2007-2011 Ahmet Öztürk (aoz_2@yahoo.com)
 
 	This file is part of Lifeograph.
 
@@ -125,15 +125,45 @@ class ChapterView : public Chapter::Shower
 		void					show( Chapter& );
 
 	protected:
+		void					update_labels( void );
+		void					dismiss_chapter( void );
+
 		void					handle_name_changed( void );
 		void					handle_name_applied( void );
 
+		void					handle_date_changed( void );
+		void					handle_date_applied( void );
+
+		void					add_new_entry( void );
+
 		Gtk::VBox				*m_vbox;
+		Gtk::Alignment			*m_alignment_toolbar;
+		Gtk::Button				*m_button_dismiss;
 		Gtk::Entry				*m_entry_name;
 		Gtk::Button				*m_button_name_apply;
+
+		//Gtk::ComboBox			*m_combo_type;
+		//enum ChapterType { CT_TEMPORAL, CT_ORDINAL };
+		//class ColrecChapterType : public Gtk::TreeModel::ColumnRecord
+		//{
+			//public:
+				//ColrecChapterType() { add( name ); add( type ); }
+				//Gtk::TreeModelColumn< Glib::ustring >	name;
+				//Gtk::TreeModelColumn< ChapterType >		type;
+		//};
+
+		//Glib::RefPtr< Gtk::ListStore >
+								//m_liststore_chapter_type;
+		//ColrecChapterType		colrec_chapter_type;
+
+		Gtk::Label				*m_label_period;
+		Gtk::HBox				*m_hbox_period;
 		Gtk::Entry				*m_entry_begins;
+		Gtk::Button				*m_button_date_apply;
 		Gtk::Label				*m_label_ends;
 
+		Gtk::Frame				*m_frame_actions;
+
 		Chapter					*m_ptr2chapter;
 };
 
@@ -180,7 +210,7 @@ class DiaryView : public Diary::Shower
 								DiaryView( void );
 								~DiaryView( void );
 
-		void					update_combobox_chapter_ctg( void );
+		void					handle_login( void );
 
 		void					show( Diary& );
 
@@ -188,29 +218,58 @@ class DiaryView : public Diary::Shower
 		void					start_dialog_password( void );
 		void					start_dialog_import( void );
 		void					start_dialog_export( void );
-		void					start_dialog_chaptereditor( void );
 		void					open_diary_folder( void );
 
 	protected:
+		void					handle_startup_type_changed( void );
+		void					handle_startup_elem_dropped(
+										const Glib::RefPtr< Gdk::DragContext >&,
+										int, int,
+										const Gtk::SelectionData&,
+										uint, guint );
+		void					go_to_startup_elem( void );
+
+		void					update_combobox_chapter_ctg( void );
 		void					handle_cur_chapter_ctg_changed( void );
 		void					rename_cur_chapter_ctg( void );
 		void					dismiss_cur_chapter_ctg( void );
 		bool					is_row_separator( const Glib::RefPtr< Gtk::TreeModel >&,
 												  const Gtk::TreeIter );
 
+		void					add_topic( void );
+
 		Gtk::VBox				*m_vbox_diary;
+		Gtk::Alignment			*m_alignment_toolbar;
 		Gtk::Label				*m_label_path;
 		Gtk::Label				*m_label_encryption;
 		Gtk::Button				*m_button_logoutwosaving;
 		Gtk::CheckButton		*m_checkbutton_autologout;
 		Gtk::Button				*m_button_password;
 
-		// CHAPTER CATEGORIES
+		// STARTUP
+		Gtk::ComboBox			*m_combobox_startup_type;
+		Gtk::Label				*m_label_startup_elem;
+		Gtk::Label				*m_label_startup_elem_drop;
+		Gtk::Button				*m_button_startup_elem_go;
+
+		class ColrecStartupType : public Gtk::TreeModel::ColumnRecord
+		{
+			public:
+				ColrecStartupType() { add( name ); add( type ); }
+				Gtk::TreeModelColumn< Glib::ustring >	name;
+				Gtk::TreeModelColumn< DEID >			type;
+		};
+
+		Glib::RefPtr< Gtk::ListStore >
+								m_liststore_startup_type;
+		ColrecStartupType		colrec_startup_type;
+
+		// CHAPTER CATEGORIES & TOPICS
 		Gtk::ComboBoxEntry		*m_combobox_chapter_ctg;
 		Gtk::Button				*m_button_chapter_ctg_dismiss;
 		Gtk::Button				*m_button_chapter_ctg_rename;
 
-		enum ChapterCtgComboType { CCCT_NONE, CCCT_CATEGORY, CCCT_NEW, CCCT_SEPARATOR };
+		enum ChapterCtgComboType { CCCT_CATEGORY, CCCT_NEW, CCCT_SEPARATOR };
 
 		class ColrecChapterCtg : public Gtk::TreeModel::ColumnRecord
 		{
