@@ -1,6 +1,6 @@
 /***********************************************************************************
 
-	Copyright (C) 2007-2009 Ahmet Öztürk (aoz_2@yahoo.com)
+	Copyright (C) 2007-2011 Ahmet Öztürk (aoz_2@yahoo.com)
 
 	This file is part of Lifeograph.
 
@@ -76,11 +76,23 @@ Link::~Link()
 // static variable:
 LinkEntry::Signal_void_Date LinkEntry::m_signal_activated;
 
-LinkEntry::LinkEntry(	const Glib::RefPtr< Gtk::TextMark > &start,
-						const Glib::RefPtr< Gtk::TextMark > &end,
-						Date date,
-						unsigned int order )
-	:	Link( start, end ), m_date( date ), m_order( order )
+LinkID::LinkID( const Glib::RefPtr< Gtk::TextMark > &start,
+				const Glib::RefPtr< Gtk::TextMark > &end,
+				DEID id )
+:	Link( start, end ), m_id( id )
+{
+}
+
+void
+LinkID::go( void )
+{
+	Diary::d->get_element( m_id )->show();
+}
+
+LinkEntry::LinkEntry( const Glib::RefPtr< Gtk::TextMark > &start,
+					  const Glib::RefPtr< Gtk::TextMark > &end,
+					  Date date )
+:	Link( start, end ), m_date( date )
 {
 }
 
@@ -181,9 +193,16 @@ TextbufferDiary::TextbufferDiary( void )
 	tag_table->add( m_tag_match );
 
 	m_tag_markup = Tag::create( "markup" );
-	//m_tag_markup->property_invisible() = true;	// for now visible is the better default
+	m_tag_markup->property_scale() = 0.5;
 	tag_table->add( m_tag_markup );
 
+	m_tag_markup_link = Tag::create( "markup.link" );
+	tag_table->add( m_tag_markup_link );
+
+	m_tag_hidden = Tag::create( "hidden" );
+	m_tag_hidden->property_invisible() = true;
+	tag_table->add( m_tag_hidden );
+
 	m_tag_bold = Tag::create( "bold" );
 	m_tag_bold->property_weight() = Pango::WEIGHT_BOLD;
 	tag_table->add( m_tag_bold );
@@ -207,22 +226,16 @@ TextbufferDiary::TextbufferDiary( void )
 	m_tag_link_broken->property_underline() = Pango::UNDERLINE_SINGLE;
 	tag_table->add( m_tag_link_broken );
 
+	// this is just for keeping the boundaries:
+	m_tag_link_hidden = Tag::create( "link.hidden" );
+	tag_table->add( m_tag_link_hidden );
+
 	m_tag_checkbox = Tag::create( "checkbox" );
 	m_tag_checkbox->property_weight() = Pango::WEIGHT_BOLD;
 	m_tag_checkbox->property_scale() = 1.2;
 	tag_table->add( m_tag_checkbox );
 
 	//PLANS
-	m_plan_date.push_back( LF_NUMBER|LF_APPLY_NUMBER );
-	m_plan_date.push_back( LF_NUMBER|LF_APPLY_NUMBER );
-	m_plan_date.push_back( LF_NUMBER|LF_APPLY_NUMBER );
-	m_plan_date.push_back( LF_DOTYM|LF_APPLY_DOTYM );
-	m_plan_date.push_back( LF_NUMBER|LF_APPLY_NUMBER );
-	m_plan_date.push_back( LF_NUMBER|LF_APPLY_NUMBER );
-	m_plan_date.push_back( LF_DOTMD|LF_APPLY_DOTMD );
-	m_plan_date.push_back( LF_NUMBER|LF_APPLY_NUMBER );
-	m_plan_date.push_back( LF_NUMBER|LF_APPLY_LINK_DATE );
-
 	m_plan_subheading.push_back( LF_NONSPACE );
 	m_plan_subheading.push_back( LF_NEWLINE|LF_APPLY_SUBHEADING );
 
@@ -237,8 +250,11 @@ TextbufferDiary::TextbufferDiary( void )
 	m_parsers[ LF_HASH|LF_APPLY_MARKUP ] = &TextbufferDiary::parsing_applier_highlight;
 	m_parsers[ LF_SPACE|LF_APPLY_LINK ] = &TextbufferDiary::parsing_applier_link;
 	m_parsers[ LF_NUMBER|LF_APPLY_LINK_DATE ] = &TextbufferDiary::parsing_applier_link_date;
+	m_parsers[ LF_TAB|LF_APPLY_LINK ] = &TextbufferDiary::parsing_triggerer_link_hidden_tab;
+	m_parsers[ LF_MORE|LF_APPLY_LINK ] = &TextbufferDiary::parsing_applier_link;
+	m_parsers[ LF_MORE|LF_APPLY_LINK_DATE ] = &TextbufferDiary::parsing_applier_link_date;
+	m_parsers[ LF_MORE|LF_APPLY_LINK_ID ] = &TextbufferDiary::parsing_applier_link_id;
 
-	m_parsers[ LF_NUMBER|LF_APPLY_NUMBER ] = &TextbufferDiary::parsing_applier_number;
 	m_parsers[ LF_DOTYM|LF_APPLY_DOTYM ] = &TextbufferDiary::parsing_applier_dotym;
 	m_parsers[ LF_DOTMD|LF_APPLY_DOTMD ] = &TextbufferDiary::parsing_applier_dotmd;
 
@@ -408,13 +424,14 @@ TextbufferDiary::parse( unsigned int pos_start, unsigned int pos_end )
 			case '\n':
 			case '\r':
 				process_char( LF_NEWLINE|LF_SPACE,
-							  LF_NUM_SPCIM_CHK|LF_NONSPACE|LF_FORMATCHAR|LF_SLASH|LF_DOTDATE,
+							  LF_NUM_SPCIM_CHK|LF_NONSPACE|LF_FORMATCHAR|
+							  LF_SLASH|LF_DOTDATE|LF_MORE|LF_TAB,
 							  0, NULL,
 							  CC_NEWLINE );
 				break;
 			case ' ':
 				process_char( LF_SPACE|LF_SPACE_IMMEDIATE,
-							  LF_NUMBER|LF_SLASH|LF_DOTDATE|LF_CHECKBOX,
+							  LF_NONSPACE|LF_NUMBER|LF_SLASH|LF_DOTDATE|LF_CHECKBOX,
 							  LF_NOTHING, &TextbufferDiary::parsing_triggerer_subheading,
 							  CC_SPACE );
 				break;
@@ -444,6 +461,8 @@ TextbufferDiary::parse( unsigned int pos_start, unsigned int pos_end )
 				break;
 			case '0': case '1': case '2': case '3': case '4':
 			case '5': case '6': case '7': case '8': case '9':
+				if( parser_lookingfor.front() & ( LF_NOTHING|LF_NUMBER|LF_TAB ) )
+					parsing_handler_number();	// calculates numeric value
 				process_char( LF_NONSPACE|LF_NUMBER,
 							  LF_SPACE_IMMEDIATE|LF_SLASH|LF_DOTDATE|LF_CHECKBOX,
 							  LF_NOTHING, &TextbufferDiary::parsing_triggerer_link_date,
@@ -473,6 +492,18 @@ TextbufferDiary::parse( unsigned int pos_start, unsigned int pos_end )
 							  LF_NOTHING, &TextbufferDiary::parsing_triggerer_link_at,
 							  CC_SIGN );
 				break;
+			//case '<':
+				//process_char( LF_NONSPACE|LF_LESS,
+							  //LF_NUM_SPCIM_CHK|LF_SLASH|LF_DOTDATE,
+							  //LF_NOTHING, &TextbufferDiary::parsing_triggerer_link_hidden,
+							  //CC_SIGN );
+				//break;
+			case '>':
+				process_char( LF_NONSPACE|LF_MORE,
+							  LF_NUM_SPCIM_CHK|LF_SLASH|LF_DOTDATE,
+							  0, NULL,
+							  CC_SIGN );
+				break;
 			case '\t':
 				process_char( LF_TAB|LF_SPACE,
 							  LF_NUM_SPCIM_SLH|LF_NONSPACE|LF_DOTDATE,
@@ -484,13 +515,13 @@ TextbufferDiary::parse( unsigned int pos_start, unsigned int pos_end )
 			case L'☑':
 			case L'☒':
 				process_char( LF_CHECKBOX,
-							  LF_NUMBER|LF_SPACE_IMMEDIATE|LF_SLASH|LF_DOTDATE,
+							  LF_NUM_SPCIM_SLH|LF_DOTDATE,
 							  0, NULL,
 							  CC_SIGN );
 				break;
 			default:
 				process_char( LF_NONSPACE,
-							  LF_NUMBER|LF_DOTDATE|LF_SLASH|LF_CHECKBOX,
+							  LF_NUM_SPCIM_CHK|LF_DOTDATE|LF_SLASH,
 							  0, NULL,
 							  CC_ALPHA );	// most probably :)
 				break;
@@ -553,18 +584,19 @@ void
 TextbufferDiary::parsing_triggerer_link( void )
 {
 	PRINT_DEBUG( "word_last: " + parser_word_last );
+	bool flag_hidden( parser_word_last[ 0 ] == '<' );
+	if( flag_hidden )
+		parser_word_last.erase( 0, 1 );
+
 	if( parser_word_last == "http" || parser_word_last == "https" ||
 	    parser_word_last == "ftp" || parser_word_last == "file" )
 	{
 		parser_lookingfor.clear();
 		parser_lookingfor.push_back( LF_SLASH );
 		parser_lookingfor.push_back( LF_SLASH );
-		// should we really disable relative links?
 		if(	parser_word_last == "file" )
 			parser_lookingfor.push_back( LF_SLASH );
 		parser_lookingfor.push_back( LF_NONSPACE );
-		parser_lookingfor.push_back( LF_SPACE|LF_APPLY_LINK );
-		parser_iter_start = parser_iter_word;
 	}
 	else
 	if( parser_word_last == "mailto" )
@@ -573,9 +605,32 @@ TextbufferDiary::parsing_triggerer_link( void )
 		parser_lookingfor.push_back( LF_NONSPACE );
 		parser_lookingfor.push_back( LF_AT );
 		parser_lookingfor.push_back( LF_NONSPACE );
-		parser_lookingfor.push_back( LF_SPACE|LF_APPLY_LINK );
-		parser_iter_start = parser_iter_word;
 	}
+	else
+	if( parser_word_last == "deid" && flag_hidden )
+	{
+		parser_lookingfor.clear();
+		parser_lookingfor.push_back( LF_NUMBER );
+		parser_lookingfor.push_back( LF_TAB|LF_APPLY_LINK );
+		parser_lookingfor.push_back( LF_NONSPACE );
+		parser_lookingfor.push_back( LF_MORE|LF_APPLY_LINK_ID );
+		parser_iter_start = parser_iter_word;
+		return;
+	}
+	else
+		return;
+
+	if( flag_hidden )
+	{
+		parser_lookingfor.push_back( LF_TAB|LF_APPLY_LINK );
+		parser_lookingfor.push_back( LF_NONSPACE );
+		parser_lookingfor.push_back( LF_MORE|LF_APPLY_LINK );
+	}
+	else
+	{
+		parser_lookingfor.push_back( LF_SPACE|LF_APPLY_LINK );
+	}
+	parser_iter_start = parser_iter_word;
 }
 
 void
@@ -595,9 +650,38 @@ TextbufferDiary::parsing_triggerer_link_at( void )
 void
 TextbufferDiary::parsing_triggerer_link_date( void )
 {
-	parser_int_last = ( parser_char_current - '0' );
-	parser_lookingfor = m_plan_date;
-	parser_iter_start = parser_iter_current;
+	parser_lookingfor.clear();
+	parser_lookingfor.push_back( LF_NUMBER );
+	parser_lookingfor.push_back( LF_NUMBER );
+	parser_lookingfor.push_back( LF_NUMBER );
+	parser_lookingfor.push_back( LF_DOTYM|LF_APPLY_DOTYM );
+	parser_lookingfor.push_back( LF_NUMBER );
+	parser_lookingfor.push_back( LF_NUMBER );
+	parser_lookingfor.push_back( LF_DOTMD|LF_APPLY_DOTMD );
+	parser_lookingfor.push_back( LF_NUMBER );
+	if( parser_word_last == "<" )
+	{
+		parser_lookingfor.push_back( LF_NUMBER );
+		parser_lookingfor.push_back( LF_TAB|LF_APPLY_LINK );
+		parser_lookingfor.push_back( LF_NONSPACE );
+		parser_lookingfor.push_back( LF_MORE|LF_APPLY_LINK_DATE );
+		parser_iter_start = parser_iter_current;
+		parser_iter_start--;
+	}
+	else
+	{
+		parser_lookingfor.push_back( LF_NUMBER|LF_APPLY_LINK_DATE );
+		parser_iter_start = parser_iter_current;
+	}
+
+}
+
+void
+TextbufferDiary::parsing_triggerer_link_hidden_tab( void )
+{
+	PRINT_DEBUG( "parsing_triggerer_link_hidden_tab()" );
+	parser_lookingfor.pop_front();
+	parser_iter_1 = parser_iter_current;
 }
 
 void
@@ -618,12 +702,8 @@ TextbufferDiary::parsing_applier_heading( void )
 
 	if( ! m_flag_settextoperation )
 	{
-		// TODO: what about storing title in entry for faster later use
-		Lifeobase::base->handle_entry_title_changed(
-				m_ptr2entry,
-				( parser_char_current == 0 && parser_iter_current.get_offset() == 0 ) ?
-						_( "<empty entry>" )	:
-						get_slice( parser_iter_start, parser_iter_current ) );
+		m_ptr2entry->calculate_title( get_text() );
+		Lifeobase::base->handle_entry_title_changed( m_ptr2entry );
 	}
 	parser_lookingfor.front() = LF_NOTHING;
 }
@@ -663,24 +743,92 @@ TextbufferDiary::parsing_applier_highlight( void )
 	parser_lookingfor.front() = LF_NOTHING;
 }
 
+void
+TextbufferDiary::apply_hidden_link_tags( Gtk::TextIter &iter_end )
+{
+	apply_tag( m_tag_markup_link, parser_iter_start, parser_iter_1 );
+	apply_tag( m_tag_markup_link, parser_iter_current, iter_end );
+
+	apply_tag( m_tag_link, parser_iter_1, parser_iter_current );
+	apply_tag( m_tag_link_hidden, parser_iter_start, iter_end );
+
+	// hide link markup if cursor is not in it:
+	Gtk::TextIter iter_insert = get_iter_at_mark( get_insert() );
+	if( iter_insert.get_offset() < parser_iter_start.get_offset() ||
+		iter_insert.get_offset() > parser_iter_current.get_offset() )
+	{
+		apply_tag( m_tag_hidden, parser_iter_start, parser_iter_1 );
+		apply_tag( m_tag_hidden, parser_iter_current, iter_end );
+		parser_open_tag_begin = Glib::ustring::npos;
+	}
+	else
+	{
+		parser_open_tag_begin = parser_iter_start.get_offset();
+		parser_open_tag_end = iter_end.get_offset();
+	}
+}
+
 void
 TextbufferDiary::parsing_applier_link( void )
 {
-	PRINT_DEBUG( "url: " + parser_word_last );
+	if( parser_lookingfor.front() & LF_MORE )
+	{
+		Gtk::TextIter iter_end = parser_iter_current;
+		iter_end++;
+		Gtk::TextIter iter_url_start( parser_iter_start );
+		Gtk::TextIter iter_url_end( parser_iter_1 );
+		iter_url_start++;
+		parser_iter_1++;
+		m_list_links.push_back( new LinkUri( create_mark( parser_iter_1 ),
+											 create_mark( parser_iter_current ),
+											 get_slice( iter_url_start, iter_url_end ) ) );
+
+		apply_hidden_link_tags( iter_end );
+	}
+	else
+	{
+		PRINT_DEBUG( "url: " + parser_word_last );
+		m_list_links.push_back( new LinkUri( create_mark( parser_iter_start ),
+											 create_mark( parser_iter_current ),
+											 parser_word_last ) );
+
+		apply_tag( m_tag_link, parser_iter_start, parser_iter_current );
+	}
+
+	parser_lookingfor.front() = LF_NOTHING;
+}
+
+void
+TextbufferDiary::parsing_applier_link_id( void )
+{
+	DiaryElement *element = Diary::d->get_element( parser_int_last );
+
 	Gtk::TextIter iter_end = parser_iter_current;
-	iter_end--;
-	m_list_links.push_back( new LinkUri( create_mark( parser_iter_start ),
-										 create_mark( iter_end ),
-										 parser_word_last ) );
+	iter_end++;
+
+	if( element != NULL )
+	{
+		Gtk::TextIter iter_url_start( parser_iter_start );
+		Gtk::TextIter iter_url_end( parser_iter_1 );
+		iter_url_start++;
+		parser_iter_1++;
+		m_list_links.push_back( new LinkID( create_mark( parser_iter_1 ),
+											create_mark( parser_iter_current ),
+											parser_int_last ) );
+
+		apply_hidden_link_tags( iter_end );
+	}
+	else	// indicate dead links
+	{
+		apply_tag( m_tag_link_broken, parser_iter_start, iter_end );
+	}
 
-	apply_tag( m_tag_link, parser_iter_start, parser_iter_current );
 	parser_lookingfor.front() = LF_NOTHING;
 }
 
 void
 TextbufferDiary::parsing_applier_link_date( void )
 {
-	parsing_applier_number();
 	parser_date_link.set_day( parser_int_last );
 
 	if( parser_date_link.is_valid() )
@@ -697,18 +845,51 @@ TextbufferDiary::parsing_applier_link_date( void )
 
 		if( status < LS_INVALID )
 		{
-			Gtk::TextIter iter_tagend = parser_iter_current;
-			++iter_tagend;
-			m_list_links.push_back(
-					new LinkEntry(	create_mark( parser_iter_start ),
-									create_mark( parser_iter_current ),
-									parser_date_link ) );
+			Gtk::TextIter iter_end = parser_iter_current;
+			iter_end++;
+			// if hidden link:
+			if( parser_char_current == '>' )
+			{
+				Gtk::TextIter iter_url_start( parser_iter_start );
+				Gtk::TextIter iter_url_end( parser_iter_1 );
+				iter_url_start++;
+				parser_iter_1++;
+				m_list_links.push_back( new LinkEntry( create_mark( parser_iter_1 ),
+													   create_mark( parser_iter_current ),
+													   parser_date_link ) );
 
-			if( status == LS_OK )
-				apply_tag( m_tag_link, parser_iter_start, iter_tagend );
+				apply_tag( m_tag_markup_link, parser_iter_start, parser_iter_1 );
+				apply_tag( m_tag_markup_link, parser_iter_current, iter_end );
+
+				apply_tag( status == LS_OK ? m_tag_link :
+							m_tag_link_broken, parser_iter_1, parser_iter_current );
+				apply_tag( m_tag_link_hidden, parser_iter_start, iter_end );
+
+				// hide link markup if cursor is not in it:
+				Gtk::TextIter iter_insert = get_iter_at_mark( get_insert() );
+				if( iter_insert.get_offset() < parser_iter_start.get_offset() ||
+					iter_insert.get_offset() > parser_iter_current.get_offset() )
+				{
+					apply_tag( m_tag_hidden, parser_iter_start, parser_iter_1 );
+					apply_tag( m_tag_hidden, parser_iter_current, iter_end );
+					parser_open_tag_begin = Glib::ustring::npos;
+				}
+				else
+				{
+					parser_open_tag_begin = parser_iter_start.get_offset();
+					parser_open_tag_end = iter_end.get_offset();
+				}
+			}
 			else
-			if( status == LS_ENTRY_UNAVAILABLE )
-				apply_tag(	m_tag_link_broken, parser_iter_start, iter_tagend );
+			{
+				m_list_links.push_back(
+						new LinkEntry( create_mark( parser_iter_start ),
+									   create_mark( iter_end ),
+									   parser_date_link ) );
+
+				apply_tag( status == LS_OK ? m_tag_link :
+								m_tag_link_broken, parser_iter_start, iter_end );
+			}
 		}
 	}
 
@@ -716,7 +897,7 @@ TextbufferDiary::parsing_applier_link_date( void )
 }
 
 void
-TextbufferDiary::parsing_applier_number( void )
+TextbufferDiary::parsing_handler_number( void )
 {
 	if( parser_char_last == CC_NUMBER )
 	{
@@ -725,21 +906,36 @@ TextbufferDiary::parsing_applier_number( void )
 	}
 	else
 		parser_int_last = ( parser_char_current - '0' );
-	parser_lookingfor.pop_front();
 }
 
 void
 TextbufferDiary::parsing_applier_dotym( void )
 {
-	parser_date_link.set_year( parser_int_last );
-	parser_lookingfor.pop_front();
+	if( parser_int_last >= Date::YEAR_MIN && parser_int_last <= Date::YEAR_MAX )
+	{
+		parser_date_link.set_year( parser_int_last );
+		parser_lookingfor.pop_front();
+	}
+	else
+	{
+		parser_lookingfor.clear();
+		parser_lookingfor.push_back( LF_NOTHING );
+	}
 }
 
 void
 TextbufferDiary::parsing_applier_dotmd( void )
 {
-	parser_date_link.set_month( parser_int_last );
-	parser_lookingfor.pop_front();
+	if( parser_int_last >= 1 && parser_int_last <= 12 )
+	{
+		parser_date_link.set_month( parser_int_last );
+		parser_lookingfor.pop_front();
+	}
+	else
+	{
+		parser_lookingfor.clear();
+		parser_lookingfor.push_back( LF_NOTHING );
+	}
 }
 
 void
@@ -747,7 +943,8 @@ TextbufferDiary::parsing_junction_list( void )
 {
 	parser_iter_start = parser_iter_current;
 
-	parser_lookingfor.front() = LF_NONSPACE;
+	parser_lookingfor.front() = LF_SPACE_IMMEDIATE;
+	parser_lookingfor.push_back( LF_NONSPACE );
 
 	switch( parser_char_current )
 	{
@@ -832,6 +1029,8 @@ TextbufferDiary::process_char( unsigned int targeted, unsigned int broken,
 	{
 		parser_lookingfor.clear();
 		parser_lookingfor.push_back( LF_NOTHING );
+		if( triggering & LF_NOTHING )
+			( this->*parsing_triggerer )();
 	}
 	else
 	if( lf & triggering )
@@ -852,18 +1051,21 @@ TextbufferDiary::process_char( unsigned int targeted, unsigned int broken,
 }
 
 inline void
-TextbufferDiary::apply_markup( const Glib::RefPtr<Tag> &tag)
+TextbufferDiary::apply_markup( const Glib::RefPtr< Tag > &tag)
 {
+	// TODO: move to an earlier point
 	if( parser_char_last & CC_SEPARATOR )
 		return;
 
 	Gtk::TextIter iter2 = parser_iter_start;
 	iter2++;
 	apply_tag( m_tag_markup, parser_iter_start, iter2 );
+	apply_tag( m_tag_hidden, parser_iter_start, iter2 );
 	apply_tag( tag, iter2, parser_iter_current );
 	iter2 = parser_iter_current;
 	iter2++;
 	apply_tag( m_tag_markup, parser_iter_current, iter2 );
+	apply_tag( m_tag_hidden, parser_iter_current, iter2 );
 }
 
 
@@ -872,10 +1074,14 @@ TextbufferDiary::set_richtext( Entry *entry )
 {
 	m_flag_settextoperation = true;
 	clear_links();
+	parser_open_tag_begin = Glib::ustring::npos;
+	parser_open_tag_end = Glib::ustring::npos;
+	parser_offset_insert_prev = Glib::ustring::npos;
 	UndoManager::m->clear();
 	UndoManager::m->freeze();
 	m_ptr2entry = entry;
-	set_theme( entry->get_theme() );
+	set_theme( entry->get_theme_is_set() ?
+			entry->get_theme() : Diary::d->get_default_theme() );
 	Gtk::TextBuffer::set_text( entry->get_text() );
 	place_cursor( begin() );
 	UndoManager::m->thaw();
@@ -892,7 +1098,7 @@ TextbufferDiary::reparse( void )
 }
 
 void
-TextbufferDiary::on_insert(	const Gtk::TextIter& iterator,
+TextbufferDiary::on_insert( const Gtk::TextIter& iterator,
 							const Glib::ustring& text,
 							int bytes )
 {
@@ -971,7 +1177,7 @@ TextbufferDiary::process_space( void )
 				lookingfor = ']';
 				break;
 			case ']':
-				if( lookingfor != ']' )
+				if( lookingfor != ']' || i != ( size - 1 ) )
 					return;
 				//lookingfor = 'C';	// checkbox
 				PRINT_DEBUG( "time to insert a checkbox" );
@@ -983,7 +1189,7 @@ TextbufferDiary::process_space( void )
 				insert( iter_start, "☐" );
 				break;
 			case '*':
-				if( lookingfor != 'A' )
+				if( lookingfor != 'A' || i != ( size - 1 ) )
 					return;
 				iter_start = get_iter_at_offset( pos_list );	// get rid of the tab chars
 				m_flag_ongoingoperation = true;
@@ -1125,9 +1331,9 @@ TextbufferDiary::on_erase(	const Gtk::TextIter& iter_start,
 }
 
 void
-TextbufferDiary::on_apply_tag(	const Glib::RefPtr< TextBuffer::Tag >& tag,
-								const Gtk::TextIter& iter_begin,
-								const Gtk::TextIter& iter_end )
+TextbufferDiary::on_apply_tag( const Glib::RefPtr< TextBuffer::Tag >& tag,
+							   const Gtk::TextIter& iter_begin,
+							   const Gtk::TextIter& iter_end )
 {
 	// do not check spelling of links:
 	if( iter_begin.has_tag( m_tag_link ) && tag->property_name() == "gtkspell-misspelled" )
@@ -1148,12 +1354,108 @@ TextbufferDiary::on_remove_tag(	const Glib::RefPtr< TextBuffer::Tag >& tag,
 		Gtk::TextBuffer::on_remove_tag( tag, iter_begin, iter_end );
 }
 
+void
+TextbufferDiary::on_mark_set( const TextBuffer::iterator &iter,
+							  const Glib::RefPtr< TextBuffer::Mark > &mark )
+{
+	Gtk::TextBuffer::on_mark_set( iter, mark );
+
+	if( mark == get_insert() )
+	{
+		PRINT_DEBUG( "on_mark_set()" );
+		bool flag_within_open_tag( false );
+		if( parser_open_tag_begin != Glib::ustring::npos )
+		{
+			if( iter.get_offset() >= int( parser_open_tag_begin ) &&
+				iter.get_offset() <= int( parser_open_tag_end ) )
+				flag_within_open_tag = true;
+			else
+			{
+				if( parser_open_tag_is_link )
+				{
+					PRINT_DEBUG( "OPEN TAG IS LINK" );
+					parse( parser_open_tag_begin, parser_open_tag_end );
+					parser_open_tag_begin = parser_open_tag_end = Glib::ustring::npos;
+				}
+				else
+				{
+					apply_tag( m_tag_hidden,
+							   get_iter_at_offset( parser_open_tag_begin ),
+							   get_iter_at_offset( parser_open_tag_end ) );
+					parser_open_tag_begin = parser_open_tag_end = Glib::ustring::npos;
+				}
+			}
+		}
+		if( ! flag_within_open_tag )	// not within an already open tag
+		{
+			PRINT_DEBUG( "NOT WITHIN OPEN TAG" );
+			if( iter.has_tag( m_tag_link_hidden ) )
+			{
+				PRINT_DEBUG( "WITHIN HIDDEN LINK" );
+				Gtk::TextIter begin( iter );
+				Gtk::TextIter end( iter );
+				if( ! iter.begins_tag( m_tag_link_hidden ) )
+				{
+					PRINT_DEBUG( "BEGINS TAG" );
+					begin.backward_to_tag_toggle( m_tag_link_hidden );
+				}
+				end.forward_to_tag_toggle( m_tag_link_hidden );
+				parser_open_tag_begin = begin.get_offset();
+				parser_open_tag_end = end.get_offset();
+				parser_open_tag_is_link = true;
+				parse( parser_open_tag_begin, parser_open_tag_end );
+			}
+			// STARTING MARKUP
+			else
+			if( iter.begins_tag( m_tag_markup ) )
+			{
+				parser_open_tag_begin = iter.get_offset();
+				parser_open_tag_end = parser_open_tag_begin + 1;
+				parser_open_tag_is_link = false;
+				Gtk::TextIter begin( get_iter_at_offset( parser_open_tag_begin ) );
+				Gtk::TextIter end( get_iter_at_offset( parser_open_tag_end ) );
+				PRINT_DEBUG( "BEGINS MARKUP |" + get_slice(begin, end) + "|" );
+
+				remove_tag( m_tag_hidden, begin, end );
+
+				if( parser_offset_insert_prev != parser_open_tag_begin - 1 )
+				{
+					move_mark( get_selection_bound(), end );
+					move_mark( mark, end );
+				}
+			}
+			// FINISHING MARKUP
+			else
+			if( iter.ends_tag( m_tag_bold ) || iter.ends_tag( m_tag_italic ) ||
+				iter.ends_tag( m_tag_highlight ) || iter.ends_tag( m_tag_strikethrough ) )
+			{
+				parser_open_tag_begin = iter.get_offset();
+				parser_open_tag_end = parser_open_tag_begin + 1;
+				parser_open_tag_is_link = false;
+				Gtk::TextIter begin( get_iter_at_offset( parser_open_tag_begin ) );
+				Gtk::TextIter end( get_iter_at_offset( parser_open_tag_end ) );
+				PRINT_DEBUG( "ENDS MARKUP |" + get_slice(begin, end) + "|" );
+
+				remove_tag( m_tag_hidden, begin, end );
+
+				if( parser_offset_insert_prev == parser_open_tag_begin + 2 )
+				{
+					move_mark( get_selection_bound(), end );
+					move_mark( mark, end );
+				}
+			}
+		}
+
+		parser_offset_insert_prev = iter.get_offset();
+	}
+}
+
 Link*
 TextbufferDiary::get_link( int offset ) const
 {
-	for (	ListLinks::const_iterator iter = m_list_links.begin();
-			iter != m_list_links.end();
-			++iter )
+	for ( ListLinks::const_iterator iter = m_list_links.begin();
+		  iter != m_list_links.end();
+		  ++iter )
 	{
 		if(	offset >= ( *iter )->m_mark_start->get_iter().get_offset() &&
 			offset <= ( *iter )->m_mark_end->get_iter().get_offset() )
@@ -1163,12 +1465,25 @@ TextbufferDiary::get_link( int offset ) const
 	return NULL;
 }
 
+Link*
+TextbufferDiary::get_link( const Gtk::TextIter &iter ) const
+{
+	for ( ListLinks::const_iterator link = m_list_links.begin();
+		  link != m_list_links.end();
+		  ++link )
+	{
+		if( iter.in_range( ( *link )->m_mark_start->get_iter(),
+						   ( *link )->m_mark_end->get_iter() ) )
+			return( *link );
+	}
+	return NULL;
+}
+
 void
 TextbufferDiary::clear_links( int pos_start, int pos_end )
 {
 	ListLinks::iterator iter_tmp;
-	for(	ListLinks::iterator iter = m_list_links.begin();
-			iter != m_list_links.end(); )
+	for( ListLinks::iterator iter = m_list_links.begin(); iter != m_list_links.end(); )
 		if(	pos_start <= ( *iter )->m_mark_start->get_iter().get_offset() &&
 			pos_end >= ( *iter )->m_mark_end->get_iter().get_offset() )
 		{
@@ -1184,9 +1499,9 @@ TextbufferDiary::clear_links( int pos_start, int pos_end )
 void
 TextbufferDiary::clear_links( void )
 {
-	for(	ListLinks::iterator iter = m_list_links.begin();
-			iter != m_list_links.end();
-			++iter )
+	for( ListLinks::iterator iter = m_list_links.begin();
+		 iter != m_list_links.end();
+		 ++iter )
 	{
 		delete( *iter );
 	}
@@ -1196,9 +1511,6 @@ TextbufferDiary::clear_links( void )
 void
 TextbufferDiary::handle_menu( Gtk::Menu *menu )
 {
-	Gtk::MenuItem *menuitem_format = Gtk::manage(
-			new Gtk::MenuItem( _( "Format" ) ) );
-
 	Gtk::CheckMenuItem *menuitem_spell = Gtk::manage(
 			new Gtk::CheckMenuItem( _( "Check Spelling" ) ) );
 
@@ -1210,52 +1522,11 @@ TextbufferDiary::handle_menu( Gtk::Menu *menu )
 			sigc::bind(	sigc::mem_fun( *this, &TextbufferDiary::set_spellcheck ),
 						! Diary::d->get_spellcheck() ) );
 
-	menuitem_format->show();
 	menuitem_spell->show();
 	separator->show();
 
-	// FORMATTING
-	Gtk::Menu *menu_format = new Gtk::Menu;
-	
-	Gtk::MenuItem *menuitem_bold = create_menuitem_markup(
-			Glib::ustring::compose( "*<b>%1</b>*", _( "Bold" ) ),
-			sigc::mem_fun( *this, &TextbufferDiary::toggle_bold ) );
-	Gtk::MenuItem *menuitem_italic = create_menuitem_markup(
-			Glib::ustring::compose( "_<i>%1</i>_", _( "Italic" ) ),
-			sigc::mem_fun( *this, &TextbufferDiary::toggle_italic ) );
-	Gtk::MenuItem *menuitem_strikethrough = create_menuitem_markup(
-			Glib::ustring::compose( "=<s>%1</s>=", _( "Strikeout" ) ),
-			sigc::mem_fun( *this, &TextbufferDiary::toggle_strikethrough ) );
-	Gtk::MenuItem *menuitem_highlight = create_menuitem_markup(
-			Glib::ustring::compose(	"#<span foreground=\"%2\" background=\"%3\">%1</span>#",
-					_( "Highlight" ),
-					convert_gdkcolor_to_html(
-							m_ptr2textview->get_style()->get_text( Gtk::STATE_NORMAL ) ),
-					convert_gdkcolor_to_html( m_tag_highlight->property_background_gdk() ) ),
-			sigc::mem_fun( *this, &TextbufferDiary::toggle_highlight ) );
-
-	Gtk::MenuItem *separator1 = Gtk::manage( new Gtk::SeparatorMenuItem );
-	Gtk::CheckMenuItem *menuitem_hide_markup = Gtk::manage(
-			new Gtk::CheckMenuItem( _( "Hide Markup Characters" ) ) );
-
-	menuitem_hide_markup->set_active( m_tag_markup->property_invisible() );
-
-	menuitem_hide_markup->signal_toggled().connect(
-			sigc::mem_fun( *this, &TextbufferDiary::toggle_hide_markup ) );
-
-	menu_format->append( *menuitem_bold );
-	menu_format->append( *menuitem_italic );
-	menu_format->append( *menuitem_strikethrough );
-	menu_format->append( *menuitem_highlight );
-	menu_format->append( *separator1 );
-	menu_format->append( *menuitem_hide_markup );
-	menu_format->show_all_children();
-
-	menuitem_format->set_submenu( *menu_format );
-
 	menu->prepend( *separator );
 	menu->prepend( *menuitem_spell );
-	menu->prepend( *menuitem_format );
 }
 
 void
@@ -1358,7 +1629,6 @@ TextbufferDiary::toggle_format(	Glib::RefPtr< Tag > tag, const Glib::ustring &ma
 			}
 		}
 	}
-
 }
 
 void
@@ -1385,12 +1655,6 @@ TextbufferDiary::toggle_highlight( void )
 	toggle_format( m_tag_highlight, "#" );
 }
 
-void
-TextbufferDiary::toggle_hide_markup( void )
-{
-	m_tag_markup->property_invisible() = ! m_tag_markup->property_invisible();
-}
-
 void
 TextbufferDiary::handle_indent( void )
 {
@@ -1473,7 +1737,7 @@ TextbufferDiary::add_bullet( void )
 void
 TextbufferDiary::add_checkbox( void )
 {
-		Gtk::TextIter iter = get_iter_at_mark( get_insert() );
+	Gtk::TextIter iter( get_iter_at_mark( get_insert() ) );
 	if( iter.get_line() < 1 )	// no list in the first line
 		return;
 	const Glib::ustring::size_type	pos_cursor = iter.get_offset();
@@ -1518,6 +1782,64 @@ TextbufferDiary::add_checkbox( void )
 	}
 }
 
+void
+TextbufferDiary::add_empty_line_above( void )
+{
+	Gtk::TextIter iter( get_iter_at_mark( get_insert() ) );
+	if( iter.backward_line() )
+		iter.forward_line();
+	insert( iter, "\n" );
+}
+
+void
+TextbufferDiary::remove_empty_line_above( void )
+{
+	Gtk::TextIter iter( get_iter_at_mark( get_insert() ) );
+	if( iter.backward_line() )
+		iter.forward_line();
+
+	if( iter.get_line() < 1 )
+		return;
+
+	Gtk::TextIter iter_begin( --iter );
+	iter_begin--;
+	if( iter_begin.get_char() == '\n' )
+		erase( iter_begin, iter );
+}
+
+void
+TextbufferDiary::insert_link( DiaryElement *element )
+{
+	// TODO: implement a custom insert function that adds spaces where necessary
+	Gtk::TextIter iter( get_iter_at_mark( get_insert() ) );
+	if( iter.get_tags().size() > 0 )
+		return;
+	if( iter.get_offset() > 0 )
+		iter--;
+	char c( iter.get_char() );
+	if( c != ' ' && c != '\n' && c != '\t' )
+		insert( get_iter_at_mark( get_insert() ), " ");
+	insert( get_iter_at_mark( get_insert() ),
+				Glib::ustring::compose( "<deid:%1\t%2>", element->get_id(), element->get_name() ) );
+}
+
+void
+TextbufferDiary::insert_time_stamp( void )
+{
+	// TODO: implement a custom insert function that adds spaces where necessary
+	Gtk::TextIter iter( get_iter_at_mark( get_insert() ) );
+	//if( iter.get_tags().size() > 0 )
+		//return;
+	if( iter.get_offset() > 0 )
+		iter--;
+	char c( iter.get_char() );
+	if( c != ' ' && c != '\n' && c != '\t' )
+		insert( get_iter_at_mark( get_insert() ), " ");
+
+	time_t date( time( NULL ) );
+	insert( get_iter_at_mark( get_insert() ), Date::format_string( &date ) );
+}
+
 void
 TextbufferDiary::set_spellcheck( bool option_spell )
 {
@@ -1537,10 +1859,8 @@ TextbufferDiary::set_spellcheck( bool option_spell )
 }
 
 void
-TextbufferDiary::set_theme( const Theme *theme0 )
+TextbufferDiary::set_theme( const Theme *theme )
 {
-	const Theme *theme = ( bool( theme0 ) ? theme0 : Diary::d->get_default_theme() );
-
 	m_ptr2textview->modify_font( theme->font );
 	m_ptr2textview->modify_base( Gtk::STATE_NORMAL, theme->color_base );
 	m_ptr2textview->modify_text( Gtk::STATE_NORMAL, theme->color_text );
@@ -1552,6 +1872,8 @@ TextbufferDiary::set_theme( const Theme *theme0 )
 			theme->color_base, m_theme_color_match );
 	m_tag_markup->property_foreground_gdk() = midtone(
 			theme->color_base, theme->color_text );
+	m_tag_markup_link->property_foreground_gdk() = midtone(
+			theme->color_base, theme->color_text );
 	m_tag_link->property_foreground_gdk() = contrast(
 			theme->color_base, m_theme_color_link );
 	m_tag_link_broken->property_foreground_gdk() = contrast(
@@ -1563,7 +1885,7 @@ TextbufferDiary::set_theme( const Theme *theme0 )
 TextbufferDiary			*TextviewDiary::m_buffer;
 
 TextviewDiary::TextviewDiary( BaseObjectType *cobject, const Glib::RefPtr<Gtk::Builder>& )
-	:	Gtk::TextView( cobject ),
+:	Gtk::TextView( cobject ),
 	m_cursor_hand( Gdk::HAND2 ), m_cursor_xterm( Gdk::XTERM ),
 	m_ptr2cursor_last( &m_cursor_xterm ), m_link_hovered( NULL )
 {
@@ -1582,15 +1904,25 @@ TextviewDiary::update_link( void )
 	Gtk::TextIter		iter;
 	const Gdk::Cursor	*ptr2cursor = &m_cursor_xterm;
 	int					pointer_x, pointer_y;
-	int					buffer_x, buffer_y;
+	int					trailing, buffer_x, buffer_y;
     Gdk::ModifierType	modifiers;
 
     Gtk::Widget::get_window()->get_pointer( pointer_x, pointer_y, modifiers );
-	window_to_buffer_coords(	Gtk::TEXT_WINDOW_WIDGET,
-								pointer_x, pointer_y,
-								buffer_x, buffer_y );
-	get_iter_at_location( iter, buffer_x, buffer_y );
-	m_link_hovered = m_buffer->get_link( iter.get_offset() );
+	window_to_buffer_coords( Gtk::TEXT_WINDOW_WIDGET,
+							 pointer_x, pointer_y,
+							 buffer_x, buffer_y );
+	get_iter_at_position( iter, trailing, buffer_x, buffer_y );
+
+	// FIX ITER IF NEEDED:
+	if( iter.has_tag( m_buffer->m_tag_hidden ) )
+		iter.forward_to_tag_toggle( m_buffer->m_tag_hidden );
+	Gtk::TextIter iter2( iter );
+	iter2++;
+	if( ( iter2.ends_tag( m_buffer->m_tag_link ) ||
+		  iter2.ends_tag( m_buffer->m_tag_link_broken ) ) && trailing > 0 )
+		iter++;
+
+	m_link_hovered = m_buffer->get_link( iter );
 
 	if( m_link_hovered != NULL )
 	{
@@ -1612,12 +1944,25 @@ TextviewDiary::on_motion_notify_event( GdkEventMotion *event )
 	return Gtk::TextView::on_motion_notify_event( event );
 }
 
+bool
+TextviewDiary::on_button_press_event( GdkEventButton *event )
+{
+	if( m_link_hovered != NULL )
+		if( ( event->state & Gdk::CONTROL_MASK ) != Gdk::CONTROL_MASK )
+			return true;
+
+	return Gtk::TextView::on_button_press_event( event );
+}
+
 bool
 TextviewDiary::on_button_release_event( GdkEventButton *event )
 {
 	if( m_link_hovered != NULL )
 		if( ( event->state & Gdk::CONTROL_MASK ) != Gdk::CONTROL_MASK )
+		{
 			m_link_hovered->go();
+			return true;
+		}
 
 	return Gtk::TextView::on_button_release_event( event );
 }
@@ -1630,33 +1975,52 @@ TextviewDiary::on_key_press_event( GdkEventKey *event )
 		switch( event->keyval )
 		{
 			case GDK_b:
+			case GDK_B:
 				m_buffer->toggle_bold();
 				break;
 			case GDK_i:
+			case GDK_I:
 				m_buffer->toggle_italic();
 				break;
 			case GDK_s:
+			case GDK_S:
 				m_buffer->toggle_strikethrough();
 				break;
 			case GDK_h:
+			case GDK_H:
 				m_buffer->toggle_highlight();
 				break;
 			case GDK_t:
+			case GDK_T:
 				Lifeobase::base->focus_tag();
 				break;
 		}
 	}
 	else
-	if( ( event->state & Gdk::MOD1_MASK ) == Gdk::MOD1_MASK )
+	if( ( event->state & Gdk::MOD1_MASK )/* == Gdk::MOD1_MASK */)
 	{
+		PRINT_DEBUG( "mod1" );
 		switch( event->keyval )
 		{
 			case GDK_i:
+			case GDK_I:
 				m_buffer->handle_indent();
 				break;
 			case GDK_u:
+			case GDK_U:
 				m_buffer->handle_unindent();
 				break;
+			case GDK_l:
+			case GDK_L:
+				if( event->state & Gdk::SHIFT_MASK )
+					m_buffer->remove_empty_line_above();
+				else
+					m_buffer->add_empty_line_above();
+				break;
+			case GDK_t:
+			case GDK_T:
+				m_buffer->insert_time_stamp();
+				break;
 		}
 	}
 	else
