@@ -77,8 +77,8 @@ enum LookingFor
 	LF_APPLY_MARKUP		= 0x7000000,
 	LF_APPLY_HEADING	= 0x9000000,
 	LF_APPLY_SUBHEADING	= 0xB000000,
+	LF_APPLY_LINK_ID	= 0xD000000,
 
-	LF_APPLY_NUMBER		= 0xD000000,
 	LF_APPLY_DOTMD		= 0xF000000,
 	LF_APPLY_DOTYM		= 0x11000000,
 
@@ -256,6 +256,26 @@ class Link
 											const Glib::RefPtr< Gtk::TextMark >& );
 };
 
+class LinkID : public Link
+{
+	public:
+		typedef sigc::signal< void, Date >
+									Signal_void_Date;
+
+									LinkID( const Glib::RefPtr< Gtk::TextMark >&,
+											const Glib::RefPtr< Gtk::TextMark >&,
+											DEID );
+		void						go( void );
+
+		static Signal_void_Date		signal_activated( void )
+		{ return m_signal_activated; }
+
+		DEID						m_id;
+
+	protected:
+		static Signal_void_Date		m_signal_activated;
+};
+
 class LinkEntry : public Link
 {
 	public:
@@ -264,15 +284,14 @@ class LinkEntry : public Link
 
 									LinkEntry(	const Glib::RefPtr< Gtk::TextMark >&,
 												const Glib::RefPtr< Gtk::TextMark >&,
-												Date,
-												unsigned int = 0 );
+												Date );
 		void						go( void );
 
 		static Signal_void_Date		signal_activated( void )
 		{ return m_signal_activated; }
 
 		Date						m_date;
-		unsigned int				m_order;	// a day can have multiple entries
+
 	protected:
 		static Signal_void_Date		m_signal_activated;
 };
@@ -319,6 +338,7 @@ class TextbufferDiary : public Gtk::TextBuffer
 		void						handle_login( void );
 
 		Link*						get_link( int ) const;
+		Link*						get_link( const Gtk::TextIter& ) const;
 
 		void						set_richtext( Entry* );
 		void						reparse( void );
@@ -333,22 +353,20 @@ class TextbufferDiary : public Gtk::TextBuffer
 		void						toggle_italic( void );
 		void						toggle_highlight( void );
 		void						toggle_strikethrough( void );
-		void						toggle_hide_markup( void );
 
 		void						handle_indent( void );
 		void						handle_unindent( void );
 
 		void						add_bullet( void );
 		void						add_checkbox( void );
+		void						add_empty_line_above( void );
+		void						remove_empty_line_above( void );
+		void						insert_link( DiaryElement* );
+		void						insert_time_stamp( void );
 
 		void						set_spellcheck( bool );
 		void						set_theme( const Theme* );
 
-//		Signal_LinkStatus_Date		signal_link_needs_checking( void )
-//		{
-//			return m_signal_link_needs_checking;
-//		}
-
 		void						handle_menu( Gtk::Menu* );
 
 	protected:
@@ -366,6 +384,9 @@ class TextbufferDiary : public Gtk::TextBuffer
 												   const Gtk::TextIter&,
 												   const Gtk::TextIter& );
 
+		void						on_mark_set( const TextBuffer::iterator&,
+												 const Glib::RefPtr< TextBuffer::Mark >& );
+
 		void						process_space( void );
 		bool						process_new_line( void );
 
@@ -379,6 +400,8 @@ class TextbufferDiary : public Gtk::TextBuffer
 
 		void						apply_markup( const Glib::RefPtr<Tag>& );
 
+		void						apply_hidden_link_tags( Gtk::TextIter& );
+
 		void						parsing_triggerer_subheading( void );
 		void						parsing_triggerer_bold( void );
 		void						parsing_triggerer_italic( void );
@@ -387,6 +410,7 @@ class TextbufferDiary : public Gtk::TextBuffer
 		void						parsing_triggerer_link( void );
 		void						parsing_triggerer_link_at( void );
 		void						parsing_triggerer_link_date( void );
+		void						parsing_triggerer_link_hidden_tab( void );
 		void						parsing_triggerer_list( void );
 		void						parsing_junction_list( void );
 
@@ -398,25 +422,30 @@ class TextbufferDiary : public Gtk::TextBuffer
 		void						parsing_applier_highlight( void );
 		void						parsing_applier_link( void );
 		void						parsing_applier_link_date( void );
+		void						parsing_applier_link_id( void );
 		void						parsing_applier_check_can( void );
 		void						parsing_applier_check_unf( void );
 		void						parsing_applier_check_fin( void );
 
-		void						parsing_applier_number( void );
+		void						parsing_handler_number( void );
 		void						parsing_applier_dotym( void );
 		void						parsing_applier_dotmd( void );
 
 		Gtk::TextIter				parser_iter_start;
 		Gtk::TextIter				parser_iter_current;
 		Gtk::TextIter				parser_iter_word;
+		Gtk::TextIter				parser_iter_1;		// helper
 		CharClass					parser_char_last;
 		gunichar					parser_char_current;
 		Glib::ustring				parser_word_last;
 		unsigned int				parser_int_last;
 		Date						parser_date_link;
+		Glib::ustring::size_type	parser_open_tag_begin;
+		Glib::ustring::size_type	parser_open_tag_end;
+		bool						parser_open_tag_is_link;
+		Glib::ustring::size_type	parser_offset_insert_prev;
 		std::deque< unsigned int >	parser_lookingfor;
 
-		std::deque< unsigned int >	m_plan_date;
 		std::deque< unsigned int >	m_plan_subheading;
 		std::deque< unsigned int >	m_plan_bold;
 
@@ -428,16 +457,21 @@ class TextbufferDiary : public Gtk::TextBuffer
 		void						clear_links( void );
 
 		// TAGS
+		// TODO: move these program-wide objects out of this class as...
+		// ...there may be other instances of TextbufferDiary in the future
 		static Glib::RefPtr< Tag >	m_tag_heading;
 		static Glib::RefPtr< Tag >	m_tag_subheading;
 		static Glib::RefPtr< Tag >	m_tag_match;
 		static Glib::RefPtr< Tag >	m_tag_markup;
+		Glib::RefPtr< Tag >			m_tag_markup_link;
+		Glib::RefPtr< Tag >			m_tag_hidden;
 		static Glib::RefPtr< Tag >	m_tag_bold;
 		static Glib::RefPtr< Tag >	m_tag_italic;
 		static Glib::RefPtr< Tag >	m_tag_strikethrough;
 		static Glib::RefPtr< Tag >	m_tag_highlight;
 		static Glib::RefPtr< Tag >	m_tag_link;
 		static Glib::RefPtr< Tag >	m_tag_link_broken;
+		Glib::RefPtr< Tag >			m_tag_link_hidden;
 		static Glib::RefPtr< Tag >	m_tag_checkbox;
 
 		// THEMING
@@ -477,6 +511,7 @@ class TextviewDiary : public Gtk::TextView
 	protected:
 		void						update_link( void );
 		virtual bool				on_motion_notify_event( GdkEventMotion* );
+		virtual bool				on_button_press_event( GdkEventButton* );
 		virtual bool				on_button_release_event( GdkEventButton* );
 		virtual bool				on_key_press_event( GdkEventKey* );
 		virtual bool				on_key_release_event( GdkEventKey* );
